/* ========================================================================
 * Bootstrap: affix.js v3.3.5
 * http://getbootstrap.com/javascript/#affix
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // AFFIX CLASS DEFINITION
  // ======================

  var Affix = function (element, options) {
    this.options = $.extend({}, Affix.DEFAULTS, options)

    this.$target = $(this.options.target)
      .on('scroll.bs.affix.data-api', $.proxy(this.checkPosition, this))
      .on('click.bs.affix.data-api',  $.proxy(this.checkPositionWithEventLoop, this))

    this.$element     = $(element)
    this.affixed      = null
    this.unpin        = null
    this.pinnedOffset = null

    this.checkPosition()
  }

  Affix.VERSION  = '3.3.5'

  Affix.RESET    = 'affix affix-top affix-bottom'

  Affix.DEFAULTS = {
    offset: 0,
    target: window
  }

  Affix.prototype.getState = function (scrollHeight, height, offsetTop, offsetBottom) {
    var scrollTop    = this.$target.scrollTop()
    var position     = this.$element.offset()
    var targetHeight = this.$target.height()

    if (offsetTop != null && this.affixed == 'top') return scrollTop < offsetTop ? 'top' : false

    if (this.affixed == 'bottom') {
      if (offsetTop != null) return (scrollTop + this.unpin <= position.top) ? false : 'bottom'
      return (scrollTop + targetHeight <= scrollHeight - offsetBottom) ? false : 'bottom'
    }

    var initializing   = this.affixed == null
    var colliderTop    = initializing ? scrollTop : position.top
    var colliderHeight = initializing ? targetHeight : height

    if (offsetTop != null && scrollTop <= offsetTop) return 'top'
    if (offsetBottom != null && (colliderTop + colliderHeight >= scrollHeight - offsetBottom)) return 'bottom'

    return false
  }

  Affix.prototype.getPinnedOffset = function () {
    if (this.pinnedOffset) return this.pinnedOffset
    this.$element.removeClass(Affix.RESET).addClass('affix')
    var scrollTop = this.$target.scrollTop()
    var position  = this.$element.offset()
    return (this.pinnedOffset = position.top - scrollTop)
  }

  Affix.prototype.checkPositionWithEventLoop = function () {
    setTimeout($.proxy(this.checkPosition, this), 1)
  }

  Affix.prototype.checkPosition = function () {
    if (!this.$element.is(':visible')) return

    var height       = this.$element.height()
    var offset       = this.options.offset
    var offsetTop    = offset.top
    var offsetBottom = offset.bottom
    var scrollHeight = Math.max($(document).height(), $(document.body).height())

    if (typeof offset != 'object')         offsetBottom = offsetTop = offset
    if (typeof offsetTop == 'function')    offsetTop    = offset.top(this.$element)
    if (typeof offsetBottom == 'function') offsetBottom = offset.bottom(this.$element)

    var affix = this.getState(scrollHeight, height, offsetTop, offsetBottom)

    if (this.affixed != affix) {
      if (this.unpin != null) this.$element.css('top', '')

      var affixType = 'affix' + (affix ? '-' + affix : '')
      var e         = $.Event(affixType + '.bs.affix')

      this.$element.trigger(e)

      if (e.isDefaultPrevented()) return

      this.affixed = affix
      this.unpin = affix == 'bottom' ? this.getPinnedOffset() : null

      this.$element
        .removeClass(Affix.RESET)
        .addClass(affixType)
        .trigger(affixType.replace('affix', 'affixed') + '.bs.affix')
    }

    if (affix == 'bottom') {
      this.$element.offset({
        top: scrollHeight - height - offsetBottom
      })
    }
  }


  // AFFIX PLUGIN DEFINITION
  // =======================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.affix')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.affix', (data = new Affix(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.affix

  $.fn.affix             = Plugin
  $.fn.affix.Constructor = Affix


  // AFFIX NO CONFLICT
  // =================

  $.fn.affix.noConflict = function () {
    $.fn.affix = old
    return this
  }


  // AFFIX DATA-API
  // ==============

  $(window).on('load', function () {
    $('[data-spy="affix"]').each(function () {
      var $spy = $(this)
      var data = $spy.data()

      data.offset = data.offset || {}

      if (data.offsetBottom != null) data.offset.bottom = data.offsetBottom
      if (data.offsetTop    != null) data.offset.top    = data.offsetTop

      Plugin.call($spy, data)
    })
  })

}(jQuery);
;/* ========================================================================
 * Bootstrap: alert.js v3.3.5
 * http://getbootstrap.com/javascript/#alerts
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // ALERT CLASS DEFINITION
  // ======================

  var dismiss = '[data-dismiss="alert"]'
  var Alert   = function (el) {
    $(el).on('click', dismiss, this.close)
  }

  Alert.VERSION = '3.3.5'

  Alert.TRANSITION_DURATION = 150

  Alert.prototype.close = function (e) {
    var $this    = $(this)
    var selector = $this.attr('data-target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    var $parent = $(selector)

    if (e) e.preventDefault()

    if (!$parent.length) {
      $parent = $this.closest('.alert')
    }

    $parent.trigger(e = $.Event('close.bs.alert'))

    if (e.isDefaultPrevented()) return

    $parent.removeClass('in')

    function removeElement() {
      // detach from parent, fire event then clean up data
      $parent.detach().trigger('closed.bs.alert').remove()
    }

    $.support.transition && $parent.hasClass('fade') ?
      $parent
        .one('bsTransitionEnd', removeElement)
        .emulateTransitionEnd(Alert.TRANSITION_DURATION) :
      removeElement()
  }


  // ALERT PLUGIN DEFINITION
  // =======================

  function Plugin(option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.alert')

      if (!data) $this.data('bs.alert', (data = new Alert(this)))
      if (typeof option == 'string') data[option].call($this)
    })
  }

  var old = $.fn.alert

  $.fn.alert             = Plugin
  $.fn.alert.Constructor = Alert


  // ALERT NO CONFLICT
  // =================

  $.fn.alert.noConflict = function () {
    $.fn.alert = old
    return this
  }


  // ALERT DATA-API
  // ==============

  $(document).on('click.bs.alert.data-api', dismiss, Alert.prototype.close)

}(jQuery);
;var alertbox = null;
var alertMgr = {
    alerts:[],
    alertElements:[],
    previous: null,
    count: 0,
    wrapper: null,
    currentAlert: null,
};

+function ($){
    'use strict';

    alertbox = function(type, msg, conf){

        var config = conf,
            id = 0;

        //handle type
        if(!typeof(type) === 'string'){
            //console.log("type invalid -- casting type to 'success'");
            type = "success";
        }else{
            if(type!='success'&& type != 'error' && type != 'warning' && type != 'info'){
                //console.log("alert invalid -- casting type to 'success");
                type = "success";
            }
        }

        //handle id
        var i;
        for (i=0; i<Number.MAX_VALUE; i++){
            if(alertMgr.alerts[i] == null){
                id = i;
                break;
            }
        }

        //single object of alert
        var alertobj = {
            type: type,
            config: config,
            message: msg,
            id: id,
            next: null,
        };

        //add functionalities to alertMgr
        alertMgr.clearAll = clearAll;

        //put alert into collection
        alertMgr.alerts[id] = alertobj;
        alertMgr.count++;

        init(alertobj);

        return alertobj;

        /*
        INITIALIZE ALERT OBJECT
         */
        function init(alertobj){
            //handle config inheritance
            var i;
            var alert_config = getDefaultConfig();
            for (i in alertobj.config){
                alert_config[i] = alertobj.config[i];
            }
            alertobj.config = alert_config;

            //check duplicate
            if(alert_config.preventDuplicates){
                if(alertobj.message === alertMgr.previous){
                    alertMgr.alerts[alertobj.id] = null;
                    alertMgr.count--;
                    return;
                }else{
                    alertMgr.previous = alertobj.message;
                }
            }

            //creating DOM element
            var alertElement = document.createElement('div')
            var messageElement = document.createElement('div');
            var confirmElement = document.createElement('BUTTON');

            //create a wrapper if none exists
            if(document.getElementById("alertbox-wrapper")){
                alertMgr.wrapper = document.getElementById("alertbox-wrapper");

            }
            if(!alertMgr.wrapper){
                alertMgr.wrapper = createWrapper(alert_config);
                var largeElement = document.createElement('div');
                largeElement.style.position = 'absolute';
                largeElement.style.x = 0;
                largeElement.style.y = 0;
                largeElement.style.width = $(window).width()+"px";
                largeElement.style.height = $(window).height()+"px";
                largeElement.className = "largeElement";
                largeElement.style.zIndex = "99999998";
                $(alertMgr.wrapper).append(largeElement);
            }

            //put alertElement into collection
            alertMgr.alertElements[alertobj.id] = alertElement;

            //set alertbox default to be hidden
            alertElement.style.visibility = 'hidden';

            //set alertbox zindex to make sure it is on top of everything
            alertElement.style.zIndex = "99999999";

            //choosing classname by type
            var type_class;
            if(alertobj.type.toUpperCase()=='INFO'){
                type_class = alert_config.typeClasses.info;
            }else if(alertobj.type.toUpperCase()=='ERROR'){
                type_class = alert_config.typeClasses.error;
            }else if(alertobj.type.toUpperCase() == 'WARNING'){
                type_class = alert_config.typeClasses.warning;
            }else{//if success
                type_class = alert_config.typeClasses.success;
            }

            //applying class name
            alertElement.className = alert_config.alertBoxClass + " " + type_class;

            //applying tap to dismiss
            if(alertobj.config.clickToDismiss){
                alertElement.onclick = function(){
                    hideAlert(alertElement, alertobj);
                };
            }

            //applying on click
            if(alertobj.config.onClick){
                alertElement.onclick = function(event){
                    alertobj.config.onClick(event);
                }
            }

            //applying alert element
            if(!alertMgr.currentAlert) { // if there is no alert displaying
                $(alertMgr.wrapper).append(alertElement);
                alertMgr.currentAlert = alertobj;
            }else{ //if there is an alert displaying, store the element and display later
                if(alertMgr.currentAlert.next) {
                    var latestAlert = alertMgr.currentAlert.next;
                    while (latestAlert.next) {
                        latestAlert = latestAlert.next;
                    }
                    latestAlert.next = alertobj;
                }else{
                    alertMgr.currentAlert.next = alertobj;
                }
            }

            //applying title
            if(alertobj.config.title){
                var titleElement = document.createElement('h4');
                titleElement.className = alertobj.config.titleClass;
                titleElement.innerHTML = alertobj.config.title;
                $(alertElement).append(titleElement);
            }

            //close button
            if(alertobj.config.closeButton){
                var closeElement;
                closeElement  = document.createElement('BUTTON');
                closeElement.onclick = function(){
                    hideAlert(alertElement, alertobj);
                }
                closeElement.innerHTML = "x";
                closeElement.className = alertobj.config.closeButtonClass;
                $(alertElement).append(closeElement);
            }

            //applying message
            messageElement.innerHTML = alertobj.message;
            messageElement.className = alertobj.config.messageClass;
            $(alertElement).append(messageElement);

            //preventAdditional option check box
            if(alertobj.config.preventAdditionalOption && alertMgr.count > 1){
                var paoElement = document.createElement('input');
                paoElement.type = 'checkbox';
                paoElement.name = 'pao';
                paoElement.id = alertobj.config.alertBoxClass+"-"+alertobj.config.paoTextClass+"-"+alertobj.id;
                $(alertElement).append(paoElement);

                var paoTextElement = document.createElement('label');
                paoTextElement.htmlFor = alertobj.config.alertBoxClass+"-"+alertobj.config.paoTextClass+"-"+alertobj.id;
                $(paoTextElement).append(document.createTextNode(alertobj.config.paoText));
                $(alertElement).append(paoTextElement);
            }

            //applying confirm element
            confirmElement.innerHTML = alertobj.config.confirmText;
            confirmElement.className = alertobj.config.confirmClass;
            confirmElement.onclick = alertobj.config.onConfirm;
            confirmElement.addEventListener('click', function(){
                hideAlert(alertElement, alertobj);
            })
            $(alertElement).append(confirmElement);


            //display the alertbox
            display(alertElement, alertobj)
        }



    }

    function createWrapper(config) {
        alertMgr.wrapper = document.createElement('div');
        alertMgr.wrapper.id = config.wrapperId;
        alertMgr.wrapper.className = config.positionClass;

        var target;
        if(config.target.charAt(0)=='#'){
            target = document.getElementById(config.target.substring(1));
        }else{
            target = document.getElementsByTagName(config.target)[0];
        }
        $(target).append(alertMgr.wrapper);

        if(config.topPadding){
            alertMgr.wrapper.style.top = config.topPadding + 'px';
        }

        return alertMgr.wrapper;
    }

    function getDefaultConfig(){
        return {
            alertBoxClass: 'alertbox',
            wrapperId: 'alertbox-wrapper',
            messageClass: 'alertbox-message',
            target: 'body',
            topPadding: 5,
            positionClass: 'alertbox-top-center',
            //width: ,  the width of the alertbox

            closeButton: true,
            closeButtonClass: "alertbox-close-button",
            clickToDismiss: false,
            preventAdditionalOption: true, //check box to enable prevent additional alertbox
            paoTextClass: 'paoText',
            paoText: 'prevent additional alerts',

            confirmText: 'ok',
            confirmClass: 'alertbox-confirm',
            onConfirm: undefined,

            typeClasses: {
                error: 'alertbox-error',
                info: 'alertbox-info',
                success: 'alertbox-success',
                warning: 'alertbox-warning'
            },

            title: "alert from the page: ",
            titleClass: "alertbox-title",
        }
    }

    function display(alertElement, alertobj){
        //show the alert
        alertElement.style.visibility = "visible";
    }

    function hideAlert(element, alertobj){
        //if prevent additional alerts
        if(alertobj.config.preventAdditionalOption && $("#"+alertobj.config.alertBoxClass+"-"+alertobj.config.paoTextClass+"-"+alertobj.id).is(":checked")){
            clearAll();

            return;
        }
        //if there is next alert in queue, display it
        if(alertobj.next){
            $(alertMgr.wrapper).append(alertMgr.alertElements[alertobj.next.id]);
            alertMgr.currentAlert = alertobj.next;
            alertMgr.alertElements[alertobj.next.id].style.display = 'visible';
        }else{
            alertMgr.currentAlert = null;
        }
        element.remove();
        alertMgr.count--;
        alertMgr.alerts[alertobj.id] = null;
        alertMgr.alertElements[alertobj.id] = null;
        if(alertMgr.count == 0){
            alertMgr.wrapper.remove();
            alertMgr.wrapper = null;
            alertMgr.previous = null;
        }
    }

    /*
    FUNCTIONALITIES FOR ALERTMGR USE
     */
    //remove all alerts
    function clearAll(){
        var i;
        for (i in alertMgr.alerts){
            alertMgr.alerts[i] = null;
        }
        for( i in alertMgr.alertElements){
            alertMgr.alertElements[i] = null;
        }
        alertMgr.count = 0;
        alertMgr.currentAlert = null;
        alertMgr.wrapper.remove();
        alertMgr.wrapper = null;
        alertMgr.previous = null;
    }
}(jQuery)
;/* ========================================================================
 * Bootstrap: button.js v3.3.5
 * http://getbootstrap.com/javascript/#buttons
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // BUTTON PUBLIC CLASS DEFINITION
  // ==============================

  var Button = function (element, options) {
    this.$element  = $(element)
    this.options   = $.extend({}, Button.DEFAULTS, options)
    this.isLoading = false
  }

  Button.VERSION  = '3.3.5'

  Button.DEFAULTS = {
    loadingText: 'loading...'
  }

  Button.prototype.setState = function (state) {
    var d    = 'disabled'
    var $el  = this.$element
    var val  = $el.is('input') ? 'val' : 'html'
    var data = $el.data()

    state += 'Text'

    if (data.resetText == null) $el.data('resetText', $el[val]())

    // push to event loop to allow forms to submit
    setTimeout($.proxy(function () {
      $el[val](data[state] == null ? this.options[state] : data[state])

      if (state == 'loadingText') {
        this.isLoading = true
        $el.addClass(d).attr(d, d)
      } else if (this.isLoading) {
        this.isLoading = false
        $el.removeClass(d).removeAttr(d)
      }
    }, this), 0)
  }

  Button.prototype.toggle = function () {
    var changed = true
    var $parent = this.$element.closest('[data-toggle="buttons"]')

    if ($parent.length) {
      var $input = this.$element.find('input')
      if ($input.prop('type') == 'radio') {
        if ($input.prop('checked')) changed = false
        $parent.find('.active').removeClass('active')
        this.$element.addClass('active')
      } else if ($input.prop('type') == 'checkbox') {
        if (($input.prop('checked')) !== this.$element.hasClass('active')) changed = false
        this.$element.toggleClass('active')
      }
      $input.prop('checked', this.$element.hasClass('active'))
      if (changed) $input.trigger('change')
    } else {
      this.$element.attr('aria-pressed', !this.$element.hasClass('active'))
      this.$element.toggleClass('active')
    }
  }


  // BUTTON PLUGIN DEFINITION
  // ========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.button')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.button', (data = new Button(this, options)))

      if (option == 'toggle') data.toggle()
      else if (option) data.setState(option)
    })
  }

  var old = $.fn.button

  $.fn.button             = Plugin
  $.fn.button.Constructor = Button


  // BUTTON NO CONFLICT
  // ==================

  $.fn.button.noConflict = function () {
    $.fn.button = old
    return this
  }


  // BUTTON DATA-API
  // ===============

  $(document)
    .on('click.bs.button.data-api', '[data-toggle^="button"]', function (e) {
      var $btn = $(e.target)
      if (!$btn.hasClass('btn')) $btn = $btn.closest('.btn')
      Plugin.call($btn, 'toggle')
      if (!($(e.target).is('input[type="radio"]') || $(e.target).is('input[type="checkbox"]'))) e.preventDefault()
    })
    .on('focus.bs.button.data-api blur.bs.button.data-api', '[data-toggle^="button"]', function (e) {
      $(e.target).closest('.btn').toggleClass('focus', /^focus(in)?$/.test(e.type))
    })

}(jQuery);
;/************BEGIN date-utilities**************************************
 * Utilities to help working with dates easier
 * Specifically made for caCalendar
 *
 * Author
 * shikhar sharma (shash27@ca.com)
 **********************************************************************/

// this is the exposed variable which has all the utility functions on it
var dateUtilities = {};


// closure
(function () {


    /*
     * Format a JavaScript Date object according to the specified format
     *
     * Example:
     * (new Date()).format("YYYY/MM/DD hh:mm:ss A");
     * "2016/06/09 11:56:08 AM"
     *
     *
     * Currently it supports :
     * M	      1   2 ... 11  12
     * MM	     01  02 ... 11  12
     * MMM	    Jan Feb ... Nov Dec
     * MMMM 	January February ... November December
     *
     * D	      1   2 ...  30  31
     * DD   	 01  02 ...  30  31
     * DDD	      1   2 ... 364 365
     * DDDD    	001 002 ... 364 365
     *
     * d    	0  1 ... 5  6
     * dd	    Su     Mo ...     Fr     Sa
     * ddd	    Sun    Mon ...    Fri    Sat
     * dddd	    Sunday Monday ... Friday Saturday
     *
     * w    	 1  2 ... 52 53
     * ww	    01 02 ... 52 53
     *
     * YY	      70   71 ...   29   30
     * YYYY 	1970 1971 ... 2029 2030
     *
     * A    	AM PM
     * a	    am pm

     * H	     0  1 ... 22 23
     * HH	    00 01 ... 22 23
     * h	     1  2 ... 11 12
     * hh	    01 02 ... 11 12
     *
     * m         0  1 ... 58 59
     * mm       00 01 ... 58 59
     *
     * s	     0  1 ... 58 59
     * ss       00 01 ... 58 59
     *
     * X    	1465493739
     *
     */
    Date.prototype.format = function (formatStr) {
        var helper = {
            /*
             * Makes str's length finalLength by padding zeroes on the left
             * Example 1: padWithZeroes("3", 2);
             * "03"
             * Example 2: padWithZeroes("23", 3);
             * "023"
             */
            padWithZeroes: function (str, finalLength) {
                var numZeroes = finalLength - str.length;
                var zeroStr = "";
                for (var i = 0; i < numZeroes; i++)
                    zeroStr += "0";
                return zeroStr + str;
            },

            /*
             * Main code logic resides here
             * returns string corresponding to the format specifier
             * Example
             * formatToFunctionMapper['MMMM']();
             * "June"
             */
            formatToFunctionMapper: {

                M: function () {
                    return (date.getMonth() + 1).toString();
                },

                MM: function () {
                    var m = helper.formatToFunctionMapper['M']();
                    return helper.padWithZeroes(m, 2);
                },

                MMM: function () {
                    var months = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
                    return months[date.getMonth()];
                },

                MMMM: function () {
                    var months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August',
                        'September', 'October', 'November', 'December'];
                    return months[date.getMonth()];
                },


                D: function () {
                    return date.getDate().toString();
                },

                DD: function () {
                    var d = helper.formatToFunctionMapper['D']();
                    return helper.padWithZeroes(d, 2);
                },

                DDD: function () {
                    var t_start_of_year = (new Date(date.getFullYear(), 0, 1)).getTime();
                    var t_current = (new Date(date.getFullYear(), date.getMonth(), date.getDate(), 12)).getTime();
                    var t_diff = t_current - t_start_of_year;
                    return Math.ceil(t_diff / (24 * 60 * 60 * 1000)).toString();
                },

                DDDD: function () {
                    var d = helper.formatToFunctionMapper['DDD']();
                    return helper.padWithZeroes(d, 3);
                },


                d: function () {
                    return date.getDay().toString();
                },

                dd: function () {
                    var days = ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'];
                    return days[date.getDay()];
                },

                ddd: function () {
                    var days = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
                    return days[date.getDay()];
                },

                dddd: function () {
                    var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday'];
                    return days[date.getDay()];
                },


                w: function () {
                    var d = parseInt(helper.formatToFunctionMapper['DDD']());
                    return Math.ceil(d / 7).toString();
                },

                ww: function () {
                    var w = helper.formatToFunctionMapper['w']();
                    return helper.padWithZeroes(w, 2);
                },


                YY: function () {
                    var y = date.getFullYear();
                    y = (y < 2000) ? y - 1900 : y - 2000;
                    var yStr = y.toString();
                    return helper.padWithZeroes(yStr, 2);
                },

                YYYY: function () {
                    return date.getFullYear().toString();
                },


                A: function () {
                    var hr = date.getHours();
                    return (hr < 12) ? "AM" : "PM";
                },

                a: function () {
                    return helper.formatToFunctionMapper['A']().toLowerCase();
                },


                H: function () {
                    return date.getHours();
                },

                HH: function () {
                    var h = helper.formatToFunctionMapper['H']();
                    return helper.padWithZeroes(h, 2);
                },


                h: function () {
                    var h = date.getHours() % 12;
                    return h ? h.toString() : (12).toString();
                },

                hh: function () {
                    var h = helper.formatToFunctionMapper['h']();
                    return helper.padWithZeroes(h, 2);
                },


                m: function () {
                    return date.getMinutes().toString();
                },

                mm: function () {
                    var m = helper.formatToFunctionMapper['m']();
                    return helper.padWithZeroes(m, 2);
                },


                s: function () {
                    return date.getSeconds().toString();
                },

                ss: function () {
                    var s = helper.formatToFunctionMapper['s']();
                    return helper.padWithZeroes(s, 2);
                },


                X: function () {
                    return Math.ceil(date.getTime() / 1000);
                }
            },

            // do we support this format
            hasFormat: function (format) {
                return helper.formatToFunctionMapper.hasOwnProperty(format);
            },


            /*
             * Takes in an array containing format specifiers, and
             * returns a string which contains the date formatted according to the given formatArr
             * Example
             * format(['DD', '-', 'MM', '-', 'YYYY', ' ', 'hh', ':', 'mm', 'A']);
             * "21-06-2016 3:20PM"
             */
            format: function (formatArr) {
                var str = "";

                for (var i = 0; i < formatArr.length; i++) {

                    var format = formatArr[i];

                    // if this is a supported format-specifier
                    if (helper.hasFormat(format))
                        str += helper.formatToFunctionMapper[format]();
                    else
                        str += format;
                }
                return str;
            }
        };

        /*
         * this is where the execution starts
         */
        var date = this;
        var formatArr = getFormatArray(formatStr, false);
        return helper.format(formatArr);
    }




    /*
     * Parse a date string according to specified format and
     * return a Date object
     *
     * Example:
     * dateUtilities.getDateFromStr("7:12AM, Dec 23, 1990", "hh:mmA, MMM DD, YYYY");
     * returns a Date object with everything set up according to date string
     * (Sun Dec 23 1990 07:12:00 GMT-0500 (EST))
     *
     * Note: Returns null if parsing fails
     *
     * Currently it supports :
     * MM	     01  02 ... 11  12
     * MMM	    Jan Feb ... Nov Dec
     *
     * DD   	 01  02 ...  30  31
     *
     * YY	      70   71 ...   29   30
     * YYYY 	1970 1971 ... 2029 2030
     *
     * A    	AM PM
     *
     * HH	    00 01 ... 22 23
     * hh	    01 02 ... 11 12
     *
     * mm       00 01 ... 58 59
     *
     * ss       00 01 ... 58 59
     *
     * X    	1465493739
     *
     */
    dateUtilities.getDateFromStr = function (dateStr, formatStr) {
        var helper = {

            /*
             * Main code logic resides here
             * sets the right properties on the Date object according to the format specifier
             * returns true if successful
             *
             * Example
             * var d = new Date();
             * var str = "12"
             * formatToFunctionMapper['DD'](str, d);
             *
             * Returns true
             * and now d's date is set to 12
             *
             */
            formatToFunctionMapper: {
                MM: function (monthStr, date) {
                    var month = parseInt(monthStr);
                    if (isNaN(month) || month < 1 || month > 12)
                        return false;
                    date.setMonth(month - 1); // JavaScript months begin with 0
                    return true;
                },

                MMM: function (monthStr, date) {
                    monthStr = monthStr.toLowerCase();
                    monthStr = monthStr.trim();
                    var months = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'];
                    if (months.indexOf(monthStr) == -1)
                        return false;
                    date.setMonth(months.indexOf(monthStr));
                    return true;
                },

                DD: function (dateStr, date) {
                    var dateNum = parseInt(dateStr);
                    if (isNaN(dateNum) || dateNum < 1 || dateNum > 31)
                        return false;
                    date.setDate(dateNum);
                    return true;
                },

                YY: function (yearStr, date) {
                    var year = parseInt(yearStr);
                    if (isNaN(year) || year < 0 || year > 99)
                        return false;
                    // this is hard-coded stuff
                    // if year near the current year (within 20 years in future)
                    // we consider it 20**
                    // otherwise, we consider it 19**
                    var currentYearInTwoDigits = (new Date()).getFullYear() - 2000;
                    HOW_FAR_IN_FUTURE = 20;
                    year = (year > currentYearInTwoDigits + HOW_FAR_IN_FUTURE) ? year + 1900 : year + 2000;

                    date.setYear(year);
                    return true;
                },

                YYYY: function (yearStr, date) {
                    var year = parseInt(yearStr);
                    if (isNaN(year) || year < 0)
                        return false;
                    date.setYear(year);
                    return true;
                },

                A: function (meridiem, date) {
                    meridiem = meridiem.trim();
                    meridiem = meridiem.toLowerCase();
                    var meridiems = ['am', 'pm'];
                    if (meridiems.indexOf(meridiem) == -1)
                        return false;
                    if (meridiem == "pm")
                        date.setHours(date.getHours() + 12);

                    return true;
                },
                HH: function (hoursStr, date) {
                    var hours = parseInt(hoursStr);
                    if (isNaN(hours) || hours < 0 || hours > 23)
                        return false;
                    date.setHours(hours);
                    return true;
                },
                hh: function (hoursStr, date) {
                    var hours = parseInt(hoursStr);
                    if (isNaN(hours) || hours < 1 || hours > 12)
                        return false;
                    hours %= 12;
                    date.setHours(hours);
                    return true;
                },
                mm: function (minutesStr, date) {
                    var minutes = parseInt(minutesStr);
                    if (isNaN(minutes) || minutes < 0 || minutes > 59)
                        return false;
                    date.setMinutes(minutes);
                    return true;
                },

                ss: function (secondsStr, date) {
                    var seconds = parseInt(secondsStr);
                    if (isNaN(seconds) || seconds < 0 || seconds > 59)
                        return false;
                    date.setSeconds(seconds);
                    return true;
                },
                X: function (timeStr, date) {
                    var time = parseInt(timeStr);
                    if (isNaN(time))
                        return false;
                    date.setTime(time);
                    return true;
                }
            },

            // do we support this format
            hasFormat: function (format) {
                return helper.formatToFunctionMapper.hasOwnProperty(format);
            },

            /*
             * Takes in a date-time string & an array containing format specifiers, and
             * tries to parse the date-time string according to the given formatArr
             * Returns the Date object if successful
             * returns null otherwise
             *
             * Example
             * var dateStr = "7:12AM, Dec 23, 1990";
             * var formatArr = ['hh', ':', 'mm', 'A', ',', 'MMM', 'DD', ',', 'YYYY'];
             * getDateFromFormat(dateStr, formatArr);
             *
             * Returns a Date object set up according to the date-time string
             * Sun Dec 23 1990 07:12:00 GMT-0500 (EST)
             */
            getDateFromFormat: function (dateStr, formatArr) {
                dateStr = dateStr.trim();

                // this is the date on which all time-related properties are set
                // during parsing
                var date = new Date(0, 0, 1, 0, 0, 0);

                // this is used to maintain state
                // while parsing
                var strObj = {
                    str: dateStr,
                    index: 0
                };

                for (var i = 0; i < formatArr.length; i++) {
                    var format = formatArr[i];

                    // if this is a supported format-specifier
                    if (helper.hasFormat(format)) {

                        // get substring until next delimiter
                        var delimiterIndex = getDelimiterIndex(strObj);
                        var substr = strObj.str.substring(strObj.index, delimiterIndex);
                        strObj.index = delimiterIndex;

                        var isParseSuccessful = helper.formatToFunctionMapper[format](substr, date);

                        // if not valid
                        if (!isParseSuccessful)
                            return null;
                    }
                    else {
                        strObj.index += format.length;
                    }
                }

                // the whole string was not parsed
                if (strObj.index != dateStr.length)
                    return null;

                return date;
            }
        };

        /*
         * this is where the execution starts
         */
        var formatArr = getFormatArray(formatStr, true);
        return helper.getDateFromFormat(dateStr, formatArr);
    }



    /*
     * Parse a date string according to specified format and
     * return a string with the right delimiter added at the end if possible
     * with a bunch of other helpful properties like :
     *         valid     => whether the string is valid
     *           str     => original string with delimiters added at end if possible
     * parseComplete     => is the string completely parsed
     * moreCharsPossible => whether there is any room for extra characters for any component of the dateStr
     *
     *
     * Example:
     * dateUtilities.decorateDateWithFormat("12/4", "MM/DD/YYYY");
     * {
     *     valid : true,
     *     str : "12/4/",
     *     parseComplete : false,
     *     moreCharsPossible : false
     * }
     *
     * dateUtilities.decorateDateWithFormat("1/4", "MM/DD/YYYY");
     * {
     *     valid : true,
     *     str : "1/4/",
     *     parseComplete : false,
     *     moreCharsPossible : true
     * }
     * // moreCharsPossible is true since the first character '1' has more room;
     * // there could be a '2' after it, if the user edits it, thus making it '12'
     *
     *
     * dateUtilities.decorateDateWithFormat("12/4/1990", "MM/DD/YYYY");
     * {
     *     valid : true,
     *     str : "12/4/1990",
     *     parseComplete : true,
     *     moreCharsPossible : false
     * }
     *
     * dateUtilities.decorateDateWithFormat("14/4/1990", "MM/DD/YYYY");
     * {
     *     valid : false,
     *     str : "14/4/1990",
     *     parseComplete : false,
     *     moreCharsPossible : false
     * }
     *
     * Currently it supports :
     * MM	     01  02 ... 11  12
     * MMM	    Jan Feb ... Nov Dec
     *
     * DD   	 01  02 ...  30  31
     *
     * YY	      70   71 ...   29   30
     * YYYY 	1970 1971 ... 2029 2030
     *
     * A    	AM PM
     *
     * HH	    00 01 ... 22 23
     * hh	    01 02 ... 11 12
     *
     * mm       00 01 ... 58 59
     *
     * ss       00 01 ... 58 59
     *
     * X    	1465493739
     *
     */
    dateUtilities.decorateDateWithFormat = function (dateStr, formatStr) {
        var helper = {

            SubstrParseObj: function () {
                this.valid = false;
                this.moreCharsPossible = true;
            },


            /*
             * Main code logic resides here
             * tries to parse the string according to the format specifier
             * and sets properties like valid and moreCharsPossible on substrParseObj
             *
             * Example
             *
             * var str = "2";
             * formatToFunctionMapper['DD'](str, d);
             * {
             *     valid : true,
             *     moreCharsPossible : true
             * }
             *
             * var str = "5";
             * formatToFunctionMapper['DD'](str, d);
             * {
             *     valid : true,
             *     moreCharsPossible : false
             * }
             *
             * var str = "34";
             * formatToFunctionMapper['DD'](str, d);
             * {
             *     valid : false,
             *     moreCharsPossible : true
             * }
             *
             */

            formatToFunctionMapper: {
                MM: function (monthNumStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var month = parseInt(monthNumStr);

                    // invalid number for month
                    if (isNaN(month) || month < 1 || month > 12 || monthNumStr.length > 2)
                        return substrParseObj;

                    // month is complete, no more chars possible
                    if (monthNumStr.length == 2 || month > 1)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                MMM: function (monthStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var monthStr = monthStr.toLowerCase();
                    monthStr = monthStr.trim();
                    var months = ['jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec'];

                    // invalid month string
                    if (monthStr.length == 3 && months.indexOf(monthStr) == -1)
                        return substrParseObj;

                    // month is complete, no more chars possible
                    if (monthStr.length == 3)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                DD: function (dateNumStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var dateNum = parseInt(dateNumStr);

                    // invalid number for date
                    if (isNaN(dateNum) || dateNum < 1 || dateNum > 31 || dateNumStr.length > 2)
                        return substrParseObj;

                    // date is complete, no more chars possible
                    if (dateNumStr.length == 2 || dateNum > 3)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                YY: function (yearStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var year = parseInt(yearStr);

                    // invalid number for year
                    if (isNaN(year) || year < 0 || year > 99 || yearStr.length > 2)
                        return substrParseObj;

                    // year is complete, no more chars possible
                    if (yearStr.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                YYYY: function (yearStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var year = parseInt(yearStr);

                    // invalid number for year
                    if (isNaN(year) || year < 0 || yearStr.length > 4)
                        return substrParseObj;

                    // year is complete, no more chars possible
                    if (yearStr.length == 4)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                A: function (meridiem) {

                    var substrParseObj = new helper.SubstrParseObj();
                    meridiem = meridiem.trim();
                    meridiem = meridiem.toLowerCase();
                    var meridiems = ['am', 'pm'];

                    // invalid meridiem
                    if (meridiem.length == 2 && meridiems.indexOf(meridiem) == -1)
                        return substrParseObj;

                    // meridiem is complete, no more chars possible
                    if (meridiem.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                HH: function (hoursStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var hours = parseInt(hoursStr);

                    // invalid number for hours
                    if (isNaN(hours) || hours < 0 || hours > 23)
                        return substrParseObj;

                    // 'hours' is complete, no more chars possible
                    if (hours > 2 || hoursStr.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                hh: function (hoursStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var hours = parseInt(hoursStr);

                    // invalid number for hours
                    if (isNaN(hours) || hours < 1 || hours > 12)
                        return substrParseObj;

                    // 'hours' is complete, no more chars possible
                    if (hours > 1 || hoursStr.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                mm: function (minutesStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var minutes = parseInt(minutesStr);

                    // invalid number for minutes
                    if (isNaN(minutes) || minutes < 0 || minutes > 59)
                        return substrParseObj;

                    // 'minutes' is complete, no more chars possible
                    if (minutes > 5 || minutesStr.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                ss: function (secondsStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var seconds = parseInt(secondsStr);

                    // invalid number for seconds
                    if (isNaN(seconds) || seconds < 0 || seconds > 59)
                        return substrParseObj;

                    // 'seconds' is complete, no more chars possible
                    if (seconds > 5 || secondsStr.length == 2)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                },

                X: function (timeStr) {

                    var substrParseObj = new helper.SubstrParseObj();
                    var time = parseInt(timeStr);

                    // invalid
                    if (isNaN(time))
                        return substrParseObj;

                    // time (unix) is complete, no more chars possible
                    if (timeStr.length == 10)
                        substrParseObj.moreCharsPossible = false;

                    substrParseObj.valid = true;
                    return substrParseObj;

                }
            },

            // do we support this format
            hasFormat: function (format) {
                return helper.formatToFunctionMapper.hasOwnProperty(format);
            },


            /*
             * Parse a date string according to specified format array and
             * return a string with the right delimiter added at the end if possible
             */
            getDecoratedDateFromFormat: function (dateStr, formatArr) {
                // this is what is returned
                // we update this as we go
                var parseObj = {
                    str: dateStr,
                    parseComplete: false,
                    valid: false,
                    moreCharsPossible: false
                };

                // this is used to maintain state within the string
                // so that we know where we are in the string, and
                // where to start parsing next
                var strObj = {
                    str: dateStr,
                    index: 0
                };

                // if string is empty, we need a dummy ParseInfo for later checking
                var substrParsedObj = new helper.SubstrParseObj();
                substrParsedObj.moreCharsPossible = false;
                var formatArrIndex;

                // basic parsing based on formatArr
                // try matching each format specifier with the string
                // and keep parsing
                for (formatArrIndex = 0; formatArrIndex < formatArr.length && strObj.index < strObj.str.length; formatArrIndex++) {

                    var format = formatArr[formatArrIndex];

                    // if this is a supported format-specifier
                    if (helper.hasFormat(format)) {

                        // get substring until next delimiter
                        var delimiterIndex = getDelimiterIndex(strObj);
                        var substr = strObj.str.substring(strObj.index, delimiterIndex);
                        strObj.index = delimiterIndex;

                        // try to parse string according to format
                        substrParsedObj = helper.formatToFunctionMapper[format](substr);

                        if (!substrParsedObj.valid)
                            return parseObj;

                        // this is used to help with the user editing the middle of the string
                        // if there is any possibility of adding more chars to any part of the date-time,
                        // returnObj.moreCharsPossible is set to true
                        if (substrParsedObj.moreCharsPossible)
                            parseObj.moreCharsPossible = true;
                    }
                    else {
                        strObj.index += format.length;
                    }
                }


                // if whole string parsed and last part of the string is also complete
                // that means the parse is complete
                if (formatArrIndex == formatArr.length && !substrParsedObj.moreCharsPossible) {
                    parseObj.parseComplete = true;
                }

                // append the delimiters now
                if (formatArrIndex < formatArr.length && !substrParsedObj.moreCharsPossible) {

                    while (formatArrIndex < formatArr.length) {

                        var format = formatArr[formatArrIndex];
                        // if this is not a supported format specifier
                        // treat it as a delimiter and append it
                        if (!helper.formatToFunctionMapper.hasOwnProperty(format)) {
                            parseObj.str += format;
                            formatArrIndex++;
                            if (formatArrIndex == formatArr.length) {
                                parseObj.parseComplete = true;
                                break;
                            }

                        }
                        else {
                            break;
                        }
                    }

                }

                parseObj.valid = true;
                return parseObj;
            }
        };

        /*
         * this is where the execution starts
         */
        var formatArr = getFormatArray(formatStr, false);
        return helper.getDecoratedDateFromFormat(dateStr, formatArr);
    }



    Date.prototype.getPreviousDate = function () {
        var date = this;
        return new Date(date.getTime() - (24 * 60 * 60 * 1000));
    }

    Date.prototype.getNextDate = function () {
        var date = this;
        return new Date(date.getTime() + (24 * 60 * 60 * 1000));
    }

    Date.prototype.isWeekend = function () {
        var date = this;
        return date.getDay() == 6 || date.getDay() == 0;
    }

    dateUtilities.datesAreEqual = function (date1, date2) {
        if (date1 === null || date2 === null)
            return false;
        return date1.format("YYYY/MM/DD") == date2.format("YYYY/MM/DD");
    }

    dateUtilities.daysDiff = function (startDate, endDate) {
        var oneDay = 24 * 60 * 60 * 1000;
        return Math.abs(Math.round((startDate - endDate) / (oneDay))) + 1;
    }

    /*
     * Used for fixed size ranges
     *
     * Takes in a duration, and a pivotBehaviour, and
     * Returns {startDate:________, endDate:________}
     * with the date(this) as the anchor
     *   anchored at left   if pivotBehaviour == -1
     *   anchored at middle if pivotBehaviour ==  0
     *   anchored at right  if pivotBehaviour ==  1
     *
     * Example
     * var d = new Date(2016, 5, 21);
     * dateUtilities.getFixedRange(d, 3, 0);
     * {
     *     startDate: Mon Jun 20 2016 00:00:00 GMT-0400 (EDT),
     *     endDate: Wed Jun 22 2016 23:59:59 GMT-0400 (EDT)
     * }
     *
     * Note: the startDate and endDate are Date objects
     *
     */
    dateUtilities.getFixedRange = function (selectedDate, rangeDurationInDays, pivotBehaviour) {
        var range = { startDate: null, endDate: null };
        rangeDurationInDays--;

        if (pivotBehaviour < 0) {
            range.startDate = new Date(selectedDate);
            range.endDate = new Date(selectedDate);
            range.endDate.setDate(range.endDate.getDate() + rangeDurationInDays);
        }
        else if (pivotBehaviour > 0) {
            range.endDate = new Date(selectedDate);
            range.startDate = new Date(selectedDate);
            range.startDate.setDate(range.startDate.getDate() - rangeDurationInDays);
        }
        else { // pivotBehaviour == 0
            range.startDate = new Date(selectedDate);
            range.startDate.setDate(range.startDate.getDate() - (rangeDurationInDays / 2));
            range.endDate = new Date(range.startDate);
            range.endDate.setDate(range.endDate.getDate() + rangeDurationInDays);
        }

        range.startDate.setHours(0); range.startDate.setMinutes(0); range.startDate.setSeconds(0);
        range.endDate.setHours(23); range.endDate.setMinutes(59); range.endDate.setSeconds(59);

        return range;
    }


    /*
     * Used for fixed size ranges
     *
     * Takes in a duration, and a pivotBehaviour, and
     * returns an array of dates in the range
     * with the date(this) as the anchor
     *   anchored at left   if pivotBehaviour == -1
     *   anchored at middle if pivotBehaviour ==  0
     *   anchored at right  if pivotBehaviour ==  1
     *
     * Example
     * var d = new Date(2016, 5, 21);
     * d.getDatesInFixedRange(3, 0);
     * [
     *     Mon Jun 20 2016 00:00:00 GMT-0400 (EDT),
     *     Tue Jun 21 2016 00:00:00 GMT-0400 (EDT),
     *     Wed Jun 22 2016 00:00:00 GMT-0400 (EDT)
     * ]
     *
     * Note: the returned array contains Date objects
     */
    dateUtilities.getDatesInFixedRange = function (selectedDate, rangeDurationInDays, pivotBehaviour) {

        var range = dateUtilities.getFixedRange(selectedDate, rangeDurationInDays, pivotBehaviour);
        var rangeArr = [];

        for (var date = range.startDate; date < range.endDate; date = date.getNextDate())
            rangeArr.push(date);

        return rangeArr;
    }


    /*
     * Returns true if the date(this) is in the specified range (both inclusive)
     * Assumes that range is an object with properties startDate & endDate
     */
    dateUtilities.isInRange = function (date, startDate, endDate) {
        if (date === null || startDate === null || endDate === null)
            return false;

        var f = "YYYYMMDD";
        return startDate.format(f) <= date.format(f) && date.format(f) <= endDate.format(f);
    }


    /* Example
     * dateUtilities.getMonth(0);
     * "Jan"
     */
    dateUtilities.getMonth = function (month) {
        return (new Date(2016, month).format("MMM"));
    }



    /*
     * Given a year and a month, returns an array of dates to be displayed on the calendar
     * including the dates from previous and next months that should be displayed
     * because they fall in the first and the last week of the month
     * based on the firstDayOfWeek
     * if firstDayOfWeek == 0, calendar starts with Sunday, and so on
     *
     * Example
     *
     * dateUtilities.getDatesInCalendarMonth(2016, 5, 0);
     *
     * Returns the following array
     *
     * [
     *      Sun May 29 2016 12:00:00 GMT-0400 (EDT),
     *      Mon May 30 2016 12:00:00 GMT-0400 (EDT),
     *      Tue May 31 2016 12:00:00 GMT-0400 (EDT),
     *      Wed Jun 01 2016 12:00:00 GMT-0400 (EDT),
     *  . . .
     *      Thu Jun 30 2016 12:00:00 GMT-0400 (EDT),
     *      Fri Jul 01 2016 12:00:00 GMT-0400 (EDT),
     *      Sat Jul 02 2016 12:00:00 GMT-0400 (EDT)
     * ]
     *
     */
    dateUtilities.getDatesInCalendarMonth = function (year, month, firstDayOfWeek) {
        if (isNaN(year) || isNaN(month) || isNaN(firstDayOfWeek)) {
            console.log("WTF! Enter some numbers");
            return [];
        }

        // Guard against user entering something other than [0-6]
        firstDayOfWeek %= 7;

        if (month < 0 || month > 11) {
            console.log("Error: month range out of 0-11");
            return [];
        }

        var dates = [];
        var date = new Date(year, month, 1, 12);
        var firstOfMonth = date;

        // start from 1st of the month and go until
        // the last date of the same month
        while (date.getMonth() == firstOfMonth.getMonth()) {
            dates.push(date);
            date = date.getNextDate();
        }

        // go even further to the next month until you reach firstDayOfWeek
        while (date.getDay() != firstDayOfWeek) {
            dates.push(date);
            date = date.getNextDate();
        }

        // if the 1st of the month is the firstDayOfWeek
        // then, we're done
        if (dates[0].getDay() == firstDayOfWeek)
            return dates;


        date = firstOfMonth.getPreviousDate();

        // go back from the 1st of the month to previous month
        // until you reach firstDayOfWeek
        while (date.getDay() != firstDayOfWeek) {
            dates.unshift(date);
            date = date.getPreviousDate();
        }
        // and also push that date (that is firstDayOfWeek)
        dates.unshift(date);

        return dates;
    }



    /*
     * Takes in a format-specifier string, breaks it up into its components
     * and returns an array with each format specifier or delimiter
     *
     * used by all other functions like dateUtilities.getDateFromStr(),
     * dateUtilities.decorateDateWithFormat(), Date.prototype.format()
     *
     * Examples
     *
     * getFormatArray("DD-MM-YYYY hh:mmA", false);
     * ['DD', '-', 'MM', '-', 'YYYY', ' ', 'hh', ':', 'mm', 'A']
     *
     * getFormatArray("DD-MM-YYYY hh:mmA", true);
     * ['DD', '-', 'MM', '-', 'YYYY', 'hh', ':', 'mm', 'A']
     */


    function getFormatArray(formatStr, ignoreSpaces) {
        var arr = [];
        var i = 0;
        var len = formatStr.length;

        function findLetterSequence() {
            var currentLetter = formatStr[i];
            var j = i;
            var currentStr = "";
            while (j < len) {
                if (formatStr[j] == currentLetter)
                    currentStr += formatStr[j++];
                else
                    break;
            }
            return currentStr;
        }

        function isAllSpaces(str) {
            for (var i = 0; i < str.length; i++) {
                if (str[i] != ' ')
                    return false;
            }
            return true;
        }

        var format;
        while (i < len) {
            var format = findLetterSequence();
            if (!(ignoreSpaces && isAllSpaces(format)))
                arr.push(format);
            i += format.length;
        }

        return arr;
    }




    /*
     * Used by formatToFunctionMapper's functions to find the next delimiter
     * while parsing the date-time string
     *
     * Example
     * var strObj = {str : "12Jan", index:0};
     * getDelimiterIndex(strObj);
     * Returns 2 (corresponding to the 'J' in "12Jan")
     */
    function getDelimiterIndex(strObj) {
        var i = strObj.index;
        while (strObj.str[i] == " ")
            i++;
        var stopFunc = (isCharNumber(strObj.str[i])) ? isCharNotNumber : isCharNumber;
        var delimiters = "`~!@#$%^&*()-_=+[{]}\\|;:'\",<.>/? ";

        for (; i < strObj.str.length; i++) {
            if (delimiters.search(strObj.str[i]) != -1 || stopFunc(strObj.str[i]))
                break;
        }
        return i;
    }



    /* simple functions used by
     * getDelimiterIndex()
     */
    function isCharNotNumber(aChar) {
        return isNaN(parseInt(aChar));
    }
    function isCharNumber(nChar) {
        return !isNaN(parseInt(nChar));
    }

})();


/***************************END date-utilities**********************************/

// calender code starts here
(function ($) {

    var defaults = {
        date: null,
        autoclose: false, //option for whether or not Calendar should close upon date selection
        restrictDatesFunc: false, //Can be passed a function to restrict a certain date or groups of dates
        firstDayOfWeek: 0,         //sets the first day of each week for the Calendar, 0 being Sunday
        format: "MM/DD/YYYY",
        inline: false,
        datePicker: true,
        startDateBoundary: false, //boundary for which all days previous to this will be unselectable
        endDateBoundary: false,  //boundary for which all dates after this will be unselectable
        timePicker: false,
        rangePicker: false,
        setValue: null,
        rangeOptions: {
            duration: 1,
            pivots: { "LEFT": -1, "MIDDLE": 0, "RIGHT": 1 },
            selectedPivot: "LEFT",
            fixed: false,
            startDate: null,
            endDate: null,
            minDays: 0,             //minimum amount of days that can be selected for a range
            maxDays: 0,             //maximum of days that can be selected for a range
            selectForward: false,   //if true, only dates after the range startDate will be selectable
            selectBackward: false, //if true, only dates before the range startDate will be selectable
            rangeSeparator: 'to',  //String that seperates the range startDate from the range endDate in the input field
        }
    }

    //constuctor function for calender plugins
    function Calendar(element, options) {
        this.element = element;
        //initialize optionss
        this.initOptions(defaults, options);
        //set calender variables
        this.selectedDate = this.options.date;
        this.previousDate = new Date(this.selectedDate);
        this.calendarMonth = this.selectedDate.getMonth();
        this.calendarYear = this.selectedDate.getFullYear();
        this.rangeOptions = this.options.rangeOptions;
        this.rangeStates = { 'noclicks': 0, 'oneclick': 1 }; //0 - startDate clicked  1- endDate Clicked
        this.rangeState = this.rangeStates.noclicks;
        this.timeoutHandle = null;
        //initialize dom
        this.initDOM();
    }

    /**public functions
     * exposed to developers
     */
    Calendar.prototype = {
        initOptions: function (defaults, options) {
            // initialization logic for options
            this.options = $.extend(true, {}, defaults, options);
            this.options.date = this.options.date !== null ? this.options.date : new Date();
            this.options.timePicker = this.options.rangeOptions.duration > 1 ? false : this.options.timePicker;
            //set up default time for range picker
            if (this.options.rangePicker) {
                if (this.isFlexibleRangePicker()) {
                    if (this.options.rangeOptions.startDate === null)
                        this.options.rangeOptions.startDate = new Date();
                    if (this.options.rangeOptions.endDate === null)
                        this.options.rangeOptions.endDate = new Date();
                }
            }
            //set up default time format
            if (options != undefined && !options.format) {
                if (this.options.timePicker && !this.options.datePicker) {
                    this.options.format = "hh:mm A";
                } else if (this.options.timePicker && this.options.datePicker && !this.options.rangePicker) {
                    this.options.format = "MM/DD/YYYY hh:mm A";
                } else if (this.options.rangePicker) {
                    this.options.format = "MM/DD/YYYY";
                }
            }
            if (typeof this.options.startDateBoundary === 'string')
                this.options.startDateBoundary = dateUtilities.getDateFromStr(this.options.startDateBoundary, this.options.format);
            if (typeof this.options.endDateBoundary === 'string')
                this.options.endDateBoundary = dateUtilities.getDateFromStr(this.options.endDateBoundary, this.options.format);
        },
        initDOM: function () {
            // Place initialization logic here
            // You already have access to the DOM element and
            // the options via the instance, e.g. this.element
            // and this.options
            this.CACalendar = $("<div class=ca-date-time-picker>");
            this.calendarDiv = $("<div class=ca-calendar>");
            this.timeDiv = $("<div class=ca-timeselector>");
            this.CACalendar.append(this.calendarDiv);
            this.CACalendar.append(this.timeDiv);
            $("body").append(this.CACalendar);
            if (!this.options.inline) {
                this.calendarDiv.hide();
                this.timeDiv.hide();
                this.element.on("click", $.proxy(this.open, this));
            }
            else {
                $(window).on('resize', $.proxy(this.reposition, this));
                $(window).scroll($.proxy(this.reposition, this));
                this.reposition();
                this.CACalendar.show();
                this.options.datePicker ? this.calendarDiv.show() : this.calendarDiv.hide();
                this.options.timePicker ? this.timeDiv.show() : this.timeDiv.hide();
            }
            this.element.addClass('ca-date-time-input');
            this.element.on("keyup", $.proxy(this.parseDateHandler, this));
            if (this.isFlexibleRangePicker()) {
                this.rangeToolTip = $(HTMLUtilities.getToolTipHTML());
                this.CACalendar.append(this.rangeToolTip);
            }
            this.renderMonth();
            this.renderTimePicker();
        },
        open: function (e) {
            this.reposition();
            $(window).on('resize', $.proxy(this.reposition, this));
            this.CACalendar.show();
            if (this.options.datePicker) {
                this.calendarDiv.show();
            }
            if (this.options.timePicker) {
                this.timeDiv.show();
            }
            if (e) {
                e.stopPropagation();
                e.preventDefault();
            }
            var self = this;
            $(document).unbind("mousedown").bind("mousedown", $.proxy(this.close, this));
        },
        close: function (e) {
            var instPlugin = $(this.element).data("caCalendar");
            if (instPlugin) {
                var container = instPlugin.CACalendar;
                if (!container.is(e.target) && container.has(e.target).length === 0 && !$(e.target).is(instPlugin.element)) {
                    this.CACalendar.fadeOut(150);
                }
            }
        },
        closeAllCalendars: function (e) {
            $('.ca-date-time-input').each(function (inst) {
                var instPlugin = $(this).data("caCalendar");
                if (instPlugin) {
                    var container = instPlugin.CACalendar;
                    if (!container.is(e.target) && container.has(e.target).length === 0 && !$(e.target).is(instPlugin.element)) {
                        instPlugin.close();
                    }
                }
            });
        },
        reposition: function () {
            var inputPosition = this.element.offset();
            this.CACalendar.css({ top: inputPosition.top + this.element.height(), left: inputPosition.left });
        },
        getDate: function (formatStr) {
            if (arguments.length == 0)
                return this.selectedDate;
            else
                return this.selectedDate.format(formatStr);
        },
        getDates: function (formatStr) {
            if (this.isFlexibleRangePicker()) {
                var flexibleDuration = dateUtilities.daysDiff(this.rangeOptions.startDate, this.rangeOptions.endDate);
                var dates = dateUtilities.getDatesInFixedRange(this.rangeOptions.startDate, flexibleDuration, -1);
            }
            else
                var dates = dateUtilities.getDatesInFixedRange(this.selectedDate, this.rangeOptions.duration, this.rangeOptions.pivots[this.rangeOptions.selectedPivot]);
            if (arguments.length == 0)
                return dates;
            else {
                var formattedDates = []
                for (var i = 0; i < dates.length; i++)
                    formattedDates.push(dates[i].format(formatStr));
                return formattedDates;
            }
        },
        restrictDateBoundaries: function (startBound, endBound) {
            //restricts dates on the Calendar from being selected which are before the startBound, after the endBound, or both.
            //accepts a Date Object for the bounds and toggles the 'unselectable' class
            var self = this;
            if (startBound && !endBound) {
                startBound.setHours(0); startBound.setMinutes(0); startBound.setSeconds(0); startBound.setMilliseconds(0);
                this.toggleCellClasses('unselectable', function (date) {
                    return date < startBound;
                });
            }
            else if (!startBound && endBound) {
                endBound.setHours(0); endBound.setMinutes(0); endBound.setSeconds(0); endBound.setMilliseconds(0);
                this.toggleCellClasses('unselectable', function (date) {
                    return date > endBound;
                });
            }
            else if (startBound && endBound) {
                startBound.setHours(0); startBound.setMinutes(0); startBound.setSeconds(0); startBound.setMilliseconds(0);
                endBound.setHours(0); endBound.setMinutes(0); endBound.setSeconds(0); endBound.setMilliseconds(0);
                this.toggleCellClasses('unselectable', function (date) {
                    return date < startBound || date > endBound;
                });
            }
        },
        restrictDates: function () {
            //takes the restrictDatesFunc function passed in config and uses it to restrict a certain date(s) from being selected
            //i.e. this.options.restrictDatesFunc = funcion(date){ return date.getDay == 0 || date.getDay == 6} will restrict weekends from selection
            if (this.options.restrictDatesFunc)
                this.toggleCellClasses('unselectable', this.options.restrictDatesFunc);
        },
        forwardSelect: function (clickedDate) {
            //when rangePicker is true this will allow for only forward selection of dates and toggle the 'unselectable' class on
            //all previous dates
            var self = this;
            if (this.rangeOptions.selectForward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date < clickedDate || date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
        },
        backwardSelect: function (clickedDate) {
            //when rangePicker is true, this will allow for only backward selection of dates and toggle the 'unselectable' class on
            //all future dates
            var self = this;
            if (this.rangeOptions.selectBackward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date > clickedDate || date < (self.options.startDateBoundary ? self.options.startDateBoundary : date);
                });
        },
        selectMinMaxDays: function (clickedDate) {
            //this method, when rangePicker is true, will allow for only a certain minimum, maximum, or both, number of days to be selectable
            //and takes into account startDateBoundary and endDateBoundary
            var maxForwardDate = new Date(clickedDate.getTime());
            var maxBackwardDate = new Date(clickedDate.getTime());
            var minForwardDate = new Date(clickedDate.getTime());
            var minBackwardDate = new Date(clickedDate.getTime());
            maxForwardDate.setDate(clickedDate.getDate() + this.rangeOptions.maxDays - 1);
            maxBackwardDate.setDate(clickedDate.getDate() - this.rangeOptions.maxDays + 1);
            minForwardDate.setDate(clickedDate.getDate() + this.rangeOptions.minDays - 1);
            minBackwardDate.setDate(clickedDate.getDate() - this.rangeOptions.minDays + 1);
            if (this.rangeOptions.minDays != 0 && this.rangeOptions.maxDays == 0)
                this.transformMinDays(clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate);
            else if (this.rangeOptions.minDays == 0 && this.rangeOptions.maxDays != 0)
                this.transformMaxDays(clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate);
            else if (this.rangeOptions.minDays != 0 && this.rangeOptions.maxDays != 0)
                this.transformMinAndMaxDays(clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate);
        },
        transformMinDays: function (clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate) {
            var self = this;
            if (this.rangeOptions.selectForward)
                this.toggleCellClasses('unselectable', function (date) {
                    return (date < minForwardDate && date > clickedDate) || (date < clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else if (this.rangeOptions.selectBackward)
                this.toggleCellClasses('unselectable', function (date) {
                    return (date > clickedDate) || (date > minBackwardDate && date < clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else
                this.toggleCellClasses('unselectable', function (date) {
                    return (date < minForwardDate && date > clickedDate) || (date > minBackwardDate && date < clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
        },
        transformMaxDays: function (clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate) {
            var self = this;
            if (this.rangeOptions.selectForward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date > maxForwardDate || date < clickedDate ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else if (this.rangeOptions.selectBackward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date > clickedDate || date < maxBackwardDate ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else
                this.toggleCellClasses('unselectable', function (date) {
                    return date > maxForwardDate || date < maxBackwardDate ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
        },
        transformMinAndMaxDays: function (clickedDate, maxForwardDate, maxBackwardDate, minForwardDate, minBackwardDate) {
            var self = this;
            if (this.rangeOptions.selectForward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date < clickedDate || maxForwardDate < date ||
                        (date < minForwardDate && date >= clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else if (this.rangeOptions.selectBackward)
                this.toggleCellClasses('unselectable', function (date) {
                    return date < maxBackwardDate || clickedDate < date ||
                        (date > minBackwardDate && date <= clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
            else
                this.toggleCellClasses('unselectable', function (date) {
                    return date < maxBackwardDate || maxForwardDate < date ||
                        (date < minForwardDate && date >= clickedDate) ||
                        (date > minBackwardDate && date <= clickedDate) ||
                        date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                        date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                });
        },
        setDate: function (date) {
            if (date && date instanceof Date) {
                this.selectedDate = date;
            }
            this.setTimeToElement(this.selectedDate);
        },
        setStartDate: function (date) {
            this.rangeOptions.startDate = date;
            this.renderMonth();
        },
        setEndDate: function (date) {
            this.rangeOptions.endDate = date;
            this.renderMonth();
        },
        renderMonth: function () {
            var htmlStr = HTMLUtilities.getCalenderHTML(this.calendarYear, this.calendarMonth, this.selectedDate, this.options.firstDayOfWeek);
            this.calendarDiv.html(htmlStr)
                .unbind()
                .on({
                    click: $.proxy(this.calenderClickHandler, this)
                })
                .on("mouseover", ".year", $.proxy(this.showYearPicker, this))
                .on("mouseleave", ".year-picker", $.proxy(this.hideYearPicker, this));
            if (this.isFlexibleRangePicker()) {
                this.calendarDiv.on("mouseover", ".date", $.proxy(this.onHoverHighlightRange, this));
            }
            this.adjustCssClasses();
            this.setDate();
            this.restrictDateBoundaries(this.options.startDateBoundary, this.options.endDateBoundary);
            if (this.rangeState === this.rangeStates.oneclick) {
                //this is to correctly grey out dates if the user changes the year or month in the middle of their range selection
                //i.e. if they hit the next month arrow after clicking the start date for a range (before finishing the range selection),
                // this will 'gray' out any dates that were gray before they left
                this.rangeToolTip.hide();
                if (this.rangeOptions.selectForward)
                    this.restrictDateBoundaries(this.selectedDate)
                else if (this.rangeOptions.selectBackward)
                    this.restrictDateBoundaries(false, this.selectedDate)
                this.selectMinMaxDays(this.selectedDate);
            }
            this.restrictDates();
            this.onDateChangeCallBack();
        },
        renderTimePicker: function () {
            this.timeDiv.html(HTMLUtilities.getTimePickerHTML(this.selectedDate))
                .unbind()
                .on({
                    click: $.proxy(this.timeClickHandler, this)
                })
                .on("keyup", ".hour", $.proxy(this.parseHourHandler, this))
                .on("keyup", ".minute", $.proxy(this.parseMinuteHandler, this));
            this.onDateChangeCallBack();
        },
        onDateChangeCallBack: function () {
            if (this.previousDate.getTime() != this.selectedDate.getTime()) {
                if (typeof this.options.onDateChange === "function") {
                    this.options.onDateChange(this.selectedDate, this.previousDate, this);
                }
                this.previousDate = new Date(this.selectedDate);
            }
        },
        parseHourHandler: function (event) {
            if (event.keyCode == 8 || event.keyCode == 46 || (event.keyCode > 36 && event.keyCode < 41))
                return;
            $elem = $(event.target);
            var hours = parseInt($elem.val());
            if (hours <= 12) {
                this.selectedDate.setHours(hours);
                if ($elem.val().length == 2) {
                    this.renderTimePicker();
                    this.setTimeToElement(this.selectedDate);
                }
            }
        },
        parseMinuteHandler: function (event) {
            if (event.keyCode == 8 || event.keyCode == 46 || (event.keyCode > 36 && event.keyCode < 41))
                return;
            $elem = $(event.target);
            var minutes = parseInt($elem.val());
            if (minutes < 60) {
                this.selectedDate.setMinutes(minutes);
                if ($elem.val().length == 2) {
                    this.renderTimePicker();
                    this.setTimeToElement(this.selectedDate);
                }
            }
        },
        setTimeToElement: function () {
            var str, s, s1, s2;
            if (this.options.rangePicker) {
                if (this.isFixedRangePicker()) {
                    var range = dateUtilities.getFixedRange(this.selectedDate, this.rangeOptions.duration, this.rangeOptions.pivots[this.rangeOptions.selectedPivot]);
                    this.rangeOptions.startDate = range.startDate;
                    this.rangeOptions.endDate = range.endDate;
                }
                str = this.rangeOptions.startDate ? this.rangeOptions.startDate.format(this.options.format) : "";
                str += " " + this.rangeOptions.rangeSeparator + " ";
                str += this.rangeOptions.endDate ? this.rangeOptions.endDate.format(this.options.format) : "";
                s = str;
                s1 = str.split((' ' + this.rangeOptions.rangeSeparator + ' '))[0];
                s2 = str.split((' ' + this.rangeOptions.rangeSeparator + ' '))[1];
            } else {
                str = this.selectedDate.format(this.options.format);
                s = str;
            }
            if (typeof this.options.setValue === 'function') {
                this.options.setValue(s, s1, s2);
            }
            else
                this.element.val(str);
        },
        calenderClickHandler: function (e) {
            //determine the area and pass it over
            e.stopPropagation();
            e.preventDefault();
            var target = $(e.target).closest('div, td, th');
            if (target.hasClass("date")) {
                this.dateClickHandler(target);
            } else if (target.hasClass("change-month")) {
                this.changeMonth(target);
            } else if (target.hasClass("change-year")) {
                this.changeYear(target);
            } else if (target.hasClass("today")) {
                this.setSelectedDateToToday(target);
            }
        },
        dateClickHandler: function (target) {
            var d = new Date(target.data("year"), target.data("month"), target.data("date"));
            this.changeDate(d);
        },
        changeDateTime: function (date) {
            this.selectedDate = date;
            this.changeDate(date);
            this.renderTimePicker();
        },
        changeDate: function (date) {
            //check whether date is in the current month and change accordingly
            if (this.calendarMonth != date.getMonth() || this.calendarYear != date.getFullYear()) {
                this.calendarMonth = date.getMonth();
                this.calendarYear = date.getFullYear();
                this.renderMonth();
            }
            if (this.isFlexibleRangePicker()) {
                this.rangePickerClickHandler(date);
            } else {
                this.moveDate(date);
            }
            this.setTimeToElement();
            this.adjustCssClasses();
        },
        rangePickerClickHandler: function (date) {
            if (!date)
                return;
            var self = this;
            if (this.rangeState === this.rangeStates.noclicks) {
                //set start date and change state
                this.rangeState = this.rangeStates.oneclick;
                this.rangeOptions.endDate = null;
                this.rangeOptions.startDate = date;
                this.forwardSelect(date);
                this.backwardSelect(date);
                this.selectMinMaxDays(date);
                this.selectedDate = date;
            } else if (this.rangeState === this.rangeStates.oneclick) {
                //set end date and change state
                if (this.rangeOptions.minDays || this.rangeOptions.maxDays || this.rangeOptions.selectForward || this.rangeOptions.selectBackward) {
                    if (this.options.startDateBoundary || this.options.endDateBoundary) {//resetting grayed out cells
                        var self = this;
                        this.toggleCellClasses('unselectable', function (date) {
                            return date < (self.options.startDateBoundary ? self.options.startDateBoundary : date) ||
                                date > (self.options.endDateBoundary ? self.options.endDateBoundary : date);
                        });
                    }
                    else
                        this.toggleCellClasses('unselectable', function (date) {
                            return false;
                        });
                }
                this.rangeState = this.rangeStates.noclicks;
                if (date < this.rangeOptions.startDate) {
                    this.rangeOptions.endDate = this.rangeOptions.startDate;
                    this.rangeOptions.startDate = date;
                    this.selectedDate = this.rangeOptions.startDate;
                } else {
                    this.rangeOptions.endDate = date;
                    this.selectedDate = this.rangeOptions.endDate;
                }
                setTimeout(function () {
                    self.rangeToolTip.hide();
                }, 300);
            }
        },
        onHoverHighlightRange: function (event) {
            if (this.rangeState === this.rangeStates.oneclick) {
                var target = $(event.target).closest('td');
                hoverdDate = new Date(target.data("year"), target.data("month"), target.data("date"));
                var self = this;
                this.toggleCellClasses('date-in-range', function (date) {
                    return hoverdDate > self.rangeOptions.startDate ? date > self.rangeOptions.startDate && date <= hoverdDate :
                        date <= self.rangeOptions.startDate && date > hoverdDate;
                });
                var cellPosition = target.position();
                this.rangeToolTip.html(dateUtilities.daysDiff(self.rangeOptions.startDate, hoverdDate) + " days");
                this.rangeToolTip.show();
                this.rangeToolTip.css({ top: cellPosition.top - target.height(), left: cellPosition.left });
            }
        },
        moveDate: function (date) {
            var time = this.selectedDate - new Date(this.selectedDate.toDateString());
            this.selectedDate = new Date(date.toDateString());
            this.selectedDate.setTime(this.selectedDate.getTime() + time);
            if (this.options.autoclose)
                this.CACalendar.fadeOut(150);
        },
        toggleCellClasses: function (cssClass, comparisionLogic) {
            this.calendarDiv.find("td.date").each(function (index) {
                var cellData = $(this).data();
                var date = new Date(cellData["year"], cellData["month"], cellData["date"]);
                if (comparisionLogic(date)) {
                    $(this).addClass(cssClass);
                } else {
                    $(this).removeClass(cssClass);
                }
            });
        },
        adjustCssClasses: function () {
            var self = this;
            this.toggleCellClasses('date-selected', function (date) {
                return false;
            });
            if (this.options.rangePicker) {
                if (this.isFixedRangePicker()) {
                    range = dateUtilities.getFixedRange(this.selectedDate, this.rangeOptions.duration, this.rangeOptions.pivots[this.rangeOptions.selectedPivot]);
                    this.rangeOptions.startDate = range.startDate;
                    this.rangeOptions.endDate = range.endDate;
                }
                this.toggleCellClasses('date-in-range', function (date) {
                    return dateUtilities.isInRange(date, self.rangeOptions.startDate, self.rangeOptions.endDate);
                });
                if (this.isFixedRangePicker()) {
                    this.toggleCellClasses('date-selected', function (date) {
                        return dateUtilities.datesAreEqual(date, self.selectedDate);
                    });
                } else if (this.isFlexibleRangePicker()) {
                    this.toggleCellClasses('date-selected', function (date) {
                        return dateUtilities.datesAreEqual(date, self.rangeOptions.startDate) ||
                            dateUtilities.datesAreEqual(date, self.rangeOptions.endDate);
                    });
                }
            }
            else {
                this.toggleCellClasses('date-selected', function (date) {
                    return dateUtilities.datesAreEqual(date, self.selectedDate);
                });
            }
        },
        changeYear: function (target) {
            var year = target.parents(".calendar-header").data("year");
            if (target.hasClass("next-year")) {
                year = year + 1;
            } else if (target.hasClass("previous-year")) {
                year = year - 1;
            }
            this.calendarYear = year;
            this.renderMonth();
            this.calendarDiv.find('.year').trigger('mouseover');
        },
        changeMonth: function (target) {
            //based on prev and next month change this
            var month = target.parent().data("month");
            var year = target.parent().data("year");
            if (target.hasClass("next-month")) {
                month = (month + 1) % 12;
                if (month == 0) {
                    year = year + 1;
                }
            } else if (target.hasClass("previous-month")) {
                month = (month - 1) % 12;
                if (month == -1) {
                    year = year - 1;
                    month = 11; //dec
                }
            }
            this.calendarYear = year;
            this.calendarMonth = month;
            this.renderMonth();
        },
        setSelectedDateToToday: function (target) {
            this.changeDate(new Date());
        },
        showYearPicker: function (e) {
            $e = $(e.target)
            var yearPicker = $e.siblings(".year-picker");
            yearPicker.show();
            $e.css("opacity", 0);
        },
        hideYearPicker: function (e) {
            $e = $(e.target)
            if (!$e.hasClass("year-picker")) {
                $e = $e.parents("year-picker");
            }
            var year = $e.siblings(".year");
            year.css("opacity", 100);
            $e.fadeOut(100);
        },

        setDateFromRange: function (startDate, endDate, isEditingStartDate) {
            if (isEditingStartDate) {
                this.selectedDate = new Date(startDate);
                if (this.rangeOptions.selectedPivot == "RIGHT") {
                    this.selectedDate.setDate(this.selectedDate.getDate() + this.rangeOptions.duration - 1);
                } else if (this.rangeOptions.selectedPivot == "MIDDLE") {
                    this.selectedDate.setDate(this.selectedDate.getDate() + this.rangeOptions.duration / 2);
                }
            } else {
                this.selectedDate = new Date(endDate);
                if (this.rangeOptions.selectedPivot == "LEFT") {
                    this.selectedDate.setDate(this.selectedDate.getDate() - this.rangeOptions.duration + 1);
                } else if (this.rangeOptions.selectedPivot == "MIDDLE") {
                    this.selectedDate.setDate(this.selectedDate.getDate() - this.rangeOptions.duration / 2 + 1);
                }
            }
        },

        parseDateHandler: function (event) {
            //skip backspace, delete and arrow keys
            if (event.keyCode == 8 || event.keyCode == 46 || (event.keyCode > 36 && event.keyCode < 41))
                return;
            var $elem = $(event.target),
                dateString = $elem.val();

            var position = $elem.getCursorPosition();
            if (this.options.rangePicker) {
                var range = dateString.split(this.rangeOptions.rangeSeparator).map(function (d) { return d.trim() });
                if (range.length == 2) {
                    var startFormattedDate = dateUtilities.decorateDateWithFormat(range[0], this.options.format),
                        endFormattedDate = dateUtilities.decorateDateWithFormat(range[1], this.options.format),
                        isEditingStartDate = false;

                    formatAndSetDateRange = function () {
                        var startDate = dateUtilities.getDateFromStr(range[0], this.options.format),
                            endDate = dateUtilities.getDateFromStr(range[1], this.options.format);
                        if (position <= range[0].length) {
                            isEditingStartDate = true;
                        }
                        if (startDate > endDate && this.isFlexibleRangePicker()) {
                            //reverse dates and position
                            this.rangeOptions.startDate = endDate;
                            this.rangeOptions.endDate = startDate;
                            if (position > range[0].length)
                                position -= range[0].length + this.rangeOptions.rangeSeparator.length + 2;
                            else
                                position += range[0].length + this.rangeOptions.rangeSeparator.length + 2;
                        } else {
                            this.rangeOptions.startDate = startDate;
                            this.rangeOptions.endDate = endDate;
                        }
                        if (this.isFixedRangePicker()) {
                            this.setDateFromRange(this.rangeOptions.startDate, this.rangeOptions.endDate, isEditingStartDate);
                            this.changeDate(this.selectedDate);
                        } else {
                            this.rangeState = this.rangeStates.oneclick;
                            this.changeDate(this.rangeOptions.endDate);
                        }
                    }
                    if (this.timeoutHandle) clearTimeout(this.timeoutHandle);
                    if (startFormattedDate.parseComplete && endFormattedDate.parseComplete) {
                        if (startFormattedDate.moreCharsPossible || endFormattedDate.moreCharsPossible) {
                            this.timeoutHandle = setTimeout($.proxy(formatAndSetDateRange, this), 1000);
                        } else {
                            formatAndSetDateRange.call(this);
                        }

                    } else {
                        if (startFormattedDate.valid)
                            range[0] = startFormattedDate.str;
                        if (endFormattedDate.valid)
                            range[1] = endFormattedDate.str;
                        $elem.val(range.join(" " + this.rangeOptions.rangeSeparator + " "));
                    }
                }
            } else {
                formattedDate = dateUtilities.decorateDateWithFormat(dateString, this.options.format);
                if (this.timeoutHandle) clearTimeout(this.timeoutHandle);
                if (formattedDate.parseComplete) {
                    if (formattedDate.moreCharsPossible) {
                        this.timeoutHandle = setTimeout($.proxy(function () {
                            date = dateUtilities.getDateFromStr(dateString, this.options.format);
                            this.changeDateTime(date);
                        }, this), 1000);
                    } else {
                        date = dateUtilities.getDateFromStr(dateString, this.options.format);
                        this.changeDateTime(date);
                    }
                } else if (formattedDate.valid) {
                    $elem.val(formattedDate.str);
                    position += formattedDate.str.length - dateString.length;
                }
            }
            $elem.setCursorPosition(position);
        },
        /**
         * Time functions
         */
        timeClickHandler: function (e) {
            //determine the area and pass it over
            e.stopPropagation();
            e.preventDefault();
            var target = $(e.target).closest('div, td, th');
            if (target.hasClass("change-hour")) {
                this.changeHour(target);
            } else if (target.hasClass("change-meridiem")) {
                this.changeMeridiem(target);
            } else if (target.hasClass("change-minutes")) {
                this.changeMinutes(target);
            } else if (target.hasClass("now")) {
                this.setTimeToNow(target);
            }
        },
        changeHour: function (target) {
            //check whether date is in the current month and change accordingly
            //hourInput = target.siblings(".hour");
            if (target.hasClass("increase-hour")) {
                this.selectedDate.setHours(this.selectedDate.getHours() + 1);
            } else if (target.hasClass("decrease-hour")) {
                this.selectedDate.setHours(this.selectedDate.getHours() - 1);
            }
            this.renderTimePicker();
            this.setTimeToElement(this.selectedDate);
        },
        changeMinutes: function (target) {
            if (target.hasClass("increase-minute")) {
                this.selectedDate.setMinutes(this.selectedDate.getMinutes() + 1);
            } else if (target.hasClass("decrease-minute")) {
                this.selectedDate.setMinutes(this.selectedDate.getMinutes() - 1);
            }
            this.renderTimePicker();
            this.setTimeToElement(this.selectedDate);
        },
        changeMeridiem: function (target) {
            //based on prev and next month change this
            if (target.hasClass("increase-am-pm") || target.hasClass("decrease-am-pm")) {
                var hours = this.selectedDate.getHours();
                if (hours >= 12) {
                    this.selectedDate.setHours(hours - 12);
                } else {
                    this.selectedDate.setHours(hours + 12);
                }
            }
            this.renderTimePicker();
            this.setTimeToElement(this.selectedDate);
        },
        setTimeToNow: function (target) {
            var now = new Date();
            this.selectedDate.setHours(now.getHours(), now.getMinutes(), now.getSeconds());
            this.renderTimePicker();
            this.setTimeToElement(this.selectedDate);
        },

        /** */
        isFlexibleRangePicker: function () {
            return this.options.rangePicker && !this.options.rangeOptions.fixed;
        },
        isFixedRangePicker: function () {
            return this.options.rangePicker && this.options.rangeOptions.fixed;
        }
    }


    //jquery plugin for calendar
    $.fn.caCalendar = function (options) {
        return this.each(function (params) {
            var $this = $(this);
            if (!$this.data("caCalendar")) {
                $this.data('caCalendar', new Calendar($this, options));
            }
        }).data('caCalendar');
    }

    var HTMLUtilities = {
        getCalenderHTML : function (calendarYear, calendarMonth, selectedDate, firstDayOfWeek) {

            var helper = {


                /*
                * creates the table body containing all rows and tds of
                * the calendar
                * Also, attaches appropriate classes
                */
                createCalendarTableBody: function (calendarYear, calendarMonth, selectedDate, firstDayOfWeek) {
                    var dates = dateUtilities.getDatesInCalendarMonth(calendarYear, calendarMonth, firstDayOfWeek);

                    var $tbody = $("<tbody>");
                    var d;

                    // used for adding css class 'top-edge'
                    var isFirstIter = true;

                    while (dates.length != 0) {

                        // row containing 7 td's
                        var $tr = $("<tr>");

                        for (var i = 0; i < 7; i++) {

                            // remove date from the beginning of the array
                            var date = dates.shift();

                            /*
                            // don't need this, since array of dates is a multiple of 7
                            // the inner for loop does 7 at a time
                            // and we have condition in while

                            // if array empty, break
                            if (date === undefined)
                                break;
                            */

                            var $td = $("<td>", {
                                'data-year': date.getFullYear(),
                                'data-month': date.getMonth(),
                                'data-date': date.getDate()
                            });

                            // standard class added to every date
                            $td.addClass("date");

                            // selected date
                            $td.addClass(dateUtilities.datesAreEqual(date, selectedDate) ? "date-selected" : "");

                            // add classes representing whether the date belongs to current month or other months
                            $td.addClass(date.getMonth() == calendarMonth ? "date-current-month" : "date-other-month");

                            // add class for saturday and sunday
                            $td.addClass(date.isWeekend() ? "date-weekend" : "");

                            // these classes help with the edge cells
                            // the styles set the border-top and border-left to 0
                            // for top-edge and left-edge, respectively
                            // since we don't want an outer boundary
                            $td.addClass(isFirstIter ? "top-edge" : "");
                            $td.addClass(i == 0 ? "left-edge" : "");

                            $td.append(date.format("D"));
                            $tr.append($td);
                        }
                        $tbody.append($tr);

                        isFirstIter = false;
                    }
                    return $tbody.html();

                },



                /*
                * Used for generating HTML for the years inside
                * the year picker at the calendar header
                */
                getThreeConsecutiveYearsHTML: function (year) {
                    var outputStr = "";
                    for (var i = year - 1; i <= year + 1; i++) {
                        var classStr = (i == year) ? "current-year" : "";
                        outputStr += '<div class=' + classStr + '>' + i + '</div>';
                    }
                    return '<div class=years>' + outputStr + '</div>';
                },


                /*
                * Used for generating HTML for the top row of table
                * that contains the days of the weekend
                * based on the first day of the week
                */
                getDayOfWeekHeaderHTML: function (firstDayOfWeek) {
                    var str = "";
                    var daysOfWeek = ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'];
                    for (var i = 0; i < 7; i++)
                        str += '<th>' + daysOfWeek[(i + firstDayOfWeek) % 7] + '</th>';
                    return str;
                },

                /*
                * Used for generating HTML for the year picker
                * at the calendar header
                */
                getYearPickerHTML: function (calendarYear) {
                    return '<div class="change-year previous-year"><span class="glyphicon glyphicon-triangle-top"></span></div>' +
                        helper.getThreeConsecutiveYearsHTML(calendarYear) +
                        '<div class="change-year next-year"><span class="glyphicon glyphicon-triangle-bottom"></span></div>';
                }

            };




            var htmlStr =
                '<div class=calendar-header data-month=' + calendarMonth + ' data-year=' + calendarYear + '>' +
                '<div class="change-month previous-month"><span class="glyphicon glyphicon-triangle-left"></span></div>' +
                '<div class="change-month next-month"><span class="glyphicon glyphicon-triangle-right"></span></div>' +
                '<div class=month-year>' +
                '<div class=month>' +
                dateUtilities.getMonth(calendarMonth) +
                '</div> ' +
                '<div class=year>' +
                calendarYear +
                '</div>' +
                '<div class=year-picker>' +
                helper.getYearPickerHTML(calendarYear) +
                '</div>' +

                '</div>' +
                '<div class="clear"></div>' +
                '</div>' +

                '<hr class=hr1>' +

                '<table class="calendar-table">' +
                '<thead>' + helper.getDayOfWeekHeaderHTML(firstDayOfWeek) + '</thead>' +
                helper.createCalendarTableBody(calendarYear, calendarMonth, selectedDate, firstDayOfWeek) +
                '</table>' +

                '<hr class=hr2>' +
                '<div class=today>TODAY</div>';

            return htmlStr;

        },

        getToolTipHTML : function () {
            return '<div class=date-range-length-tool-tip></div>';
        },

        getTimePickerHTML : function (date) {

            var helper = {
                getUTCHours: function (date) {
                    var hr_str = date.getUTCHours().toString();
                    return (hr_str.length == 1) ? "0" + hr_str : hr_str;
                },
                getUTCMinutes: function (date) {
                    var min_str = date.getUTCMinutes().toString();
                    return (min_str.length == 1) ? "0" + min_str : min_str;
                },
                getUTCTimeString: function (date) {
                    return helper.getUTCHours(date) + ":" + helper.getUTCMinutes(date) + " " + "GMT";
                },
                getLocalTimeString: function (date) {
                    return date.format("hh:mm A") + " " + "LOCAL";
                }
            };


            var htmlStr =
                '<div class=selected-time-container>' +
                '<div class="selected-time-gmt">' + helper.getUTCTimeString(date) + '</div>' +
                '<div class="selected-time-local">' + helper.getLocalTimeString(date) + '</div>' +
                '<div class="clear"></div>' +
                '</div>' +

                '<div class=inputs-container>' +

                '<div class="hour-container">' +
                '<div class="change-hour increase-hour">' +
                '<span class="glyphicon glyphicon-triangle-top">' +
                '</div>' +
                '<input type=text class=hour value=' + date.format("hh") + '>' +
                '<div class="change-hour decrease-hour">' +
                '<span class="glyphicon glyphicon-triangle-bottom">' +
                '</div>' +
                '</div>' +
                '<div class="colon-container">:</div>' +
                '<div class="am-pm-container">' +
                '<div class="change-meridiem increase-am-pm">' +
                '<span class="glyphicon glyphicon-triangle-top">' +
                '</div>' +
                '<input type=text class=am-pm value=' + date.format("A") + '>' +
                '<div class="change-meridiem decrease-am-pm">' +
                '<span class="glyphicon glyphicon-triangle-bottom">' +
                '</div>' +
                '</div>' +
                '<div class="minute-container">' +
                '<div class="change-minutes increase-minute">' +
                '<span class="glyphicon glyphicon-triangle-top">' +
                '</div>' +
                '<input type=text class=minute value=' + date.format("mm") + '>' +
                '<div class="change-minutes decrease-minute">' +
                '<span class="glyphicon glyphicon-triangle-bottom">' +
                '</div>' +
                '</div>' +

                '<div class="clear"></div>' +

                '</div>' +

                '<div class=now>NOW</div>';

            return htmlStr;

        }
    }

})(jQuery);

//jquery plugin for getting cursor position for input type fields
(function ($, undefined) {
    $.fn.getCursorPosition = function () {
        var el = $(this).get(0);
        var pos = 0;
        if ('selectionStart' in el) {
            pos = el.selectionStart;
        } else if ('selection' in document) {
            el.focus();
            var Sel = document.selection.createRange();
            var SelLength = document.selection.createRange().text.length;
            Sel.moveStart('character', -el.value.length);
            pos = Sel.text.length - SelLength;
        }
        return pos;
    }
})(jQuery);

//jquery plugin for setting cursor position for input type fields
(function ($, undefined) {
    $.fn.setCursorPosition = function (pos) {
        var elem = $(this).get(0);
        if (elem.setSelectionRange) {
            elem.setSelectionRange(pos, pos);
        } else if (elem.createTextRange) {
            var range = elem.createTextRange();
            range.collapse(true);
            range.moveEnd('character', pos);
            range.moveStart('character', pos);
            range.select();
        }
    }
})(jQuery);
;/* ========================================================================
 * Bootstrap: carousel.js v3.3.5
 * http://getbootstrap.com/javascript/#carousel
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // CAROUSEL CLASS DEFINITION
  // =========================

  var Carousel = function (element, options) {
    this.$element    = $(element)
    this.$indicators = this.$element.find('.carousel-indicators')
    this.options     = options
    this.paused      = null
    this.sliding     = null
    this.interval    = null
    this.$active     = null
    this.$items      = null

    this.options.keyboard && this.$element.on('keydown.bs.carousel', $.proxy(this.keydown, this))

    this.options.pause == 'hover' && !('ontouchstart' in document.documentElement) && this.$element
      .on('mouseenter.bs.carousel', $.proxy(this.pause, this))
      .on('mouseleave.bs.carousel', $.proxy(this.cycle, this))
  }

  Carousel.VERSION  = '3.3.5'

  Carousel.TRANSITION_DURATION = 600

  Carousel.DEFAULTS = {
    interval: 5000,
    pause: 'hover',
    wrap: true,
    keyboard: true
  }

  Carousel.prototype.keydown = function (e) {
    if (/input|textarea/i.test(e.target.tagName)) return
    switch (e.which) {
      case 37: this.prev(); break
      case 39: this.next(); break
      default: return
    }

    e.preventDefault()
  }

  Carousel.prototype.cycle = function (e) {
    e || (this.paused = false)

    this.interval && clearInterval(this.interval)

    this.options.interval
      && !this.paused
      && (this.interval = setInterval($.proxy(this.next, this), this.options.interval))

    return this
  }

  Carousel.prototype.getItemIndex = function (item) {
    this.$items = item.parent().children('.item')
    return this.$items.index(item || this.$active)
  }

  Carousel.prototype.getItemForDirection = function (direction, active) {
    var activeIndex = this.getItemIndex(active)
    var willWrap = (direction == 'prev' && activeIndex === 0)
                || (direction == 'next' && activeIndex == (this.$items.length - 1))
    if (willWrap && !this.options.wrap) return active
    var delta = direction == 'prev' ? -1 : 1
    var itemIndex = (activeIndex + delta) % this.$items.length
    return this.$items.eq(itemIndex)
  }

  Carousel.prototype.to = function (pos) {
    var that        = this
    var activeIndex = this.getItemIndex(this.$active = this.$element.find('.item.active'))

    if (pos > (this.$items.length - 1) || pos < 0) return

    if (this.sliding)       return this.$element.one('slid.bs.carousel', function () { that.to(pos) }) // yes, "slid"
    if (activeIndex == pos) return this.pause().cycle()

    return this.slide(pos > activeIndex ? 'next' : 'prev', this.$items.eq(pos))
  }

  Carousel.prototype.pause = function (e) {
    e || (this.paused = true)

    if (this.$element.find('.next, .prev').length && $.support.transition) {
      this.$element.trigger($.support.transition.end)
      this.cycle(true)
    }

    this.interval = clearInterval(this.interval)

    return this
  }

  Carousel.prototype.next = function () {
    if (this.sliding) return
    return this.slide('next')
  }

  Carousel.prototype.prev = function () {
    if (this.sliding) return
    return this.slide('prev')
  }

  Carousel.prototype.slide = function (type, next) {
    var $active   = this.$element.find('.item.active')
    var $next     = next || this.getItemForDirection(type, $active)
    var isCycling = this.interval
    var direction = type == 'next' ? 'left' : 'right'
    var that      = this

    if ($next.hasClass('active')) return (this.sliding = false)

    var relatedTarget = $next[0]
    var slideEvent = $.Event('slide.bs.carousel', {
      relatedTarget: relatedTarget,
      direction: direction
    })
    this.$element.trigger(slideEvent)
    if (slideEvent.isDefaultPrevented()) return

    this.sliding = true

    isCycling && this.pause()

    if (this.$indicators.length) {
      this.$indicators.find('.active').removeClass('active')
      var $nextIndicator = $(this.$indicators.children()[this.getItemIndex($next)])
      $nextIndicator && $nextIndicator.addClass('active')
    }

    var slidEvent = $.Event('slid.bs.carousel', { relatedTarget: relatedTarget, direction: direction }) // yes, "slid"
    if ($.support.transition && this.$element.hasClass('slide')) {
      $next.addClass(type)
      $next[0].offsetWidth // force reflow
      $active.addClass(direction)
      $next.addClass(direction)
      $active
        .one('bsTransitionEnd', function () {
          $next.removeClass([type, direction].join(' ')).addClass('active')
          $active.removeClass(['active', direction].join(' '))
          that.sliding = false
          setTimeout(function () {
            that.$element.trigger(slidEvent)
          }, 0)
        })
        .emulateTransitionEnd(Carousel.TRANSITION_DURATION)
    } else {
      $active.removeClass('active')
      $next.addClass('active')
      this.sliding = false
      this.$element.trigger(slidEvent)
    }

    isCycling && this.cycle()

    return this
  }


  // CAROUSEL PLUGIN DEFINITION
  // ==========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.carousel')
      var options = $.extend({}, Carousel.DEFAULTS, $this.data(), typeof option == 'object' && option)
      var action  = typeof option == 'string' ? option : options.slide

      if (!data) $this.data('bs.carousel', (data = new Carousel(this, options)))
      if (typeof option == 'number') data.to(option)
      else if (action) data[action]()
      else if (options.interval) data.pause().cycle()
    })
  }

  var old = $.fn.carousel

  $.fn.carousel             = Plugin
  $.fn.carousel.Constructor = Carousel


  // CAROUSEL NO CONFLICT
  // ====================

  $.fn.carousel.noConflict = function () {
    $.fn.carousel = old
    return this
  }


  // CAROUSEL DATA-API
  // =================

  var clickHandler = function (e) {
    var href
    var $this   = $(this)
    var $target = $($this.attr('data-target') || (href = $this.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '')) // strip for ie7
    if (!$target.hasClass('carousel')) return
    var options = $.extend({}, $target.data(), $this.data())
    var slideIndex = $this.attr('data-slide-to')
    if (slideIndex) options.interval = false

    Plugin.call($target, options)

    if (slideIndex) {
      $target.data('bs.carousel').to(slideIndex)
    }

    e.preventDefault()
  }

  $(document)
    .on('click.bs.carousel.data-api', '[data-slide]', clickHandler)
    .on('click.bs.carousel.data-api', '[data-slide-to]', clickHandler)

  $(window).on('load', function () {
    $('[data-ride="carousel"]').each(function () {
      var $carousel = $(this)
      Plugin.call($carousel, $carousel.data())
    })
  })

}(jQuery);
;/* ========================================================================
 * Bootstrap: collapse.js v3.3.5
 * http://getbootstrap.com/javascript/#collapse
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // COLLAPSE PUBLIC CLASS DEFINITION
  // ================================

  var Collapse = function (element, options) {
    this.$element      = $(element)
    this.options       = $.extend({}, Collapse.DEFAULTS, options)
    this.$trigger      = $('[data-toggle="collapse"][href="#' + element.id + '"],' +
                           '[data-toggle="collapse"][data-target="#' + element.id + '"]')
    this.transitioning = null

    if (this.options.parent) {
      this.$parent = this.getParent()
    } else {
      this.addAriaAndCollapsedClass(this.$element, this.$trigger)
    }

    if (this.options.toggle) this.toggle()
  }

  Collapse.VERSION  = '3.3.5'

  Collapse.TRANSITION_DURATION = 350

  Collapse.DEFAULTS = {
    toggle: true
  }

  Collapse.prototype.dimension = function () {
    var hasWidth = this.$element.hasClass('width')
    return hasWidth ? 'width' : 'height'
  }

  Collapse.prototype.show = function () {
    if (this.transitioning || this.$element.hasClass('in')) return

    var activesData
    var actives = this.$parent && this.$parent.children('.panel').children('.in, .collapsing')

    if (actives && actives.length) {
      activesData = actives.data('bs.collapse')
      if (activesData && activesData.transitioning) return
    }

    var startEvent = $.Event('show.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    if (actives && actives.length) {
      Plugin.call(actives, 'hide')
      activesData || actives.data('bs.collapse', null)
    }

    var dimension = this.dimension()

    this.$element
      .removeClass('collapse')
      .addClass('collapsing')[dimension](0)
      .attr('aria-expanded', true)

    this.$trigger
      .removeClass('collapsed')
      .addClass('open')[dimension]('')
      .attr('aria-expanded', true)

    this.transitioning = 1

    var complete = function () {
      this.$element
        .removeClass('collapsing')
        .addClass('collapse in')[dimension]('')
      this.transitioning = 0
      this.$element
        .trigger('shown.bs.collapse')
    }

    if (!$.support.transition) return complete.call(this)

    var scrollSize = $.camelCase(['scroll', dimension].join('-'))

    this.$element
      .one('bsTransitionEnd', $.proxy(complete, this))
      .emulateTransitionEnd(Collapse.TRANSITION_DURATION)[dimension](this.$element[0][scrollSize])
  }

  Collapse.prototype.hide = function () {
    if (this.transitioning || !this.$element.hasClass('in')) return

    var startEvent = $.Event('hide.bs.collapse')
    this.$element.trigger(startEvent)
    if (startEvent.isDefaultPrevented()) return

    var dimension = this.dimension()

    this.$element[dimension](this.$element[dimension]())[0].offsetHeight

    this.$element
      .addClass('collapsing')
      .removeClass('collapse in')
      .attr('aria-expanded', false)

    this.$trigger
      .addClass('collapsed')
      .attr('aria-expanded', false)

    this.transitioning = 1

    var complete = function () {
      this.transitioning = 0
      this.$element
        .removeClass('collapsing')
        .addClass('collapse')
        .trigger('hidden.bs.collapse')
    }

    if (!$.support.transition) return complete.call(this)

    this.$element
      [dimension](0)
      .one('bsTransitionEnd', $.proxy(complete, this))
      .emulateTransitionEnd(Collapse.TRANSITION_DURATION)
  }

  Collapse.prototype.toggle = function () {
    this[this.$element.hasClass('in') ? 'hide' : 'show']()
  }

  Collapse.prototype.getParent = function () {
    return $(this.options.parent)
      .find('[data-toggle="collapse"][data-parent="' + this.options.parent + '"]')
      .each($.proxy(function (i, element) {
        var $element = $(element)
        this.addAriaAndCollapsedClass(getTargetFromTrigger($element), $element)
      }, this))
      .end()
  }

  Collapse.prototype.addAriaAndCollapsedClass = function ($element, $trigger) {
    var isOpen = $element.hasClass('in')

    $element.attr('aria-expanded', isOpen)
    $trigger
      .toggleClass('collapsed', !isOpen)
      .attr('aria-expanded', isOpen)
  }

  function getTargetFromTrigger($trigger) {
    var href
    var target = $trigger.attr('data-target')
      || (href = $trigger.attr('href')) && href.replace(/.*(?=#[^\s]+$)/, '') // strip for ie7

    return $(target)
  }


  // COLLAPSE PLUGIN DEFINITION
  // ==========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.collapse')
      var options = $.extend({}, Collapse.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data && options.toggle && /show|hide/.test(option)) options.toggle = false
      if (!data) $this.data('bs.collapse', (data = new Collapse(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.collapse

  $.fn.collapse             = Plugin
  $.fn.collapse.Constructor = Collapse


  // COLLAPSE NO CONFLICT
  // ====================

  $.fn.collapse.noConflict = function () {
    $.fn.collapse = old
    return this
  }


  // COLLAPSE DATA-API
  // =================

  $(document).on('click.bs.collapse.data-api', '[data-toggle="collapse"]', function (e) {
    var $this   = $(this)

    if (!$this.attr('data-target')) e.preventDefault()

    var $target = getTargetFromTrigger($this)
    var data    = $target.data('bs.collapse')
    var option  = data ? 'toggle' : $this.data()

    Plugin.call($target, option)
  })

}(jQuery);
;/* ========================================================================
 * Bootstrap: dropdown.js v3.3.5
 * http://getbootstrap.com/javascript/#dropdowns
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
    'use strict';

    // DROPDOWN CLASS DEFINITION
    // =========================

    var backdrop = '.dropdown-backdrop';
    var toggle   = '[data-toggle="dropdown"]';
    var scrolling = '[data-dropdown-scroll="true"]'
    var scrollenabled = false;
    var Dropdown = function (element) {
        $(element).on('click.bs.dropdown', this.toggle)
        scrollenabled = enableScroll();
    }

    Dropdown.VERSION = '3.3.5'

    function getParent($this) {
        var selector = $this.attr('data-target')

        if (!selector) {
            selector = $this.attr('href')
            selector = selector && /#[A-Za-z]/.test(selector) && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
        }

        var $parent = selector && $(selector)

        return $parent && $parent.length ? $parent : $this.parent()
    }

    function clearMenus(e) {
        if (e && e.which === 3) return
        $(backdrop).remove()
        $(toggle).each(function () {
            var $this         = $(this)
            var $parent       = getParent($this)
            var relatedTarget = { relatedTarget: this }

            if (!$parent.hasClass('open')) return

            if (e && e.type == 'click' && /input|textarea/i.test(e.target.tagName) && $.contains($parent[0], e.target)) return

            $parent.trigger(e = $.Event('hide.bs.dropdown', relatedTarget))

            if (e.isDefaultPrevented()) return

            $this.attr('aria-expanded', 'false')
            $parent.removeClass('open').trigger('hidden.bs.dropdown', relatedTarget)
        })
    }

    Dropdown.prototype.toggle = function (e) {
        var $this = $(this)

        if ($this.is('.disabled, :disabled')) return

        var $parent  = getParent($this)
        var isActive = $parent.hasClass('open')

        clearMenus()

        if (!isActive) {
            if ('ontouchstart' in document.documentElement && !$parent.closest('.navbar-nav').length) {
                // if mobile we use a backdrop because click events don't delegate
                $(document.createElement('div'))
                    .addClass('dropdown-backdrop')
                    .insertAfter($(this))
                    .on('click', clearMenus)
            }

            var relatedTarget = { relatedTarget: this }
            $parent.trigger(e = $.Event('show.bs.dropdown', relatedTarget))

            if (e.isDefaultPrevented()) return

            $this
                .trigger('focus')
                .attr('aria-expanded', 'true')

            $parent
                .toggleClass('open')
                .trigger('shown.bs.dropdown', relatedTarget)

            if($parent.find(".dropdown-menu").width()<$parent.find(".dropdown-toggle").outerWidth()){
                $(".dropdown-menu").css("min-width", $parent.find(".dropdown-toggle").outerWidth())
            }
        }

        return false
    }

    var scroll = function(parent,direction, eleheight,limit){
        var isup = direction.toUpperCase() === "UP";
        var lis = parent.find("li:not('.arrow')");
        var currenttop = parseInt(lis.eq(0).css("top"),10);
        var toplimit = 0;
        var botlimit = -Math.floor(eleheight-limit+lis.eq(0).height());
        if(isup){
           parent.addClass("showbottom");
            if(currenttop >= toplimit){
                parent.removeClass("showtop");
                return;
            }
            currenttop+=5;
            currenttop = Math.min(currenttop,toplimit);


        }else{
            parent.addClass("showtop");
            if(currenttop <= botlimit){
                parent.removeClass("showbottom");
                return;
            }
            currenttop-=5;
            currenttop = Math.max(currenttop,botlimit);
        }
        lis.css("top",currenttop);
    }

    var calcPageMax = function(ele,str){
        var wheight = window.innerHeight;
        var dheight = $("body").height();
        var base = Math.min(dheight,wheight)
        if(!str || str === "window"){
            return .8 * base; //default is 80%
        }
        else{
            //process str
            var s = str.substring(1,str.length);//remove w;
            s = s.split("-");
            if(s.length === 1){
                s = s[0].trim();
                if(s.indexOf("%") > -1){
                    var perc = parseFloat(s);
                    if (isNaN(perc)) return .8 * base; //return default behavior
                    else return (perc/100.0) * base;
                }
                else if(!isNan(parseInt(s))){
                    return parseInt(s); //return the int case
                }
                else{
                    return .8 * base;
                }
            }

            else if(s.length === 2){
                //case empty string, means subtract
                if(s[0] === ""){
                    s = s[1].trim();
                    var rm = parseInt(s);
                    if (isNaN(rm)) return .8 * base; //return default behavior
                    else return base - rm;
                }
                else{
                    var perc = s[0].trim();
                    var rm = s[1].trim();
                    if(perc.indexOf("%") > -1) perc = parseFloat(perc);
                    rm = parseInt(rm);
                    if(isNaN(rm+perc)){
                        return .8 * base;
                    }
                    else return (perc/100.0) * base - rm
                }
            }
            return .8 * base; //if it hasn't returned yet
        }


    }

    var enableScroll = function(element){
        var scrollels = ((element)?$(element):$(document)).find(scrolling)
        if(!scrollels.length)return;
        scrollels.each(function(){
            var btn = $(this)
            var ul = btn.next();
            var eleheight = ul.height();
            var maxsize = btn.data("maxsize");
            var usepagemax = (maxsize.charAt(0) === "w")
            var limit = (usepagemax)?calcPageMax(btn,maxsize):(!isNaN(+maxsize))?+maxsize:300; //code dynamic options later
            ul.css("max-height",limit);
            ul.addClass("scrolling");
            if(eleheight > limit) ul.addClass("showbottom");
            if(usepagemax){
                $(window).on("resize.bs.dropdown.sizechange",function(){
                    limit = calcPageMax(btn,maxsize);
                    ul.css("max-height",limit);
                    var lis = ul.find("li:not('.arrow')");
                    var toplimit = 0;
                    var botlimit = -Math.floor(eleheight-limit+lis.eq(0).height());
                    var newtop = parseInt(lis.eq(0).css("top"),10);
                    newtop = Math.max(newtop,botlimit);
                    newtop = Math.min(newtop,toplimit);
                    lis.css("top",newtop);
                    if(eleheight > limit){
                        ul.addClass("scrolling");
                        if(newtop < 0) ul.addClass("showtop");
                        else ul.removeClass("showtop");
                        if(newtop > botlimit) ul.addClass("showbottom");
                        else ul.removeClass("showbottom");
                    }
                    else{
                        ul.removeClass("showtop");
                        ul.removeClass("showbottom");
                    }
                })
            }
            var uparrow = $("<li class='arrow up'></li>")
            var contscroll = null;
            uparrow.on("mouseover",function(e){
                contscroll = setInterval(function(){
                    scroll(ul,"up",eleheight,limit);
                },25);
            }).on("mouseleave",function(e){
                clearInterval(contscroll);
            })
            var downarrow = $("<li class='arrow down'></li>")
            downarrow.on("mouseover",function(e){
                contscroll = setInterval(function(){
                    scroll(ul,"down",eleheight,limit);
                },25);
            }).on("mouseleave",function(e){
                clearInterval(contscroll);
            })
            ul.find("li:first").before(uparrow)
            ul.find("li:last").after(downarrow)

        });

    }


    Dropdown.prototype.keydown = function (e) {
        if (!/(38|40|27|32)/.test(e.which) || /input|textarea/i.test(e.target.tagName)) return

        var $this = $(this)

        e.preventDefault()
        e.stopPropagation()

        if ($this.is('.disabled, :disabled')) return

        var $parent  = getParent($this)
        var isActive = $parent.hasClass('open')

        if (!isActive && e.which != 27 || isActive && e.which == 27) {
            if (e.which == 27) $parent.find(toggle).trigger('focus')
            return $this.trigger('click')
        }

        var desc = ' li:not(.disabled):visible a'
        var $items = $parent.find('.dropdown-menu' + desc)

        if (!$items.length) return

        var index = $items.index(e.target)

        if (e.which == 38 && index > 0)                 index--         // up
        if (e.which == 40 && index < $items.length - 1) index++         // down
        if (!~index)                                    index = 0

        $items.eq(index).trigger('focus')
    }


    // DROPDOWN PLUGIN DEFINITION
    // ==========================

    function Plugin(option) {
        return this.each(function () {
            var $this = $(this)
            var data  = $this.data('bs.dropdown')

            if (!data) $this.data('bs.dropdown', (data = new Dropdown(this)))
            if (typeof option == 'string') data[option].call($this)
        })
    }

    var old = $.fn.dropdown

    $.fn.dropdown             = Plugin
    $.fn.dropdown.Constructor = Dropdown


    // DROPDOWN NO CONFLICT
    // ====================

    $.fn.dropdown.noConflict = function () {
        $.fn.dropdown = old
        return this
    }
    enableScroll();

    // APPLY TO STANDARD DROPDOWN ELEMENTS
    // ===================================

    $(document)
        .on('click.bs.dropdown.data-api', clearMenus)
        .on('click.bs.dropdown.data-api', '.dropdown form', function (e) { e.stopPropagation() })
        .on('click.bs.dropdown.data-api', toggle, Dropdown.prototype.toggle)
        .on('keydown.bs.dropdown.data-api', toggle, Dropdown.prototype.keydown)
        .on('keydown.bs.dropdown.data-api', '.dropdown-menu', Dropdown.prototype.keydown)

}(jQuery);
;/* ========================================================================
 * Bootstrap: modal.js v3.3.5
 * http://getbootstrap.com/javascript/#modals
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // MODAL CLASS DEFINITION
  // ======================

  var Modal = function (element, options) {
    this.options             = options
    this.$body               = $(document.body)
    this.$element            = $(element)
    this.$dialog             = this.$element.find('.modal-dialog')
    this.$backdrop           = null
    this.isShown             = null
    this.originalBodyPad     = null
    this.scrollbarWidth      = 0
    this.ignoreBackdropClick = false

    if (this.options.remote) {
      this.$element
        .find('.modal-content')
        .load(this.options.remote, $.proxy(function () {
          this.$element.trigger('loaded.bs.modal')
        }, this))
    }
  }

  Modal.VERSION  = '3.3.5'

  Modal.TRANSITION_DURATION = 300
  Modal.BACKDROP_TRANSITION_DURATION = 150

  Modal.DEFAULTS = {
    backdrop: true,
    keyboard: true,
    show: true
  }

  Modal.prototype.toggle = function (_relatedTarget) {
    return this.isShown ? this.hide() : this.show(_relatedTarget)
  }

  Modal.prototype.show = function (_relatedTarget) {
    var that = this
    var e    = $.Event('show.bs.modal', { relatedTarget: _relatedTarget })

    this.$element.trigger(e)

    if (this.isShown || e.isDefaultPrevented()) return

    this.isShown = true

    this.checkScrollbar()
    this.setScrollbar()
    this.$body.addClass('modal-open')

    this.escape()
    this.resize()

    this.$element.on('click.dismiss.bs.modal', '[data-dismiss="modal"]', $.proxy(this.hide, this))

    this.$dialog.on('mousedown.dismiss.bs.modal', function () {
      that.$element.one('mouseup.dismiss.bs.modal', function (e) {
        if ($(e.target).is(that.$element)) that.ignoreBackdropClick = true
      })
    })

    this.backdrop(function () {
      var transition = $.support.transition && that.$element.hasClass('fade')

      if (!that.$element.parent().length) {
        that.$element.appendTo(that.$body) // don't move modals dom position
      }

      that.$element
        .show()
        .scrollTop(0)

      that.adjustDialog()

      if (transition) {
        that.$element[0].offsetWidth // force reflow
      }

      that.$element.addClass('in')

      that.enforceFocus()

      var e = $.Event('shown.bs.modal', { relatedTarget: _relatedTarget })

      transition ?
        that.$dialog // wait for modal to slide in
          .one('bsTransitionEnd', function () {
            that.$element.trigger('focus').trigger(e)
          })
          .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
        that.$element.trigger('focus').trigger(e)
    })
  }

  Modal.prototype.hide = function (e) {
    if (e) e.preventDefault()

    e = $.Event('hide.bs.modal')

    this.$element.trigger(e)

    if (!this.isShown || e.isDefaultPrevented()) return

    this.isShown = false

    this.escape()
    this.resize()

    $(document).off('focusin.bs.modal')

    this.$element
      .removeClass('in')
      .off('click.dismiss.bs.modal')
      .off('mouseup.dismiss.bs.modal')

    this.$dialog.off('mousedown.dismiss.bs.modal')

    $.support.transition && this.$element.hasClass('fade') ?
      this.$element
        .one('bsTransitionEnd', $.proxy(this.hideModal, this))
        .emulateTransitionEnd(Modal.TRANSITION_DURATION) :
      this.hideModal()
  }

  Modal.prototype.enforceFocus = function () {
    $(document)
      .off('focusin.bs.modal') // guard against infinite focus loop
      .on('focusin.bs.modal', $.proxy(function (e) {
        if (this.$element[0] !== e.target && !this.$element.has(e.target).length) {
          this.$element.trigger('focus')
        }
      }, this))
  }

  Modal.prototype.escape = function () {
    if (this.isShown && this.options.keyboard) {
      this.$element.on('keydown.dismiss.bs.modal', $.proxy(function (e) {
        e.which == 27 && this.hide()
      }, this))
    } else if (!this.isShown) {
      this.$element.off('keydown.dismiss.bs.modal')
    }
  }

  Modal.prototype.resize = function () {
    if (this.isShown) {
      $(window).on('resize.bs.modal', $.proxy(this.handleUpdate, this))
    } else {
      $(window).off('resize.bs.modal')
    }
  }

  Modal.prototype.hideModal = function () {
    var that = this
    this.$element.hide()
    this.backdrop(function () {
      that.$body.removeClass('modal-open')
      that.resetAdjustments()
      that.resetScrollbar()
      that.$element.trigger('hidden.bs.modal')
    })
  }

  Modal.prototype.removeBackdrop = function () {
    this.$backdrop && this.$backdrop.remove()
    this.$backdrop = null
  }

  Modal.prototype.backdrop = function (callback) {
    var that = this
    var animate = this.$element.hasClass('fade') ? 'fade' : ''

    if (this.isShown && this.options.backdrop) {
      var doAnimate = $.support.transition && animate

      this.$backdrop = $(document.createElement('div'))
        .addClass('modal-backdrop ' + animate)
        .appendTo(this.$body)

      this.$element.on('click.dismiss.bs.modal', $.proxy(function (e) {
        if (this.ignoreBackdropClick) {
          this.ignoreBackdropClick = false
          return
        }
        if (e.target !== e.currentTarget) return
        this.options.backdrop == 'static'
          ? this.$element[0].focus()
          : this.hide()
      }, this))

      if (doAnimate) this.$backdrop[0].offsetWidth // force reflow

      this.$backdrop.addClass('in')

      if (!callback) return

      doAnimate ?
        this.$backdrop
          .one('bsTransitionEnd', callback)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callback()

    } else if (!this.isShown && this.$backdrop) {
      this.$backdrop.removeClass('in')

      var callbackRemove = function () {
        that.removeBackdrop()
        callback && callback()
      }
      $.support.transition && this.$element.hasClass('fade') ?
        this.$backdrop
          .one('bsTransitionEnd', callbackRemove)
          .emulateTransitionEnd(Modal.BACKDROP_TRANSITION_DURATION) :
        callbackRemove()

    } else if (callback) {
      callback()
    }
  }

  // these following methods are used to handle overflowing modals

  Modal.prototype.handleUpdate = function () {
    this.adjustDialog()
  }

  Modal.prototype.adjustDialog = function () {
    var modalIsOverflowing = this.$element[0].scrollHeight > document.documentElement.clientHeight

    this.$element.css({
      paddingLeft:  !this.bodyIsOverflowing && modalIsOverflowing ? this.scrollbarWidth : '',
      paddingRight: this.bodyIsOverflowing && !modalIsOverflowing ? this.scrollbarWidth : ''
    })
  }

  Modal.prototype.resetAdjustments = function () {
    this.$element.css({
      paddingLeft: '',
      paddingRight: ''
    })
  }

  Modal.prototype.checkScrollbar = function () {
    var fullWindowWidth = window.innerWidth
    if (!fullWindowWidth) { // workaround for missing window.innerWidth in IE8
      var documentElementRect = document.documentElement.getBoundingClientRect()
      fullWindowWidth = documentElementRect.right - Math.abs(documentElementRect.left)
    }
    this.bodyIsOverflowing = document.body.clientWidth < fullWindowWidth
    this.scrollbarWidth = this.measureScrollbar()
  }

  Modal.prototype.setScrollbar = function () {
    var bodyPad = parseInt((this.$body.css('padding-right') || 0), 10)
    this.originalBodyPad = document.body.style.paddingRight || ''
    if (this.bodyIsOverflowing) this.$body.css('padding-right', bodyPad + this.scrollbarWidth)
  }

  Modal.prototype.resetScrollbar = function () {
    this.$body.css('padding-right', this.originalBodyPad)
  }

  Modal.prototype.measureScrollbar = function () { // thx walsh
    var scrollDiv = document.createElement('div')
    scrollDiv.className = 'modal-scrollbar-measure'
    this.$body.append(scrollDiv)
    var scrollbarWidth = scrollDiv.offsetWidth - scrollDiv.clientWidth
    this.$body[0].removeChild(scrollDiv)
    return scrollbarWidth
  }


  // MODAL PLUGIN DEFINITION
  // =======================

  function Plugin(option, _relatedTarget) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.modal')
      var options = $.extend({}, Modal.DEFAULTS, $this.data(), typeof option == 'object' && option)

      if (!data) $this.data('bs.modal', (data = new Modal(this, options)))
      if (typeof option == 'string') data[option](_relatedTarget)
      else if (options.show) data.show(_relatedTarget)
    })
  }

  var old = $.fn.modal

  $.fn.modal             = Plugin
  $.fn.modal.Constructor = Modal


  // MODAL NO CONFLICT
  // =================

  $.fn.modal.noConflict = function () {
    $.fn.modal = old
    return this
  }


  // MODAL DATA-API
  // ==============

  $(document).on('click.bs.modal.data-api', '[data-toggle="modal"]', function (e) {
    var $this   = $(this)
    var href    = $this.attr('href')
    var $target = $($this.attr('data-target') || (href && href.replace(/.*(?=#[^\s]+$)/, ''))) // strip for ie7
    var option  = $target.data('bs.modal') ? 'toggle' : $.extend({ remote: !/#/.test(href) && href }, $target.data(), $this.data())

    if ($this.is('a')) e.preventDefault()

    $target.one('show.bs.modal', function (showEvent) {
      if (showEvent.isDefaultPrevented()) return // only register focus restorer if modal will actually get shown
      $target.one('hidden.bs.modal', function () {
        $this.is(':visible') && $this.trigger('focus')
      })
    })
    Plugin.call($target, option, this)
  })

}(jQuery);
;/**
 * Created by cacewit on 5/4/16.
 */
+function ($) {
    $(document).on("click.navbar.userexpand"," .user-title-block",function(e){
        $(this).siblings(".user-menu").toggleClass("expanded");
        $(this).closest(".user-nav-wrapper").toggleClass("expanded");
    })
    $(document).on("click.navbar.globalexpand"," .global-nav",function(e){
        $(this).children(".global-nav-menu").toggleClass("expanded");
        $(this).closest(".user-nav-wrapper").removeClass("expanded");
    })
    $(document).on("click.test",function(e){
        var t = $(e.target);
        if(!t.hasClass(".user-title-block") && !t.closest(".user-title-block").length){
            $(".user-menu").removeClass("expanded");
            $(".user-nav-wrapper").removeClass("expanded");
        }
        if(!t.hasClass(".global-nav") && !t.closest(".global-nav").length){
            $(".global-nav-menu").removeClass("expanded");
        }
    })
    /*----------------------------Global Header Javascript Code ------------------------------------------------------------------------------------------*/
        /*--toggle side-nav-wrapper on click of the hamburger menu--*/
            $("#hmenu").click(function(){
                $(".side-nav-wrapper").toggleClass("expanded");
            });
        /*--close side-nav-wrapper when clicking outside of it--*/
            $(document).click(function(e){
                if(!$(e.target).closest(".side-nav-wrapper").length && !$(e.target).hasClass("ca-logo")){
                    $(".side-nav-wrapper").removeClass("expanded");
                }
            });
        /*--close side-global-nav inside side-nav-wrapper if click occurs outside its area--*/
            $(document).click(function(e){
                if(!$(e.target).closest(".side-nav-wrapper .header-menu.global-nav-menu").length && !$(e.target).hasClass("side-global-nav")){
                    $(".side-nav-wrapper .header-menu.global-nav-menu").removeClass("show-menu");
                }
            });
        /*--adjust product name position depending on whether or not there is a feature name--*/
            if($(".global-header > div.title").children().size() > 0 ){
                $(".global-header > div.title").css({"padding-top":"17px"});
                if($("#global-username").css("display") === "inline-block" && window.matchMedia("(max-width:480px)").matches){
                    $(".global-header > div.title").css({"padding-top":"7px"});
                }
            }
            else {
                $(".global-header > div.title").css({"padding-top":"25px"});
            }
        /*--if overflow occurs on the #product-name, replace the last few characters that would appear with periods instead (to show that the name has overflowed)--*/
           // if($("#product-name").css("overflow-y") === "hidden"){
             //   var string = $("#product-name").html();
               // var newString = string.slice(0, 61) + "...";
              //  $("#product-name").html(newString);
                /*--for matching the 360px viewport--*/
                //if (window.matchMedia('(max-device-width: 360px)').matches) {
                  //  var string = $("#product-name").html();
                   // var newString = string.slice(0, 48) + "...";
                   // $("#product-name").html(newString);
                //}
            //}
        /*--if overflow occurs on the #feature-name, replace the last few characters that would appear with periods instead (to show that the name has overflowed)--*/
            //if($("#feature-name").css("overflow-y") === "hidden"){
              //  var string = $("#feature-name").html();
                //var newString = string.slice(0, 34) + "...";
                //$("#feature-name").html(newString);
                ///*--for matching the 360px viewport--*/
                //if (window.matchMedia('(max-device-width: 360px)').matches) {
                  //  var string = $("#feature-name").html();
                    //var newString = string.slice(0, 26) + "...";
                    //$("#feature-name").html(newString);
                //}
            //}*/
        /*--if on viewport of 480px or less, take list items from global-nav and append it to the icon inside side-navbar-wrapper--*/
            if (window.matchMedia('(max-device-width:480px)').matches){
                $(".side-global-nav").append($(".header-menu.global-nav-menu"));
                $(".side-nav-wrapper .header-menu.global-nav-menu > button").remove();
                $(".side-global-nav").click(function(){
                   $(".header-menu.global-nav-menu").toggleClass("show-menu");
                });
            }
        /*--dynamically adjust position of the MORE button in user menu to be at the bottom of the menu--*/
            $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul+button')
            .css('margin-top', 65 + $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul').height());
        /*--expand the user menu and make the bottom border reappear (since the MORE button called for their removal)--*/
            $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul+button').click(function(e){
                e.preventDefault();
                e.stopPropagation();
                $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul').css(
                {'overflow-y':'initial','height':'initial','border':'1px solid #d8d8d8','border-bottom-left-radius':'4px','border-bottom-right-radius':'4px'});
                return false;
            });
      /*--close the menu when webpage is clicked and it is not on the menu--*/
            $(document).click(function(event) {
                if(!$(event.target).closest('.global-header > div.user-nav-wrapper > div.globalNav > div.header-menu.global-nav-menu > ul').length) {
                    $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul+button').css({
                        'margin-left': '-109px'
                    });
                        $('.global-header > div.user-nav-wrapper > div.global-nav > div.header-menu.global-nav-menu > ul').css({
                        'border-bottom-left-radius': "0",
                        'border-bottom-right-radius': "0",
                        'height':"343px",
                        'border-bottom':"none",
                        'overflow-y':"scroll"
                        });
                }
            });
    /*----------------------------------------------END Global Header Javascript Code ---------------------------------------------------------------------*/
    //Below is code to adjust Product name and Feature name for the Updated Login Page
    if($('.caLogin .product').siblings().size() > 0){
      $('.caLogin .product').css('margin-top','-8px');
    }
    if($('.caLogin .product').height() > 22 && $('.caLogin .product').siblings().size() == 0){
        $('.caLogin .product').css('margin-top','-8px');
    }

}(jQuery);
;/* ========================================================================
 * Bootstrap: scrollspy.js v3.3.5
 * http://getbootstrap.com/javascript/#scrollspy
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // SCROLLSPY CLASS DEFINITION
  // ==========================

  function ScrollSpy(element, options) {
    this.$body          = $(document.body)
    this.$scrollElement = $(element).is(document.body) ? $(window) : $(element)
    this.options        = $.extend({}, ScrollSpy.DEFAULTS, options)
    this.selector       = (this.options.target || '') + ' .nav li > a'
    this.offsets        = []
    this.targets        = []
    this.activeTarget   = null
    this.scrollHeight   = 0

    this.$scrollElement.on('scroll.bs.scrollspy', $.proxy(this.process, this))
    this.refresh()
    this.process()
  }

  ScrollSpy.VERSION  = '3.3.5'

  ScrollSpy.DEFAULTS = {
    offset: 10
  }

  ScrollSpy.prototype.getScrollHeight = function () {
    return this.$scrollElement[0].scrollHeight || Math.max(this.$body[0].scrollHeight, document.documentElement.scrollHeight)
  }

  ScrollSpy.prototype.refresh = function () {
    var that          = this
    var offsetMethod  = 'offset'
    var offsetBase    = 0

    this.offsets      = []
    this.targets      = []
    this.scrollHeight = this.getScrollHeight()

    if (!$.isWindow(this.$scrollElement[0])) {
      offsetMethod = 'position'
      offsetBase   = this.$scrollElement.scrollTop()
    }

    this.$body
      .find(this.selector)
      .map(function () {
        var $el   = $(this)
        var href  = $el.data('target') || $el.attr('href')
        var $href = /^#./.test(href) && $(href)

        return ($href
          && $href.length
          && $href.is(':visible')
          && [[$href[offsetMethod]().top + offsetBase, href]]) || null
      })
      .sort(function (a, b) { return a[0] - b[0] })
      .each(function () {
        that.offsets.push(this[0])
        that.targets.push(this[1])
      })
  }

  ScrollSpy.prototype.process = function () {
    var scrollTop    = this.$scrollElement.scrollTop() + this.options.offset
    var scrollHeight = this.getScrollHeight()
    var maxScroll    = this.options.offset + scrollHeight - this.$scrollElement.height()
    var offsets      = this.offsets
    var targets      = this.targets
    var activeTarget = this.activeTarget
    var i

    if (this.scrollHeight != scrollHeight) {
      this.refresh()
    }

    if (scrollTop >= maxScroll) {
      return activeTarget != (i = targets[targets.length - 1]) && this.activate(i)
    }

    if (activeTarget && scrollTop < offsets[0]) {
      this.activeTarget = null
      return this.clear()
    }

    for (i = offsets.length; i--;) {
      activeTarget != targets[i]
        && scrollTop >= offsets[i]
        && (offsets[i + 1] === undefined || scrollTop < offsets[i + 1])
        && this.activate(targets[i])
    }
  }

  ScrollSpy.prototype.activate = function (target) {
    this.activeTarget = target

    this.clear()

    var selector = this.selector +
      '[data-target="' + target + '"],' +
      this.selector + '[href="' + target + '"]'

    var active = $(selector)
      .parents('li')
      .addClass('active')

    if (active.parent('.dropdown-menu').length) {
      active = active
        .closest('li.dropdown')
        .addClass('active')
    }

    active.trigger('activate.bs.scrollspy')
  }

  ScrollSpy.prototype.clear = function () {
    $(this.selector)
      .parentsUntil(this.options.target, '.active')
      .removeClass('active')
  }


  // SCROLLSPY PLUGIN DEFINITION
  // ===========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.scrollspy')
      var options = typeof option == 'object' && option

      if (!data) $this.data('bs.scrollspy', (data = new ScrollSpy(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.scrollspy

  $.fn.scrollspy             = Plugin
  $.fn.scrollspy.Constructor = ScrollSpy


  // SCROLLSPY NO CONFLICT
  // =====================

  $.fn.scrollspy.noConflict = function () {
    $.fn.scrollspy = old
    return this
  }


  // SCROLLSPY DATA-API
  // ==================

  $(window).on('load.bs.scrollspy.data-api', function () {
    $('[data-spy="scroll"]').each(function () {
      var $spy = $(this)
      Plugin.call($spy, $spy.data())
    })
  })

}(jQuery);
;/**
 * Created by cacewit on 10/14/15.
 */
+function ($){
    'use strict';

    $.fn.initSelect = function() { //extend JQuery to allow users to initialize their own toggle buttons
        return new Select(this[0]);
    };
    var Select = function(t){
        this.el = t; //the element
        this.placeholder = ""; //placeholder text
        this.name = "";//the name oft he input
        this.required = false; //required
        this.initval = ""; //initial value
        this.initind = -1; //initial value's index
        this.v = ""; //value
        this.ind = -1; //index
        this.expanded = false; //is expanded
        this.init();//initialize
    }
    Select.prototype = {
        val: function(){
            return this.v;//return value
        },
        index: function(){//return index
            return this.ind;
        },
        init: function(){//initialize
            this.el.select = this;
            var el = $(this.el);
            this.menu = el.children(".select-menu");
            var menu = this.menu;
            this.placeholder = el.data('placeholder');
            this.name = el.data('input-name')
            this.required = el.data("required"); //add all the things that make it a drop menu and not a simple list.
            var str = "<label>"+this.placeholder+"</label>"+
                "<input class='hidden' readonly name='"+this.name+"' "+((this.required)?"required":"")+">"+
                "<span class='arrow'></span>";
            menu.before(str);


            var label = el.children("label")
            var input = el.children("input")
            this.setVals();
            var def = this.changeDefaultVal(el.data("val")) //set the default values
            if(def){
                this.v = this.initval;
                this.ind = this.initind;
                label.html(def.html());
                def.addClass("selected");
                el.data("val",this.initval);
                el.attr("data-val",this.initval);
            }

            var selectobj = el[0].select;
            menu.on("click.select","li",function(){ //click listener for menu items
                menu.children("li").eq(selectobj.ind).removeClass("selected");
                $(this).addClass("selected");
                selectobj.ind = $(this).index(); //set index
                selectobj.v = $(this).attr("value")?$(this).attr("value"):$(this).html(); //if no value set, use display field
                //label.html($(this).html());
                label.html($(this).html());
                el.data("val",selectobj.v);
                el.attr("data-val",selectobj.v);
                input.val(selectobj.v);
                label.addClass("touched"); //style change
            })
            el.on("click.select",function(e){ //click on menu
                e.stopPropagation();
                selectobj.expanded = !selectobj.expanded;
                $(menu).toggleClass("active");
                el.toggleClass("active");
            })
            $(document).on("click.select",function(e){
                if(e.target != el[0] && e.target != el.children(".arrow")[0] && selectobj.expanded){//click out
                    selectobj.expanded = false;
                    menu.removeClass("active");
                    el.removeClass("active")
                }
            })
            var form = el.closest("form");
            if(form){ //check for existing form, if so put in behavior on reset
                form.on("reset",function(){
                    console.log("test");
                    el[0].select.reset();
                })
            }
        },
        reset: function(){ //set values to initial and clear out all changes and stuff saved to the object beyond init vals
            this.v = this.initval;
            this.ind = this.initind;
            var el = $(this.el);
            var label = el.children("label");
            var input = el.children("input")
            //label.html(this.initmsg);
            label.removeClass("touched");
            if(this.v === "") label.html(this.placeholder);
            el.data("val",this.v);
            el.attr("data-val",this.v);
            input.val(this.v);
            var ind = this.ind;
            el.children(".select-menu").children().each(function(){
                $(this).removeClass("selected");
                if($(this).index() === ind){
                    $(this).addClass("selected");
                    label.html($(this).html());
                }

            })
        },
        setVals: function(){ //helper function to set value fields to the html if not specified
            this.menu.children().each(function(){
                if(!$(this).attr("value")) $(this).attr("value",$(this).html());
            })
        },
        //next few functions are meant for javascript manipulation of the element
        changeDefaultVal: function(newval){ //change the default value
            var selectobj = this;
            var el = $(this.el);
            if(!newval || newval === ""){ //if there isn't a newval or is empty string, initial value is nonexist
                selectobj.initval = "";
                selectobj.initind = -1;
                return;
            }
            else selectobj.initval = newval;
            this.menu.children().each(function(){ //check if any of the children are selected

                if($(this).attr("value") === selectobj.initval || (selectobj.initval < 0 && $(this).hasClass("selected"))){
                    selectobj.initind = $(this).index();
                    if(selectobj.initval < 0){
                        selectobj.initval = $(this).attr("value");
                    }
                    return $(this);
                }
            })
        },
        changeDefaultValByIndex: function(newIndex){ //change the default value by index
            var el = $(this.el);
            this.initind = newIndex;
            if(newIndex > -1){
                this.initval = this.menu.children().eq(newIndex).attr("value");
            }
            else{
                this.initval = "";
            }
        },
        changePlaceHolder: function(newPH){
            var el = $(this.el);
            this.placeholder = newPH;
            if(this.ind < 0){
                $(this.el).find("label").html(newPH);
            }
            el.data("placeholder",newPH)
            el.attr("data-placeholder",newPH)
        },
        changeInputName: function(newName){
            var el = $(this.el);
            this.name = newName;
            $(this.el).find("input").attr("name",newName);
            el.data("input-name",newName)
            el.attr("data-input-name",newName)
        },
        setRequired: function(req){
            var el = $(this.el);
            var input = el.children("input")[0];
            input.required = req;
            el.data("required",req);
            el.attr("data-required",req);
        }
    }

    var init_inDOM = function(){
        $('[data-select^="select"]').each(function(){
            var s = new Select(this);
        });
    }();
    //$(document).on('click.ca.togglebtn','[data-btn^="toggle-btn"]',clickHandler);
}(jQuery);
;/**
 * Created by cacewit on 9/30/15.
 */
+function ($){
    'use strict';
    var _init_shuttle_ = function(el){
        if(!el.data("shuttle") || el.data("shuttle") != "shuttle") return;
        var isOverAvailable = false;
        var isOverSelected = false;
        var isover = null;
        var dragging = false;
        var hasContainer = false;
        var fromAvailable = true;
        var availcontainer = el.find(".available");
        var selectcontainer = el.find(".selected");
        var right_arrow = el.find(".arrow-right");
        var right_all_arrow = el.find(".arrow-all-right");
        var left_arrow = el.find(".arrow-left");
        var left_all_arrow = el.find(".arrow-all-left");
        var highlight_block = null;
        var insert_position = "insert-bottom";
        if(document.querySelectorAll("#shuttle-dragcontainer").length === 0){
            console.log("test")
            $("body").append('<div id="shuttle-dragcontainer"></div>');
        }
        var scontain = $("#shuttle-dragcontainer");



        el.find('.available').parent().on({
                mouseenter:function(){
                    event.stopPropagation()
                    isover = "available"
                    isOverSelected = false;
                    isOverAvailable = true;
                    if(hasContainer) availcontainer.addClass("active");
                    showScroller(this);
                    //console.log(isover);
                },
                mouseleave:function(){
                    event.stopPropagation()
                    //isover = null;
                    //console.log(isover);
                    isOverAvailable = false;
                    availcontainer.removeClass("active");
                    $(this).find(".scrollup").addClass("hidden");
                    if(highlight_block){
                        highlight_block.removeClass("insert-top");
                        highlight_block.removeClass("insert-bottom");
                    }
                    highlight_block = null;
                },
            }

        );

        el.find('.selected').parent().on({
                mouseenter:function(){
                    event.stopPropagation()
                    isover = "selected";
                    //console.log(isover)
                    isOverAvailable = false;
                    isOverSelected = true;
                    if(hasContainer) selectcontainer.addClass("active");
                    showScroller(this);
                },
                mouseleave:function(){
                    event.stopPropagation()
                    //isover = null;
                    //console.log(isover);
                    isOverSelected = false;
                    selectcontainer.removeClass("active");
                    $(this).find(".scrollup").addClass("hidden");
                    if(highlight_block){
                        highlight_block.removeClass("insert-top");
                        highlight_block.removeClass("insert-bottom");
                    }
                    highlight_block = null;
                }
            }
        );

        el.find('.content-blocks').on(
            'scroll',function(){
                showScroller($(this).parent()[0]);
            }
        );

        var findinsertposition = function(e, block){
            var y = e.pageY - block.offset().top;
            var blockheight = block.height();
            if(y < blockheight/2) return "insert-top";
            else return "insert-bottom";
        }

        el.on('click','.scrollup',function(){
            $(this).parent().find(".content-blocks").eq(0).scrollTop(0);
        }).on('mouseenter', ".content-blocks .block", function(e){
            if(hasContainer){
                //if(highlight_block)highlight_block.removeClass("insert");
                highlight_block = $(this);
                insert_position = findinsertposition(e, highlight_block)
                $(this).addClass(insert_position);
            }
        }).on('mousemove', ".content-blocks .block", function(e){
            if(hasContainer){
                //if(highlight_block)highlight_block.removeClass("insert");
                if(highlight_block){
                    insert_position = findinsertposition(e, highlight_block);
                    if(insert_position === "insert-top")highlight_block.removeClass("insert-bottom");
                    else highlight_block.removeClass("insert-top");
                    highlight_block.addClass(insert_position);
                }
            }
        }).on("mouseleave", ".content-blocks .block", function(){
            if(highlight_block){
                highlight_block.removeClass("insert-top");
                highlight_block.removeClass("insert-bottom");
            }
            highlight_block = null;
        })




        var check = function(t){
            t.addClass("checked");
            t.find('input').prop('checked', true);
        }

        var uncheck = function(t){
            t.removeClass("checked");
            t.find('input').prop('checked', false);
        }
        /*
        var checkAll = function(checkbox, containerclass){
            if(checkbox.hasClass("checked")){
                el.find(containerclass).children().each(function(){
                    check($(this));
                });
            }
            else{
                el.find(containerclass).children().each(function(){
                    uncheck($(this));
                });
            }
        }*/
        var checkAll = function(containerclass){
            el.find(containerclass).children().each(function(){
                check($(this));
            });

        }

        var _init_arrow_ = function(){
            var shift = function (classfrom,classto,all){
                el.find(classfrom).children(".block").each(function(){
                    var block = $(this);
                    if (all || block.hasClass("checked")){
                        block.appendTo(el.find(classto));
                        uncheck(block);
                    }
                });
                handleSelectedEmpty();
                handleAvailEmpty();
                //if(classfrom === ".available") uncheck(el.find(".avail-header"));
                //else if(classfrom === ".selected") uncheck(el.find(".select-header"));
            };

            right_arrow.click(function(){
                if(!$(this).hasClass("disabled")){
                    shift(".available",".selected",false);
                }
            });
            left_arrow.click(function(){
                if(!$(this).hasClass("disabled")){
                    shift(".selected",".available",false);
                }
            });
            right_all_arrow.click(function(){
                if(!$(this).hasClass("disabled")){
                    shift(".available",".selected",true);
                }
            });
            left_all_arrow.click(function(){
                if(!$(this).hasClass("disabled")){
                    shift(".selected",".available",true);
                }
            });
        };

        _init_arrow_();

        var _init_block_ = function(t){
            t.click(function(){
                console.log("CLICKING")
                var l = t.find('input:checked').length > 0 &&  t.find('input').prop('checked');
                if(l){
                    check($(this));
                }
                else{
                    uncheck($(this));
                }
                /*
                if(t.hasClass("avail-header")){
                    checkAll(t,".available");
                }
                if(t.hasClass("select-header")){
                    checkAll(t,".selected");
                }*/
            });
            _make_draggable_(t);
        }
        var _make_draggable_ = function(t){
            t.on("mousedown",function(){
                if(isOverAvailable){
                    console.log("MOUSEDOWN")
                    if(!dragging){
                        fromAvailable = true;
                        //t.addClass("checked");
                        dragging = true;
                    }
                }
                else if(isOverSelected){
                    console.log("MOUSEDOWN")
                    if(!dragging){
                        fromAvailable = false;
                        //t.addClass("checked");
                        dragging = true;
                    }
                }
            })
                .on("mouseup",function(e){
                    dragging = false;
                })
                .on("mousemove",function(e){
                    var top = e.pageY;
                    var left = e.pageX;
                    e.preventDefault();
                    if(dragging){
                        if(!t.hasClass('dragging')){
                            hasContainer = true;
                            scontain.css("top",top+"px");
                            scontain.css("left",(left)+"px");
                            if(!t.find("input").prop("checked"))t.find('input').prop('checked', true);

                            if(fromAvailable && t.parent().hasClass("available") ||
                                !fromAvailable && t.parent().hasClass("selected")){
                                t.parent().children(".checked").each(function(){
                                    var s = $(this);
                                    s.addClass('dragging');
                                }) //firs run to the the order since then comes removal
                            }
                            t.parent().children(".dragging").each(function(){
                                var s = $(this);
                                s.appendTo(scontain);
                            })
                        }
                    }
                })

            $(document).on("mousemove.shuttle.container",function(e){
                var top = e.pageY;
                var left = e.pageX;
                e.preventDefault();
                if(dragging && hasContainer){
                    //var scontain= $("#shuttle-dragcontainer");
                    scontain.removeClass("hidden");
                    scontain.css("top",top+"px");
                    scontain.css("left",left+"px");

                    el.find(".available").children().each(function(){
                        $(this).find('input').prop('checked', false);
                    })
                    el.find(".selected").children().each(function(){
                        $(this).find('input').prop('checked', false);
                    })

                }
            })
                .on("mouseup.shuttle.container",function(e){
                    if(hasContainer){
                        var scontain= $("#shuttle-dragcontainer");
                        var container = "";
                        var removeselection = false;
                        /*if(isOverAvailable && fromAvailable || isOverSelected && !fromAvailable){
                            container = (fromAvailable)?".available":".selected"
                        }
                        else{
                           container = (fromAvailable)?".selected":".available"
                            removeselection = true;
                        }*/
                        if(isover) container = "."+isover;
                        if(container.length === 0){
                            container = (fromAvailable)?".selected":".available";
                        }
                        console.log(container);
                        removeselection = true;

                        var cont = el.find(container);
                        var items2add = scontain.children();
                        if (highlight_block) {
                            var target_ele = highlight_block;
                            items2add.each(function(){
                                var item = $(this);
                                item.removeClass("dragging");
                                if(removeselection){
                                    item.removeClass("checked");
                                    item.find('input').prop('checked', false);
                                }
                                if(insert_position === "insert-top") item.insertBefore(highlight_block);
                                else item.insertAfter(target_ele);
                                target_ele = item;
                            })
                            highlight_block.removeClass("insert-top");
                            highlight_block.removeClass("insert-bottom");
                            highlight_block = null;
                        }
                        else{
                            items2add.each(function(){
                                var item = $(this);
                                item.removeClass("dragging");
                                if(removeselection){
                                    item.removeClass("checked");
                                    item.find('input').prop('checked', false);
                                }
                                item.appendTo(cont);
                            })
                        }

                        /*
                        if(removeselection){
                            var avail = el.find(".avail-header");
                            var sel = el.find(".select-header");
                            avail.removeClass("checked");
                            sel.find('input').prop('checked', false);
                        }*/


                        scontain.addClass("hidden");
                        //$("#shuttle-dragcontainer").remove();
                        handleSelectedEmpty();
                        handleAvailEmpty();
                        if(isOverAvailable) availcontainer.removeClass("active");
                        if(isOverSelected) selectcontainer.removeClass("active");
                        dragging = false;
                        hasContainer = false;
                        fromAvailable = true;

                    }
                })
        }

        el.find(".availsearch").on("keyup.search",function(e){
            var t = $(e.target);
            //var searchcontainer = (t.hasClass("availsearch"))?availcontainer:selectcontainer;

            var items = availcontainer.children();
            search(t.val(), items);
        })

        el.find(".selectedsearch").on("keyup.search",function(e){
            var t = $(e.target);
            //var searchcontainer = (t.hasClass("availsearch"))?availcontainer:selectcontainer;

            var items = selectcontainer.children();
            search(t.val(), items);
        })

        el.find(".block").each(function(){
            var block = $(this);
            _init_block_(block);
        });

        var handleAvailEmpty = function(){
            var avail = el.find(".available");
            if(avail.children(".block").length === 0){
                right_arrow.addClass("disabled");
                right_all_arrow.addClass("disabled");
            }
            else{
                right_arrow.removeClass("disabled");
                right_all_arrow.removeClass("disabled");

            }

        }

        var handleSelectedEmpty = function(){
            var selected = el.find(".selected");
            if(selected.children(".block").length === 0){
                selected.addClass("empty")
                left_arrow.addClass("disabled");
                left_all_arrow.addClass("disabled");
            }
            else {
                selected.removeClass("empty");
                left_arrow.removeClass("disabled");
                left_all_arrow.removeClass("disabled");
            }
        }

        var checkContainerContents = function(){
            var check = function(classname,classheadername){
                var cont = $(classname);
                if(cont.scrollHeight>cont.clientHeight){
                    $(classheadername).prev().removeClass("hidden");
                }
            }
            handleSelectedEmpty();
            handleAvailEmpty();
        }

        checkContainerContents();

        var showScroller = function(ele){
            var contb = $(ele).find(".content-blocks").eq(0);
            var scrollpos = contb.scrollTop();
            if(contb[0].scrollHeight > contb.height() && scrollpos > 0){
                $(ele).find(".scrollup").removeClass("hidden");
            }
            else{
                $(ele).find(".scrollup").addClass("hidden");
            }
        }

    }

    var search = function(searchstring, itemlist){
        //console.log(searchstring, itemlist);
        itemlist.each(function(){
            var t = $(this);
            var text = t.find("span").html().toLowerCase();
            if(text.indexOf(searchstring.toLowerCase())>-1){
                t.removeClass("hidden");
            }
            else{
                t.addClass("hidden");
            }
        })
    }
    var _init_in_DOM = function(){
        $('[data-shuttle^="shuttle"]').each(function(){
            var t = $(this);
            _init_shuttle_(t);
        });
    }();
    //$(document).on('click.ca.togglebtn','[data-btn^="toggle-btn"]',clickHandler);
}(jQuery);;/*==============================================
 * slider, created by Yiou - 6/6/2016
 * =============================================*/

(function ($) {
  'use strict';

  //-----------------------------helper functions------------------------------------
  //return true if the value is in between the two value(inclusive). otherwise return false

  function isBetween(value, endpoint1, endpoint2) {
    return (endpoint1 <= value && value <= endpoint2) || (endpoint2 <= value && value <= endpoint1);
  }
  //clamp the value to be within the range
  function clamp(value, endpoint1, endpoint2) {
    if ((value > endpoint1) && (value > endpoint2)) { return Math.max(endpoint1, endpoint2); }
    if ((value < endpoint1) && (value < endpoint2)) { return Math.min(endpoint1, endpoint2); }
    return value;
  }
  //return the value snapped to the beginning and end of each interval.
  //interval: length of interval
  function snap(value, endpoint1, endpoint2, interval) {
    value = clamp(value, endpoint1, endpoint2);
    return Math.round((value - endpoint1) / interval) * interval + endpoint1;
  }
  //pause event propergation
  function pauseEvent(e) {
    if (e.stopPropagation) { e.stopPropagation(); }
    if (e.preventDefault) { e.preventDefault(); }
  }
  //return true if the parent has this child, otherwise return false
  function parentHasSelector(parent, selector) {
    return Boolean(parent.querySelector(selector));
  }

  // //trigger custom Jquery event, pass value together with the event object
  // function triggerValueEvent(slider, event) {
  //   slider.$el.trigger($.Event(event), {
  //     handle: slider.$el,
  //     values: slider.values//TODO: pass slider object instead of values
  //   });
  // }

  // //dont need to use this function.
  // //trigger custom Jquery event
  // function triggerNonValueEvent(slider, event) {
  //   slider.$el.trigger($.Event(event), {
  //     handle: slider.$el
  //   });
  // }

  function triggerEvent(slider, event) {
    slider.$el.trigger($.Event(event), {
      handle: slider.$el,
      slider: slider
    });
  }

  //repeat a function multiple times.
  //the function is of the form (index, lastValue)=>result
  function times(count, func, initValue) {
    for (var i = 0; i < count; i++) {
      initValue = func(i, initValue);
    }
    return initValue;
  }

  function isNumber(num) {
    return isFinite(num);
  }

  //convert num to a finite, valid number, if failed, use defaultNum,
  //if defaultNum is not provided, default to 0
  function sanitizeNumber(unsafeValue, defaultNum) {
    if (typeof defaultNum === 'undefined') {
      return isNumber(unsafeValue) ? Number(unsafeValue) : 0;
    } else {
      return isNumber(unsafeValue) ? Number(unsafeValue) : Number(defaultNum);
    }
  }

  function tagToBoolean(tagValue) {
    if (typeof tagValue === 'string') {
      return (tagValue === 'true') ||
        (tagValue === '') ||
        (tagValue !== 'false');
    } else {
      return Boolean(tagValue);
    }
  }


  //-------------------------------end helper functions----------------------------------



  //---------------------------Slider Class---------------------------
  var Slider = function (element, options) {

    this.$el = $(element);

    initSliderOptions(this, options);
    initSliderEventListeners(this, options);
    initSliderMarkup(this, options);
    initSliderThumbs(this);
    initSliderInnerEvents(this);

  };//end class constructor

  //---------------------------Slider initialization functions---------------------------
  function initSliderOptions(slider, options) {
    slider.min = sanitizeNumber(options.min, 0);
    slider.max = sanitizeNumber(options.max, 1);
    slider.interval = sanitizeNumber(options.interval, 0);
    slider.disabled = tagToBoolean(options.disabled);
    slider.isRange = tagToBoolean(options.isRange);
    slider.values = options.values || [];
    //user can use "data-value" attribute when it's a single thumb. 
    //Internally, we use the same data structure (an array) to accommondate both single thumb and multi thumb
    if (isNumber(options.value)) {
      slider.values[0] = sanitizeNumber(options.value);
    }

    // slider.randomAccess=options.randomAccess;
  }

  function initSliderEventListeners(slider, options) {
    slider._draggedThumbIndex = 0;
    slider._startDragX = 0;
    slider._startDragPos = slider.min;
    slider._isDragging = false;

    if (options.change) {
      slider.$el.on('slidechange', options.change);
    }
    if (options.create) {
      slider.$el.on('slidecreate', options.create);
    }
    if (options.slide) {
      slider.$el.on('slide', options.slide);
    }
    if (options.start) {
      slider.$el.on('slidestart', options.start);
    }
    if (options.stop) {
      slider.$el.on('slidestop', options.stop);
    }
  }

  function initSliderMarkup(slider, options) {
    function isValidHtml(element) {
      return parentHasSelector(element, '.slider-main') ||
        parentHasSelector(element, '.slider-track') ||
        parentHasSelector(element, '.slider-track-range');
    }

    //check if the markup contain all the required elements
    if (!isValidHtml(slider.$el[0])) {
      slider.initHTML(options);
    }

    slider.thumbs = slider.$el.find('.slider-thumb');

    slider.isRange = (slider.thumbs.length > 1);


    //change the position of thumbs based on values array. If values are not defined, try the data-pos attribute, If not present either, use min by default.
    slider.thumbs.each(function (index, thumb) {
      var dataIsFromAttribute = false;
      if (!isNumber(slider.values[index])) {
        var attrValue = $(thumb).attr('data-pos');
        if (isNumber(attrValue)) {
          dataIsFromAttribute = true;
          slider.values[index] = sanitizeNumber(attrValue);
        } else {
          slider.values[index] = slider.min;//no data from either the values array nor thumb attribute, use the min
        }
      }
      //if didn't get data from the attribute, set the attribute of the thumb
      if (!dataIsFromAttribute) {
        $(thumb).attr('data-pos', slider.values[index]);
      }
    });//end thumbs.each


    slider.apply();
  }

  function initSliderThumbs(slider) {
    slider.$el.find('.slider-thumb').each(function (index) {
      slider.setThumbPos(index, slider.getThumbPos(index));
    });
  }

  function initSliderInnerEvents(slider) {
    //windows resize
    $(window).resize(function (e) {
      slider.apply();
    });

    //mouse event
    slider.$el.mousedown(function (e) {

      var $target = $(e.target);
      if (!slider.disabled && !slider._isDragging) {

        if ($target.hasClass('slider-thumb')) {

          slider._isDragging = true;
          slider._draggedThumbIndex = slider.getThumbIndex(e.target);
          slider._startDragX = e.clientX;
          slider._startDragPos = sanitizeNumber($target.attr('data-pos'), 0);
          triggerEvent(slider, 'sliderstart');
          //dont pauseEvent here because we want the hightlight box
        }
        // }else if (slider.randomAccess && $target.hasClass('slider-track') && ! slider._isDragging){
        //   slider.setThumbX(slider._draggedThumbIndex, e.clientX);
        //   slider._isDragging=true;
        //   slider._startDragX=e.clientX;
        //   slider._startDragPos=slider.getThumbPos(slider._draggedThumbIndex);
        //   return pauseEvent(e);
        // }
      }
    });
    $(window).mouseup(function (e) {
      if (slider._isDragging) {
        triggerEvent(slider, 'slidestop');
      }
      slider._isDragging = false;
    });
    $(window).mousemove(function (e) {

      if (slider._isDragging && !slider.disabled) {

        var newPos = 0;
        var calculatedPos = slider._startDragPos + slider.pixelToDistance(e.clientX - slider._startDragX);
        if (slider.interval !== 0) {
          //snap
          newPos = snap(calculatedPos, slider.min, slider.max, slider.interval);
        } else {
          //not snapping
          newPos = calculatedPos;
        }
        slider.setThumbPos(slider._draggedThumbIndex, newPos);
        triggerEvent(slider, 'slide');
        pauseEvent(e);
        return false;
      }
    });
    //keyboard event
    slider.thumbs.keydown(function (e) {
      if (!slider.disabled) {
        var interval = ((slider.interval === 0) ? ((slider.max - slider.min) / 100) : slider.interval);
        var LEFT_KEY = 37;
        var RIGTH_KEY = 39;
        switch (e.keyCode) {
          case LEFT_KEY://key press left arrow 
            slider.incThumbPos(slider.getThumbIndex(e.target), -interval);
            pauseEvent(e);
            return false;
          case RIGTH_KEY://key press right arrow
            slider.incThumbPos(slider.getThumbIndex(e.target), interval);
            pauseEvent(e);
            return false;
          default:
            return true;
        }
      }
    });
  }

  //-------------------------------------slider class functions--------------------------------
  //get the total range of the slider.
  //ex. if the start is 10 and max is 25, the range is 15
  Slider.prototype.getTotalRange = function () {
    return Math.abs(this.max - this.min);
  };


  //get the position of the thumb. The position is based on the begin and max of the slider.
  //ex. the thumb in the middle of a slider with min of 20 and max of 30 with return a position of 25.
  //position can be specified in a declarative way using "data-pos" attribute in the "slider-thumb" element
  //if invalid value is return from element attribute, return slider.min
  Slider.prototype.getThumbPos = function (index) {
    var pos = sanitizeNumber($(this.thumbs[index]).attr('data-pos'), this.min);
    // pos=clamp(pos, this.min, this.max);
    // $(this.thumbs[index]).attr('data-pos',pos);
    return pos;
  };

  //set the position of the thumb. The position is the one returned from getThumbPos
  Slider.prototype.setThumbPos = function (index, pos) {
    var clampedPos = clamp(pos, this.min, this.max);
    this.thumbs = this.$el.find('.slider-thumb');

    this.values[index] = clampedPos;

    var target = this.thumbs[index];
    if (target) {
      $(target).attr('data-pos', clampedPos);
      this.apply();
    }
    triggerEvent(this, 'slidechange');
  };

  //change the pos of the thumb
  //diff could be positive or negative
  Slider.prototype.incThumbPos = function (index, diff) {
    this.setThumbPos(index, this.getThumbPos(index) + diff);
  };

  //set the physical x coordinate of the thumb at index to clientX.
  //if clientX is outside of the range of the track. clamp it so that thumb stays on the track
  Slider.prototype.setThumbX = function (index, clientX) {
    var targetThumb = this.thumbs[index];
    var bounding = targetThumb.getBoundingClientRect();
    var diff = this.pixelToDistance(clientX - (bounding.left + bounding.right) / 2);
    this.incThumbPos(index, diff);
  };

  //get the index of the thumb. the order of the thumb is the same as the order they are listed in html
  //the index doesn't necessarily match the relative position of the thumbs in the track.
  Slider.prototype.getThumbIndex = function (thumb) {
    return this.thumbs.index(thumb);
  };


  //convert the pixel distance to the distance in position
  Slider.prototype.pixelToDistance = function (px) {
    return px / this.actualLength * (this.max - this.min);
  };

  //apply data and rerender the slider graphics
  //call this function everytime the internal data change or needs to rernder the slider (ex. window size changed)
  //This function should not modify the internal data, it should only change the appearance of the slider.
  //thus, it assumes the data to be correct. Call other functions to verify data before calling this function.
  Slider.prototype.apply = function () {
    //get thumb position
    var slider = this;
    //The length of the available slider track (excluding the start and end padding) in pixels.
    this.actualLength = this.$el.find('.slider-main').outerWidth() - this.thumbs.outerWidth();
    this.thumbs = this.$el.find('.slider-thumb');
    if (slider.$el.attr('disabled')) {
      slider.disable();
      return;
    }

    //update the thumbs' positions
    slider.thumbs.each(function (index, thumb) {
      var margin_left = slider.actualLength * (slider.getThumbPos(index) - slider.min) / (slider.max - slider.min);
      $(thumb).css('margin-left', margin_left + 'px');
      thumb.at = slider.getThumbPos(index);
    });

    //update the selection bar
    if (slider.thumbs.length > 0) {
      //position of right  end of the selection bar.
      var right = slider.getThumbPos(0);
      right = clamp(right, slider.min, slider.max);
      //position of the left end of the selection bar.
      var left = slider.min;
      if (slider.thumbs[1]) {
        left = slider.getThumbPos(1);
        left = clamp(left, slider.min, slider.max);
      }
      //if (min - max) and (left - right) are differen sign, swap left and right.
      if ((slider.min - slider.max) * (left - right) < 0) {
        var temp = right;
        right = left;
        left = temp;
      }
      var rangeLeftX = (left - slider.min) / (slider.max - slider.min) * slider.actualLength;
      var rangeWidthX = Math.abs(right - left) / Math.abs(slider.max - slider.min) * slider.actualLength;

      slider.$el.find('.slider-track-range').
        css('margin-left', '' + rangeLeftX + 'px').
        css('width', '' + rangeWidthX + 'px');

      //toggle highlight for labels within range
      var labels = slider.$el.find('.slider-label');
      var labelTotal = labels.length;
      labels.each(
        function (index, element) {
          var labelPos = index / (labelTotal - 1) * (slider.max - slider.min) + slider.min;
          if (isBetween(labelPos, left, right)) {
            $(element).removeClass('inactive');
          } else {
            $(element).addClass('inactive');
          }
        }
      );
    }//end update selection bar
  };



  //create a nicely formated object file containing all the data
  Slider.prototype.getData = function () {
    return {
      min: this.min,
      max: this.max,
      interval: this.interval,
      values: this.values,
      isRange: this.isRange
    };
  };
  Slider.prototype._addThumb = function (value) {
    this.$el.find('.slider-main').append('<button type="button" class="slider-thumb" data-pos="' + value + '"></button>');
    this.setThumbPos(this.$el.find('.slider-thumb').length - 1, value);
  };
  //fill in the html markup if the user use the simplified markup.
  //If you want to use this function, call it before apply() so that apply could apply visual changes
  //This function should only be called once because it will override some of the existing elements.
  Slider.prototype.initHTML = function (options) {
    var $el = this.$el;
    $el.html(
      '<div class="slider-main">' +
      '<div class="slider-track">' +
      '<div class="slider-track-range"></div>' +
      '</div>' +
      '</div>');
    if (options.tagMin) {
      $el.prepend('<span class="slider-tag">' + options.tagMin + '</span>');
    }
    if (options.tagMax) {
      $el.append('<span class="slider-tag">' + options.tagMax + '</span>');
    }

    if (this.isRange === true) {
      this._addThumb(options.values[0]);
      this._addThumb(options.values[1]);
    } else {
      this._addThumb(options.values[0]);
    }


    if (options.mark) {
      $el.find('.slider-main').append('<div class="slider-marks"></div>');
      var markContainer = $el.find('.slider-marks');
      var labelEnabled = false;
      var labelContainer;

      function isLabelEnabled(options) {
        return options.labels === 'true' ||
          options.labels === '' ||
          options.labels === true ||
          options.labels === 'data-label' ||
          $.isArray(options.labels);
      }
      if (isLabelEnabled(options)) {
        labelEnabled = true;
        $el.find('.slider-main').append('<div class="slider-labels"></div>');
        labelContainer = $el.find('.slider-labels');
      }

      var markTotal = Math.round(sanitizeNumber(options.mark, 0));
      var miniMark = sanitizeNumber(options.miniMark, 0);
      for (var i = 0; i < markTotal; i++) {
        markContainer.append('<span class="slider-long-mark" data-label=' + i + '></span>');
        if (miniMark && (i < markTotal - 1)) {
          for (var j = 0; j < miniMark; j++) {
            markContainer.append('<span class="slider-short-mark"></span>');
          }
        }
      }

      if (labelEnabled) {
        if ($.isArray(options.labels)) {
          options.labels.forEach(function (label) {
            labelContainer.append('<span class="slider-label" data-label="' + label + '"></span>');
          });
        } else {
          times(options.mark || 0,
            function (index, labelC) {
              labelC.append('<span class="slider-label" data-label="' + ((options.mark || 0) - index) + '"></span>');
              return labelC;
            },
            labelContainer);
        }
      }

    }
  }; //initHTML

  //-----------------------------Slider control function---------------------------------------
  //The following functions are open to user. User can call them to modify the slider.
  //-----------------------------------------------------------------------------------------
  //toggle the slider, if not disabled, disable it. otherwise enable it
  Slider.prototype.toggle = function () {
    this.$el.toggleClass('disabled');
    this.disabled = this.$el.hasClass('disabled');
    if (this.disabled) {
      this.$el.attr('disabled', 'true');
    } else {
      this.$el.attr('disabled', null);
    }
    return {
      data: null,
      shouldContinue: true
    };
  };

  //disable the slider, if already disabled, do nothing
  Slider.prototype.disable = function () {
    if (!this.$el.hasClass('disabled')) {
      this.toggle();
    }
    return {
      data: null,
      shouldContinue: true
    };
  };

  //enable the slider, if already enabled, do nothing
  Slider.prototype.enable = function () {
    if (this.$el.hasClass('disabled')) {
      this.toggle();
    }
    return {
      data: null,
      shouldContinue: true
    };
  };

  //update the positions of the thumb in a batch by passing in an array of numbers.
  //If more values are given than the number of thumbs, only the values in the beginning will be used.
  //if less is passed, the thumbs declared in the begining will be updated.
  Slider.prototype.updateThumbs = function (values) {
    var thumbTotal = this.thumbs.length;
    for (var i = 0; i < thumbTotal; i++) {
      var value = values ? values[i] : this.getThumbPos(i);
      this.setThumbPos(i, value);
    }
  };

  //use this function to get or set the data of slider
  Slider.prototype.option = function () {
    var slider = this;
    if (arguments.length === 0) {
      //get an object containing essential data
      //example  .caSlider('option')
      return {
        data: this.getData(),
        shouldContinue: false
      };
    } else if (arguments.length === 1) {
      //getter
      //example  .caSlider('option','min')
      var data = null;
      switch (arguments[0]) {
        case 'min':
        case 'max':
        case 'interval':
        case 'isRange':
        case 'values': data = this[arguments[0]]; break;
        case 'value': data = this.values[0]; break;
        default:
          console.error('slider doesn\'t have this option: ' + arguments[0]);
          data = null;
          break;
      }
      return {
        data: data,
        shouldContinue: false
      };
    } else {
      //setter
      //example  .caSlider('option','min',20);
      switch (arguments[0]) {
        case 'min':
        case 'max':
        case 'interval':
          slider[arguments[0]] = arguments[1];
          slider.$el.attr('data-' + arguments[0], arguments[1]);
          slider.updateThumbs();
          break;
        case 'isRange':
          slider.isRange = arguments[1];
          //if set to range selection but only has one thumb or less, add thumbs
          if (slider.isRange === true && slider.thumbs.length <= 1) {
            var thumbTotal = slider.thumbs.length || 0;
            while (thumbTotal < 2) {
              slider._addThumb(0);
              thumbTotal++;
            }

            //if set to single thumb, but currently has more than one thumbs. remove thumbs
          } else if (slider.isRange === false && slider.thumbs.length > 1) {
            //remove slider
            while (slider.thumbs.length > 1) {
              slider.thumbs.last().remove();
              slider.thumbs = slider.$el.find('slider-thumb');
            }
          }
          break;
        case 'value':
          //change the first thumbs position,
          if (slider.thumbs[0]) {
            slider.setThumbPos(0, arguments[1]);
          }
          break;
        case 'values':
          slider.updateThumbs(arguments[1]);
          break;
        default:
          console.error('slider doesn\'t have this option: ' + arguments[0]);
          break;
      }//case
      slider.apply();
      return {
        shouldContinue: true
      };
    }
  };

  //---------------------------------end Slider control functions----------------------


  //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;Beautiful seperator;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  //    $$$$$\                                                                     $$\                $$$$$$\   $$$$$$\   //
  //    \__$$ |                                                                    $$ |              $$  __$$\ $$  __$$\  //
  //       $$ | $$$$$$\  $$\   $$\  $$$$$$\   $$$$$$\  $$\   $$\        $$$$$$$\ $$$$$$\   $$\   $$\ $$ /  \__|$$ /  \__| //
  //       $$ |$$  __$$\ $$ |  $$ |$$  __$$\ $$  __$$\ $$ |  $$ |      $$  _____|\_$$  _|  $$ |  $$ |$$$$\     $$$$\      //
  // $$\   $$ |$$ /  $$ |$$ |  $$ |$$$$$$$$ |$$ |  \__|$$ |  $$ |      \$$$$$$\    $$ |    $$ |  $$ |$$  _|    $$  _|     //
  // $$ |  $$ |$$ |  $$ |$$ |  $$ |$$   ____|$$ |      $$ |  $$ |       \____$$\   $$ |$$\ $$ |  $$ |$$ |      $$ |       //
  // \$$$$$$  |\$$$$$$$ |\$$$$$$  |\$$$$$$$\ $$ |      \$$$$$$$ |      $$$$$$$  |  \$$$$  |\$$$$$$  |$$ |      $$ |       //
  //  \______/  \____$$ | \______/  \_______|\__|       \____$$ |      \_______/    \____/  \______/ \__|      \__|       //
  //                 $$ |                              $$\   $$ |                                                         //
  //                 $$ |                              \$$$$$$  |                                                         //
  //                 \__|                               \______/                                                          //
  //;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

  //use this to convert a html element to slider
  function Plugin(option) {
    if (this.data('slider')) {
      //slider already intialized
      if (typeof option === 'undefined') {
        return this.data('slider');
      }
      var args = Array.prototype.slice.call(arguments);
      args.shift();
      var returnData = null;
      var result = null;
      this.each(function (index, element) {
        switch (args.length) {
          case 0: result = $(element).data('slider')['' + option](); break;
          case 1: result = $(element).data('slider')['' + option](args[0]); break;
          case 2: result = $(element).data('slider')['' + option](args[0], args[1]); break;
        }

        returnData = result.data;
        return result.shouldContinue;
      });
      return returnData;
    } else {
      //slider is not created, create the slider
      return this.each(function () {
        var $this = $(this);
        var slider = $this.data('slider');

        if (!slider && ((typeof option === 'object') || (typeof option === 'undefined'))) {
          $this.addClass('ca-slider');
          slider = new Slider(this, $.extend(true, {}, $.fn.caSlider.default, option || {}, $this.data()));
          $this.data('slider', slider);
          triggerEvent(slider, 'slidecreate');
          return $this;
        }
      });
    }

  }

  var old = $.fn.slider;

  $.fn.caSlider = Plugin;
  $.fn.caSlider.Constructor = Slider;
  $.fn.caSlider.default = {
    min: 0,
    max: 1,
    interval: 0,
    disabled: false,
    values: [],
    //type: slidechange
    change: null,//function(event,ui){},
    //type: slidecreate
    create: null,//function(event,ui){},//the ui object is empty but included for consistency with other events.
    //type: slide
    slide: null,//function(event, ui){},
    //type: slidestart
    start: null,//function(event, ui){},
    //type: slidestop
    stop: null//function(event, ui){},
  };

  // SLIDER NO CONFLICT
  // ==================

  $.fn.caSlider.noConflict = function () {
    $.fn.caSlider = old;
    return this;
  };

  //upgrade all slider elements. --This is used for testing only. If you want to use this plugin,
  //you should use $(selector).caSlider();
  $(window).on('load', function () {
    $('[data-slider-autoconvert]').each(function () {
      var $slider = $(this);
      Plugin.call($slider);
    });
  });
} (jQuery));
;/* ========================================================================
 * Bootstrap: tab.js v3.3.5
 * http://getbootstrap.com/javascript/#tabs
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // TAB CLASS DEFINITION
  // ====================

  var Tab = function (element) {
    // jscs:disable requireDollarBeforejQueryAssignment
    this.element = $(element)
    // jscs:enable requireDollarBeforejQueryAssignment
  }

  Tab.VERSION = '3.3.5'

  Tab.TRANSITION_DURATION = 150

  Tab.prototype.show = function () {
    var $this    = this.element
    var $ul      = $this.closest('ul:not(.dropdown-menu)')
    var selector = $this.data('target')

    if (!selector) {
      selector = $this.attr('href')
      selector = selector && selector.replace(/.*(?=#[^\s]*$)/, '') // strip for ie7
    }

    if ($this.parent('li').hasClass('active')) return

    var $previous = $ul.find('.active:last a')
    var hideEvent = $.Event('hide.bs.tab', {
      relatedTarget: $this[0]
    })
    var showEvent = $.Event('show.bs.tab', {
      relatedTarget: $previous[0]
    })

    $previous.trigger(hideEvent)
    $this.trigger(showEvent)

    if (showEvent.isDefaultPrevented() || hideEvent.isDefaultPrevented()) return

    var $target = $(selector)

    this.activate($this.closest('li'), $ul)
    this.activate($target, $target.parent(), function () {
      $previous.trigger({
        type: 'hidden.bs.tab',
        relatedTarget: $this[0]
      })
      $this.trigger({
        type: 'shown.bs.tab',
        relatedTarget: $previous[0]
      })
    })
  }

  Tab.prototype.activate = function (element, container, callback) {
    var $active    = container.find('> .active')
    var transition = callback
      && $.support.transition
      && ($active.length && $active.hasClass('fade') || !!container.find('> .fade').length)

    function next() {
      $active
        .removeClass('active')
        .find('> .dropdown-menu > .active')
          .removeClass('active')
        .end()
        .find('[data-toggle="tab"]')
          .attr('aria-expanded', false)

      element
        .addClass('active')
        .find('[data-toggle="tab"]')
          .attr('aria-expanded', true)

      if (transition) {
        element[0].offsetWidth // reflow for transition
        element.addClass('in')
      } else {
        element.removeClass('fade')
      }

      if (element.parent('.dropdown-menu').length) {
        element
          .closest('li.dropdown')
            .addClass('active')
          .end()
          .find('[data-toggle="tab"]')
            .attr('aria-expanded', true)
      }

      callback && callback()
    }

    $active.length && transition ?
      $active
        .one('bsTransitionEnd', next)
        .emulateTransitionEnd(Tab.TRANSITION_DURATION) :
      next()

    $active.removeClass('in')
  }


  // TAB PLUGIN DEFINITION
  // =====================

  function Plugin(option) {
    return this.each(function () {
      var $this = $(this)
      var data  = $this.data('bs.tab')

      if (!data) $this.data('bs.tab', (data = new Tab(this)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.tab

  $.fn.tab             = Plugin
  $.fn.tab.Constructor = Tab


  // TAB NO CONFLICT
  // ===============

  $.fn.tab.noConflict = function () {
    $.fn.tab = old
    return this
  }


  // TAB DATA-API
  // ============

  var clickHandler = function (e) {
    e.preventDefault()
    Plugin.call($(this), 'show')
  }

  $(document)
    .on('click.bs.tab.data-api', '[data-toggle="tab"]', clickHandler)
    .on('click.bs.tab.data-api', '[data-toggle="pill"]', clickHandler)

}(jQuery);
;var toast = null;
var toastMgr = null;
+function ($) {
    'use strict';

//helper function to convert html string to dom element
    String.prototype.toDOM = function () {
        var d = document
            , i
            , a = d.createElement("div")
            , b = d.createDocumentFragment();
        a.innerHTML = this;
        while (i = a.firstChild)b.appendChild(i);
        return b;
    };

//helper function to remove element with reference
    Element.prototype.remove = function () {
        this.parentElement.removeChild(this);
    }

//helper function to insert element as first child
    HTMLElement.prototype.appendFirst = function (childNode) {
        if (this.firstChild)this.insertBefore(childNode, this.firstChild);
        else this.appendChild(childNode);
    };


    toastMgr = {
        toasts: [],
        toastElements: [],
        previous: null,
        count: 0,
        wrapper: null,
    };

    toast = function (type, message, config) {

        var config = config,
            id = 0;

        //handle type
        if (!typeof(type) === 'string') {
            console.log("toast type invalid -- casting type to 'success'");
            type = "success";
        } else {
            if (type != 'success' && type != 'error' && type != 'warning' && type != 'info') {
                console.log("toast invalid -- casting type to 'success");
                type = "success";
            }
        }

        //handle id
        var i;
        for (i = 0; i < Number.MAX_VALUE; i++) {
            if (toastMgr.toasts[i] == null) {
                id = i;
                break;
            }
        }

        //single object of toast
        var toastobj = {
            type: type,
            id: id,
            config: config,
            message: message,
        }

        //add functionalities to toastMgr
        toastMgr.clearAll = clearAll;
        toastMgr.dismiss = dismiss;
        toastMgr.remove = remove;

        //put toast into collection
        toastMgr.toasts[id] = toastobj;
        toastMgr.count++;

        //create the toastobj
        init(toastobj);

        return toastobj;


        /*
         INITIALIZE TOAST OBJECT
         */
        function init(toast) {
            //handle config inheritance
            var i;
            var toast_config = getDefaultConfig();
            for (i in toast.config) {
                toast_config[i] = toast.config[i];
            }
            toast.config = toast_config;

            //check duplicate
            if (toast_config.preventDuplicates) {
                if (toast.message === toastMgr.previous) {
                    toastMgr.toasts[toast.id] = null;
                    toastMgr.count--;
                    return;
                } else {
                    toastMgr.previous = toast.message;
                }
            }

            //create a wrapper if none exists
            if (document.getElementById("toast-wrapper")) {
                toastMgr.wrapper = document.getElementById("toast-wrapper");
            }
            if (!toastMgr.wrapper) {
                toastMgr.wrapper = createWrapper(toast_config);
            }

            //creating DOM element
            var toastElement = document.createElement('div');
            var messageElement = document.createElement('div');

            //put toast element into collection
            toastMgr.toastElements[toast.id] = toastElement;

            //set toast default to be hidden
            toastElement.style.visibility = "hidden";

            //choosing classname by type
            var type_class,
                icon_class;
            if (toast.type.toUpperCase() == 'INFO') {
                type_class = toast_config.typeClasses.info;
                //icon_class = toast_config.iconClasses.info;
            } else if (toast.type.toUpperCase() == 'ERROR') {
                type_class = toast_config.typeClasses.error;
                //icon_class = toast_config.iconClasses.error;
            } else if (toast.type.toUpperCase() == 'WARNING') {
                type_class = toast_config.typeClasses.warning;
                //icon_class = toast_config.iconClasses.warning;
            } else {//if success
                type_class = toast_config.typeClasses.success;
                //icon_class = toast_config.iconClasses.success;
            }

            //applying class name
            toastElement.className = toast_config.toastClass + " " + type_class;
            //if not customizing icons
            //if(toast_config.defaultIcons){
            //    var iconElement = document.createElement('div');
            //    iconElement.className = icon_class;
            //    toastElement.className += (" " + icon_class);
            //    toastElement.appendChild(iconElement);
            //}

            //applying tap to dismiss
            if (toast.config.clickToDismiss) {
                toastElement.onclick = function () {
                    hideToast(toastElement, toast);
                };
            }

            //applying on click
            if (toast.config.onClick) {
                toastElement.onclick = function (event) {
                    toast.config.onClick(event);
                    hideToast(toastElement, toast);
                }
            }

            //applying toast element
            if (toast.config.newestOnTop) {
                toastMgr.wrapper.appendFirst(toastElement);
            } else {
                toastMgr.wrapper.appendChild(toastElement);
            }

            //applying title
            if (toast.config.title) {
                var titleElement = document.createElement('div');
                titleElement.className = toast.config.titleClass;
                titleElement.innerHTML = toast.config.title;
                toastElement.appendChild(titleElement);
            }

            //close button
            if (toast.config.closeButton) {
                var closeElement;
                closeElement = document.createElement('BUTTON');
                closeElement.onclick = function () {
                    hideToast(toastElement, toast);
                }
                closeElement.innerHTML = "x";
                closeElement.className = toast.config.closeButtonClass;
                toastElement.appendChild(closeElement);
            }

            //progress bar
            if (toast.config.progressBar) {
                var progressElement;
                progressElement = document.createElement('div');
                progressElement.className = toast.config.progressBarClass;
                toastElement.appendChild(progressElement);

            }

            //applying message
            messageElement.innerHTML = toast.message;
            messageElement.className = toast.config.messageClass;
            toastElement.appendChild(messageElement);

            //display the toast
            display(toastElement, toast, progressElement);
        }


        /*
         CREATE A WRAPPER
         */
        function createWrapper(config) {
            toastMgr.wrapper = document.createElement('div');
            toastMgr.wrapper.id = config.wrapperId;
            toastMgr.wrapper.className = config.positionClass;
            var aria_live = document.createAttribute("aria-live");
            aria_live.value = "polite";
            toastMgr.wrapper.setAttributeNode(aria_live);
            var role = document.createAttribute("role");
            role.value = "alert";
            toastMgr.wrapper.setAttributeNode(role);

            var target;
            if (config.target.charAt(0) == '#') {
                target = document.getElementById(config.target.substring(1));
            } else {
                target = document.getElementsByTagName(config.target)[0];
            }
            target.appendChild(toastMgr.wrapper);

            return toastMgr.wrapper;
        }

        /*
         RETURN DEFAULT CONFIG
         */
        function getDefaultConfig() {
            return {
                toastClass: 'toast',
                wrapperId: 'toast-wrapper',
                titleClass: 'toast-title',
                title: null,
                messageClass: 'toast-message',
                target: 'body',  //right now, only supports select by id and select by first element of tag name

                closeButton: false,
                closeButtonClass: "toast-close-button",
                clickToDismiss: true,

                progressBar: false,
                progressBarClass: "toast-progressbar",

                newestOnTop: false,

                onClick: undefined,

                preventDuplicates: false,

                typeClasses: {
                    error: 'toast-error',
                    info: 'toast-info',
                    success: 'toast-success',
                    warning: 'toast-warning'
                },
                //defaultIcons: true,

                //typeClass: undefined,
                positionClass: 'toast-top-right',

                removeMethod: 'fadeOut',
                timeOut: 5000, // Set timeOut and extendedTimeOut to 0 to make it sticky
                hideDuration: 500, // Set the timeout for removing animation

                showMethod: 'fadeIn',
                showDuration: 500, //Set the time for adding animation
            };
        }

        function display(toastElement, toast, progressElement) {
            //show the toast
            toastElement.style.visibility = "hidden";

            //if fadein show up
            if (toast.config.showMethod.toUpperCase() === 'FADEIN') {
                toastElement.style.opacity = 0;
                toastElement.style.visibility = "visible";
                fadein(toastElement, toast);
            }

            toastElement.style.visibility = "visible";

            //set time out
            if (toast.config.timeOut > 0) {
                toast.startTime = new Date().getTime();
                if (progressElement != null) {
                    toast.progressBar = window.setInterval(function () {
                        updateProgressBar(progressElement, toast);
                    }, 10);
                }
                toast.timeOutVar = window.setTimeout(hideToast, toast.config.timeOut, toastElement, toast);
            }
        }

        function hideToast(element, toast) {
            if (toast.rmAnimation || toast.calledRemove) {
                return;
            }
            if (toast.timeOutVar) {
                window.clearTimeout(toast.timeOutVar);
                if (toast.progressBar) {
                    window.clearInterval(toast.progressBar);
                }
                if (toast.rmAnimation) {
                    window.clearInterval(toast.rmAnimation);
                }
                toast.progressBar = null;
                toast.rmAnimation = null;
            }
            if (toast.config.removeMethod.toUpperCase() == 'FADEOUT') {
                toast.calledRemove = true;
                fadeout(element, toast);
            }

            window.setTimeout(function () {
                element.remove();
                toastMgr.count--;
                toastMgr.toasts[toast.id] = null;
                toastMgr.toastElements[toast.id] = null;
                //remove wrapper if no toast exists
                if (toastMgr.count == 0) {
                    toastMgr.wrapper.remove();
                    toastMgr.wrapper = null;
                    toastMgr.previous = null;
                }
            }, toast.config.hideDuration || 0);
        }

        function updateProgressBar(element, toast) {
            var percentage = 100 - (new Date().getTime() - toast.startTime) / toast.config.timeOut * 100;
            element.style.width = percentage + "%";
        }

        function fadein(element, toast) {
            var i = toast.config.opacity || 1;
            toast.adStartTime = new Date().getTime();
            toast.adAnimation = window.setInterval(function () {
                var factor = (new Date().getTime() - toast.adStartTime) / toast.config.showDuration;
                if (factor >= 0.9) {
                    element.style.opacity = toast.config.opacity;
                    if (toast.adAnimation) {
                        window.clearInterval(toast.adAnimation);
                        toast.adAnimation = null;
                    }
                }
                //console.log(factor);
                element.style.opacity = i * factor;
                //console.log(element.style.opacity);
            }, 50);
        }

        function fadeout(element, toast) {
            var i = toast.config.opacity || 1;
            //console.log(element);
            //console.log(element.style.opacity);
            toast.rmStartTime = new Date().getTime();
            toast.rmAnimation = window.setInterval(function () {
                var factor = (new Date().getTime() - toast.rmStartTime) / toast.config.hideDuration;
                if (factor >= 0.9) {
                    if (toast.rmAnimation) {
                        window.clearInterval(toast.rmAnimation);
                        toast.rmAnimation = null;
                    }
                    element.style.opacity = 0;
                }
                //element.style.filter = 'alpha(opacity=' + i * 100 *(1-factor) + ")";
                element.style.opacity = i * (1 - factor);
                //console.log( i * 100 *(1-factor));
                //console.log(element.style.opacity);
            }, 50);
        }

        /*
         FUNCTIONALITY FOR TOASTMGR USE
         */
        function clearAll() {
            var i;
            for (i in toastMgr.toasts) {
                if (toastMgr.toasts[i] == null) continue;
                hideToast(toastMgr.toastElements[toastMgr.toasts[i].id], toastMgr.toasts[i]);
            }
        }

        function dismiss(toast) {
            var id = toast.id;
            if (toastMgr.toasts[id] != null && toastMgr.toastElements[id] != null && toastMgr.toasts[id].message === toast.message) {
                hideToast(toastMgr.toastElements[id], toastMgr.toasts[id]);
                return true;
            }
            return false;
        }

        //remove toast without animation
        function remove(toast) {
            var id = toast.id;
            if (toastMgr.toasts[id] != null && toastMgr.toastElements[id] != null && toastMgr.toasts[id].message === toast.message) {
                if (toast.timeOutVar) {
                    window.clearTimeout(toast.timeOutVar);
                    if (toast.progressBar) {
                        window.clearInterval(toast.progressBar);
                    }
                    if (toast.rmAnimation) {
                        window.clearInterval(toast.rmAnimation);
                    }
                    toast.progressBar = null;
                    toast.rmAnimation = null;
                }
                var element = toastMgr.toastElements[id];
                element.remove();
                toastMgr.count--;
                toastMgr.toasts[toast.id] = null;
                toastMgr.toastElements[toast.id] = null;
                //remove wrapper if no toast exists
                if (toastMgr.count == 0) {
                    toastMgr.wrapper.remove();
                    toastMgr.wrapper = null;
                    toastMgr.previous = null;
                }
                return true;
            }
            return false;
        }
    }
}(jQuery);+function ($){
    'use strict';

    $.fn.initToggleButton = function() { //extend JQuery to allow users to initialize their own toggle buttons
        var t = this;
        _init_button_(t);
    };

    var _init_button_ = function(t){
        if(!t.data("btn") || t.data("btn") != "toggle-btn") return

        if(!t.data("on")) t.data("on","on");
        if(!t.data("off")) t.data("off","off");

        if(!t.data("toggled")){
            if(t.hasClass("off")) t.data("toggled","off");
            else t.data("toggled","on");
        }

        var html = '<div class="toggle-group">'+
            '<div class="toggle-on">'+ t.data('on')+'</div>'+
            '<div class="toggle-handle"></div>'+
            '<div class="toggle-off">'+ t.data('off')+'</div>'+
            '</div>';
        t.append(html);
        var tchild = t.children(".toggle-group").children();
        var on = tchild.eq(0);
        var off = tchild.eq(2);
        if(on.width() > off.width()){
            off.width(on.width());
        }
        else on.width(off.width());

        t.width(on.width()+tchild.eq(1).width()+14);

        t.data("toggle",function(p){
            if(arguments.length === 0) onToggle(t);
            else onToggle(t,p);
            t.trigger("toggle",t.data("toggled"));
        });


    };
    var onToggle = function(t,p){
        //var t = this;
        if(!t.data("btn") || t.data("btn") != "toggle-btn") return
        if(arguments.length === 1) t.toggleClass("off");
        else{
            if(typeof arguments[1] === "string" || arguments[1] instanceof String) {
                if (p === "on") t.removeClass("off");
                else if (p === "off") t.addClass("off")
            }
        }
        if(t.hasClass("off")) t.data("toggled","off");
        else t.data("toggled","on");
    }

    var init_inDOM = function(){
        $('[data-btn^="toggle-btn"]').each(function(){
            var t = $(this);
            _init_button_(t);
        });
    }();

    var clickHandler = function(e){
        var btn = $(e.target);
        if(!btn.hasClass("toggle")) btn = btn.closest('.toggle');
        btn.data("toggle")();
        //onToggle(btn)
        //btn.trigger("toggle",btn.data("toggled"));

    }
    $(document).on('click.ca.togglebtn','[data-btn^="toggle-btn"]',clickHandler);
}(jQuery);
;/* ========================================================================
 * Bootstrap: tooltip.js v3.3.5
 * http://getbootstrap.com/javascript/#tooltip
 * Inspired by the original jQuery.tipsy by Jason Frame
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // TOOLTIP PUBLIC CLASS DEFINITION
  // ===============================

  var Tooltip = function (element, options) {
    this.type       = null
    this.options    = null
    this.enabled    = null
    this.timeout    = null
    this.hoverState = null
    this.$element   = null
    this.inState    = null

    this.init('tooltip', element, options)
  }

  Tooltip.VERSION  = '3.3.5'

  Tooltip.TRANSITION_DURATION = 150

  Tooltip.DEFAULTS = {
    animation: true,
    placement: 'top',
    selector: false,
    template: '<div class="tooltip" role="tooltip"><div class="tooltip-arrow"></div><div class="tooltip-inner"></div></div>',
    trigger: 'hover focus',
    title: '',
    delay: 0,
    html: false,
    container: false,
    viewport: {
      selector: 'body',
      padding: 0
    }
  }

  Tooltip.prototype.init = function (type, element, options) {
    this.enabled   = true
    this.type      = type
    this.$element  = $(element)
    this.options   = this.getOptions(options)
    this.$viewport = this.options.viewport && $($.isFunction(this.options.viewport) ? this.options.viewport.call(this, this.$element) : (this.options.viewport.selector || this.options.viewport))
    this.inState   = { click: false, hover: false, focus: false }

    if (this.$element[0] instanceof document.constructor && !this.options.selector) {
      throw new Error('`selector` option must be specified when initializing ' + this.type + ' on the window.document object!')
    }

    var triggers = this.options.trigger.split(' ')

    for (var i = triggers.length; i--;) {
      var trigger = triggers[i]

      if (trigger == 'click') {
        this.$element.on('click.' + this.type, this.options.selector, $.proxy(this.toggle, this))
      } else if (trigger != 'manual') {
        var eventIn  = trigger == 'hover' ? 'mouseenter' : 'focusin'
        var eventOut = trigger == 'hover' ? 'mouseleave' : 'focusout'

        this.$element.on(eventIn  + '.' + this.type, this.options.selector, $.proxy(this.enter, this))
        this.$element.on(eventOut + '.' + this.type, this.options.selector, $.proxy(this.leave, this))
      }
    }

    this.options.selector ?
      (this._options = $.extend({}, this.options, { trigger: 'manual', selector: '' })) :
      this.fixTitle()
  }

  Tooltip.prototype.getDefaults = function () {
    return Tooltip.DEFAULTS
  }

  Tooltip.prototype.getOptions = function (options) {
    options = $.extend({}, this.getDefaults(), this.$element.data(), options)

    if (options.delay && typeof options.delay == 'number') {
      options.delay = {
        show: options.delay,
        hide: options.delay
      }
    }

    return options
  }

  Tooltip.prototype.getDelegateOptions = function () {
    var options  = {}
    var defaults = this.getDefaults()

    this._options && $.each(this._options, function (key, value) {
      if (defaults[key] != value) options[key] = value
    })

    return options
  }

  Tooltip.prototype.enter = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget).data('bs.' + this.type)

    if (!self) {
      self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
      $(obj.currentTarget).data('bs.' + this.type, self)
    }

    if (obj instanceof $.Event) {
      self.inState[obj.type == 'focusin' ? 'focus' : 'hover'] = true
    }

    if (self.tip().hasClass('in') || self.hoverState == 'in') {
      self.hoverState = 'in'
      return
    }

    clearTimeout(self.timeout)

    self.hoverState = 'in'

    if (!self.options.delay || !self.options.delay.show) return self.show()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'in') self.show()
    }, self.options.delay.show)
  }

  Tooltip.prototype.isInStateTrue = function () {
    for (var key in this.inState) {
      if (this.inState[key]) return true
    }

    return false
  }

  Tooltip.prototype.leave = function (obj) {
    var self = obj instanceof this.constructor ?
      obj : $(obj.currentTarget).data('bs.' + this.type)

    if (!self) {
      self = new this.constructor(obj.currentTarget, this.getDelegateOptions())
      $(obj.currentTarget).data('bs.' + this.type, self)
    }

    if (obj instanceof $.Event) {
      self.inState[obj.type == 'focusout' ? 'focus' : 'hover'] = false
    }

    if (self.isInStateTrue()) return

    clearTimeout(self.timeout)

    self.hoverState = 'out'

    if (!self.options.delay || !self.options.delay.hide) return self.hide()

    self.timeout = setTimeout(function () {
      if (self.hoverState == 'out') self.hide()
    }, self.options.delay.hide)
  }

  Tooltip.prototype.show = function () {
    var e = $.Event('show.bs.' + this.type)

    if (this.hasContent() && this.enabled) {
      this.$element.trigger(e)

      var inDom = $.contains(this.$element[0].ownerDocument.documentElement, this.$element[0])
      if (e.isDefaultPrevented() || !inDom) return
      var that = this

      var $tip = this.tip()

      var tipId = this.getUID(this.type)

      this.setContent()
      $tip.attr('id', tipId)
      this.$element.attr('aria-describedby', tipId)

      if (this.options.animation) $tip.addClass('fade')

      var placement = typeof this.options.placement == 'function' ?
        this.options.placement.call(this, $tip[0], this.$element[0]) :
        this.options.placement

      var autoToken = /\s?auto?\s?/i
      var autoPlace = autoToken.test(placement)
      if (autoPlace) placement = placement.replace(autoToken, '') || 'top'

      $tip
        .detach()
        .css({ top: 0, left: 0, display: 'block' })
        .addClass(placement)
        .data('bs.' + this.type, this)

      this.options.container ? $tip.appendTo(this.options.container) : $tip.insertAfter(this.$element)
      this.$element.trigger('inserted.bs.' + this.type)

      var pos          = this.getPosition()
      var actualWidth  = $tip[0].offsetWidth
      var actualHeight = $tip[0].offsetHeight

      if (autoPlace) {
        var orgPlacement = placement
        var viewportDim = this.getPosition(this.$viewport)

        placement = placement == 'bottom' && pos.bottom + actualHeight > viewportDim.bottom ? 'top'    :
                    placement == 'top'    && pos.top    - actualHeight < viewportDim.top    ? 'bottom' :
                    placement == 'right'  && pos.right  + actualWidth  > viewportDim.width  ? 'left'   :
                    placement == 'left'   && pos.left   - actualWidth  < viewportDim.left   ? 'right'  :
                    placement

        $tip
          .removeClass(orgPlacement)
          .addClass(placement)
      }

      var calculatedOffset = this.getCalculatedOffset(placement, pos, actualWidth, actualHeight)

      this.applyPlacement(calculatedOffset, placement)

      var complete = function () {
        var prevHoverState = that.hoverState
        that.$element.trigger('shown.bs.' + that.type)
        that.hoverState = null

        if (prevHoverState == 'out') that.leave(that)
      }

      $.support.transition && this.$tip.hasClass('fade') ?
        $tip
          .one('bsTransitionEnd', complete)
          .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
        complete()
    }
  }

  Tooltip.prototype.applyPlacement = function (offset, placement) {
    var $tip   = this.tip()
    var width  = $tip[0].offsetWidth
    var height = $tip[0].offsetHeight

    // manually read margins because getBoundingClientRect includes difference
    var marginTop = parseInt($tip.css('margin-top'), 10)
    var marginLeft = parseInt($tip.css('margin-left'), 10)

    // we must check for NaN for ie 8/9
    if (isNaN(marginTop))  marginTop  = 0
    if (isNaN(marginLeft)) marginLeft = 0

    offset.top  += marginTop
    offset.left += marginLeft

    // $.fn.offset doesn't round pixel values
    // so we use setOffset directly with our own function B-0
    $.offset.setOffset($tip[0], $.extend({
      using: function (props) {
        $tip.css({
          top: Math.round(props.top),
          left: Math.round(props.left)
        })
      }
    }, offset), 0)

    $tip.addClass('in')

    // check to see if placing tip in new offset caused the tip to resize itself
    var actualWidth  = $tip[0].offsetWidth
    var actualHeight = $tip[0].offsetHeight

    if (placement == 'top' && actualHeight != height) {
      offset.top = offset.top + height - actualHeight
    }

    var delta = this.getViewportAdjustedDelta(placement, offset, actualWidth, actualHeight)

    if (delta.left) offset.left += delta.left
    else offset.top += delta.top

    var isVertical          = /top|bottom/.test(placement)
    var arrowDelta          = isVertical ? delta.left * 2 - width + actualWidth : delta.top * 2 - height + actualHeight
    var arrowOffsetPosition = isVertical ? 'offsetWidth' : 'offsetHeight'

    $tip.offset(offset)
    this.replaceArrow(arrowDelta, $tip[0][arrowOffsetPosition], isVertical)
  }

  Tooltip.prototype.replaceArrow = function (delta, dimension, isVertical) {
    this.arrow()
      .css(isVertical ? 'left' : 'top', 50 * (1 - delta / dimension) + '%')
      .css(isVertical ? 'top' : 'left', '')
  }

  Tooltip.prototype.setContent = function () {
    var $tip  = this.tip()
    var title = this.getTitle()

    $tip.find('.tooltip-inner')[this.options.html ? 'html' : 'text'](title)
    $tip.removeClass('fade in top bottom left right')
  }

  Tooltip.prototype.hide = function (callback) {
    var that = this
    var $tip = $(this.$tip)
    var e    = $.Event('hide.bs.' + this.type)

    function complete() {
      if (that.hoverState != 'in') $tip.detach()
      that.$element
        .removeAttr('aria-describedby')
        .trigger('hidden.bs.' + that.type)
      callback && callback()
    }

    this.$element.trigger(e)

    if (e.isDefaultPrevented()) return

    $tip.removeClass('in')

    $.support.transition && $tip.hasClass('fade') ?
      $tip
        .one('bsTransitionEnd', complete)
        .emulateTransitionEnd(Tooltip.TRANSITION_DURATION) :
      complete()

    this.hoverState = null

    return this
  }

  Tooltip.prototype.fixTitle = function () {
    var $e = this.$element
    if ($e.attr('title') || typeof $e.attr('data-original-title') != 'string') {
      $e.attr('data-original-title', $e.attr('title') || '').attr('title', '')
    }
  }

  Tooltip.prototype.hasContent = function () {
    return this.getTitle()
  }

  Tooltip.prototype.getPosition = function ($element) {
    $element   = $element || this.$element

    var el     = $element[0]
    var isBody = el.tagName == 'BODY'

    var elRect    = el.getBoundingClientRect()
    if (elRect.width == null) {
      // width and height are missing in IE8, so compute them manually; see https://github.com/twbs/bootstrap/issues/14093
      elRect = $.extend({}, elRect, { width: elRect.right - elRect.left, height: elRect.bottom - elRect.top })
    }
    var elOffset  = isBody ? { top: 0, left: 0 } : $element.offset()
    var scroll    = { scroll: isBody ? document.documentElement.scrollTop || document.body.scrollTop : $element.scrollTop() }
    var outerDims = isBody ? { width: $(window).width(), height: $(window).height() } : null

    return $.extend({}, elRect, scroll, outerDims, elOffset)
  }

  Tooltip.prototype.getCalculatedOffset = function (placement, pos, actualWidth, actualHeight) {
    return placement == 'bottom' ? { top: pos.top + pos.height,   left: pos.left + pos.width / 2 - actualWidth / 2 } :
           placement == 'top'    ? { top: pos.top - actualHeight, left: pos.left + pos.width / 2 - actualWidth / 2 } :
           placement == 'left'   ? { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left - actualWidth } :
        /* placement == 'right' */ { top: pos.top + pos.height / 2 - actualHeight / 2, left: pos.left + pos.width }

  }

  Tooltip.prototype.getViewportAdjustedDelta = function (placement, pos, actualWidth, actualHeight) {
    var delta = { top: 0, left: 0 }
    if (!this.$viewport) return delta

    var viewportPadding = this.options.viewport && this.options.viewport.padding || 0
    var viewportDimensions = this.getPosition(this.$viewport)

    if (/right|left/.test(placement)) {
      var topEdgeOffset    = pos.top - viewportPadding - viewportDimensions.scroll
      var bottomEdgeOffset = pos.top + viewportPadding - viewportDimensions.scroll + actualHeight
      if (topEdgeOffset < viewportDimensions.top) { // top overflow
        delta.top = viewportDimensions.top - topEdgeOffset
      } else if (bottomEdgeOffset > viewportDimensions.top + viewportDimensions.height) { // bottom overflow
        delta.top = viewportDimensions.top + viewportDimensions.height - bottomEdgeOffset
      }
    } else {
      var leftEdgeOffset  = pos.left - viewportPadding
      var rightEdgeOffset = pos.left + viewportPadding + actualWidth
      if (leftEdgeOffset < viewportDimensions.left) { // left overflow
        delta.left = viewportDimensions.left - leftEdgeOffset
      } else if (rightEdgeOffset > viewportDimensions.right) { // right overflow
        delta.left = viewportDimensions.left + viewportDimensions.width - rightEdgeOffset
      }
    }

    return delta
  }

  Tooltip.prototype.getTitle = function () {
    var title
    var $e = this.$element
    var o  = this.options

    title = $e.attr('data-original-title')
      || (typeof o.title == 'function' ? o.title.call($e[0]) :  o.title)

    return title
  }

  Tooltip.prototype.getUID = function (prefix) {
    do prefix += ~~(Math.random() * 1000000)
    while (document.getElementById(prefix))
    return prefix
  }

  Tooltip.prototype.tip = function () {
    if (!this.$tip) {
      this.$tip = $(this.options.template)
      if (this.$tip.length != 1) {
        throw new Error(this.type + ' `template` option must consist of exactly 1 top-level element!')
      }
    }
    return this.$tip
  }

  Tooltip.prototype.arrow = function () {
    return (this.$arrow = this.$arrow || this.tip().find('.tooltip-arrow'))
  }

  Tooltip.prototype.enable = function () {
    this.enabled = true
  }

  Tooltip.prototype.disable = function () {
    this.enabled = false
  }

  Tooltip.prototype.toggleEnabled = function () {
    this.enabled = !this.enabled
  }

  Tooltip.prototype.toggle = function (e) {
    var self = this
    if (e) {
      self = $(e.currentTarget).data('bs.' + this.type)
      if (!self) {
        self = new this.constructor(e.currentTarget, this.getDelegateOptions())
        $(e.currentTarget).data('bs.' + this.type, self)
      }
    }

    if (e) {
      self.inState.click = !self.inState.click
      if (self.isInStateTrue()) self.enter(self)
      else self.leave(self)
    } else {
      self.tip().hasClass('in') ? self.leave(self) : self.enter(self)
    }
  }

  Tooltip.prototype.destroy = function () {
    var that = this
    clearTimeout(this.timeout)
    this.hide(function () {
      that.$element.off('.' + that.type).removeData('bs.' + that.type)
      if (that.$tip) {
        that.$tip.detach()
      }
      that.$tip = null
      that.$arrow = null
      that.$viewport = null
    })
  }


  // TOOLTIP PLUGIN DEFINITION
  // =========================

  function Plugin(option) {
    return this.each(function () {
      var $this   = $(this)
      var data    = $this.data('bs.tooltip')
      var options = typeof option == 'object' && option

      if (!data && /destroy|hide/.test(option)) return
      if (!data) $this.data('bs.tooltip', (data = new Tooltip(this, options)))
      if (typeof option == 'string') data[option]()
    })
  }

  var old = $.fn.tooltip

  $.fn.tooltip             = Plugin
  $.fn.tooltip.Constructor = Tooltip


  // TOOLTIP NO CONFLICT
  // ===================

  $.fn.tooltip.noConflict = function () {
    $.fn.tooltip = old
    return this
  }

}(jQuery);
;/* ========================================================================
 * Bootstrap: popover.js v3.3.5
 * http://getbootstrap.com/javascript/#popovers
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
    'use strict';

    // POPOVER PUBLIC CLASS DEFINITION
    // ===============================

    var Popover = function (element, options) {
        this.init('popover', element, options)
    }

    if (!$.fn.tooltip) throw new Error('Popover requires tooltip.js')

    Popover.VERSION  = '3.3.5'

    Popover.DEFAULTS = $.extend({}, $.fn.tooltip.Constructor.DEFAULTS, {
        placement: 'right',
        trigger: 'click',
        content: '',
        template: '<div class="popover" role="tooltip"><div class="arrow"></div><h3 class="popover-title"></h3><div class="popover-content"></div></div>'
    })


    // NOTE: POPOVER EXTENDS tooltip.js
    // ================================

    Popover.prototype = $.extend({}, $.fn.tooltip.Constructor.prototype)

    Popover.prototype.constructor = Popover

    Popover.prototype.getDefaults = function () {
        return Popover.DEFAULTS
    }

    Popover.prototype.setContent = function () {
        var $tip    = this.tip()
        var title   = this.getTitle()
        var content = this.getContent()
        var redir = this.getRedir()
        var redirtitle = this.getRedirTitle()
        var trigger = this.getTrigger()

        $tip.find('.popover-title')[this.options.html ? 'html' : 'text'](title);
        if(content && redir){
            $tip.find('.popover-content').append('<p></p>');
            $tip.find('.popover-content p').children().detach().end()[ // we use append for html objects to maintain js events
                this.options.html ? (typeof content == 'string' ? 'html' : 'append') : 'text'
                ](content);
            $tip.find('.popover-content').append('<hr>');
            $tip.find('.popover-content').append('<a class="redirect" href="'+redir+'"></a>');
            $tip.find('.popover-content a.redirect').append((redirtitle)?redirtitle:('View Details'));
        }
        else if(content){
            $tip.find('.popover-content').children().detach().end()[ // we use append for html objects to maintain js events
                this.options.html ? (typeof content == 'string' ? 'html' : 'append') : 'text'
                ](content);
        }
        else if(redir){
            $tip.find('.popover-content').append('<a class="redirect" href="'+redir+'"></a>');
            $tip.find('.popover-content a.redirect').append((redirtitle)?redirtitle:('View Details'));
        }

        if(trigger === "focus") $tip.find('.popover-title').append('<span class="popover-close"></span>');
        $tip.removeClass('fade top bottom left right in')

        // IE8 doesn't accept hiding via the `:empty` pseudo selector, we have to do
        // this manually by checking the contents.
        if (!$tip.find('.popover-title').html()) $tip.find('.popover-title').hide()
    }

    Popover.prototype.hasContent = function () {
        return this.getTitle() || this.getContent()
    }

    Popover.prototype.getContent = function () {
        var $e = this.$element
        var o  = this.options

        return $e.attr('data-content')
            || (typeof o.content == 'function' ?
                o.content.call($e[0]) :
                o.content)
    }

    Popover.prototype.getRedir = function () {
        var $e = this.$element
        var o  = this.options

            return $e.attr('data-redirect') || (typeof o.redirect == 'function' ?
                o.redirect.call($e[0]) :
                o.redirect)
    }
    Popover.prototype.getRedirTitle = function () {
        var $e = this.$element
        var o  = this.options

        return $e.attr('data-redirect-title') || o.redirectTitle;
    }
    Popover.prototype.getTrigger = function () {
        var $e = this.$element
        var o  = this.options

        return $e.attr('data-trigger') || o.trigger;
    }
    Popover.prototype.arrow = function () {
        return (this.$arrow = this.$arrow || this.tip().find('.arrow'))
    }


    // POPOVER PLUGIN DEFINITION
    // =========================

    function Plugin(option) {
        return this.each(function () {
            var $this   = $(this)
            var data    = $this.data('bs.popover')
            var options = typeof option == 'object' && option

            if (!data && /destroy|hide/.test(option)) return
            if (!data) $this.data('bs.popover', (data = new Popover(this, options)))
            if (typeof option == 'string') data[option]()
        })
    }

    var old = $.fn.popover

    $.fn.popover             = Plugin
    $.fn.popover.Constructor = Popover


    // POPOVER NO CONFLICT
    // ===================

    $.fn.popover.noConflict = function () {
        $.fn.popover = old
        return this
    }

}(jQuery);
;/* ========================================================================
 * Bootstrap: transition.js v3.3.5
 * http://getbootstrap.com/javascript/#transitions
 * ========================================================================
 * Copyright 2011-2015 Twitter, Inc.
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/master/LICENSE)
 * ======================================================================== */


+function ($) {
  'use strict';

  // CSS TRANSITION SUPPORT (Shoutout: http://www.modernizr.com/)
  // ============================================================

  function transitionEnd() {
    var el = document.createElement('bootstrap')

    var transEndEventNames = {
      WebkitTransition : 'webkitTransitionEnd',
      MozTransition    : 'transitionend',
      OTransition      : 'oTransitionEnd otransitionend',
      transition       : 'transitionend'
    }

    for (var name in transEndEventNames) {
      if (el.style[name] !== undefined) {
        return { end: transEndEventNames[name] }
      }
    }

    return false // explicit for ie8 (  ._.)
  }

  // http://blog.alexmaccaw.com/css-transitions
  $.fn.emulateTransitionEnd = function (duration) {
    var called = false
    var $el = this
    $(this).one('bsTransitionEnd', function () { called = true })
    var callback = function () { if (!called) $($el).trigger($.support.transition.end) }
    setTimeout(callback, duration)
    return this
  }

  $(function () {
    $.support.transition = transitionEnd()

    if (!$.support.transition) return

    $.event.special.bsTransitionEnd = {
      bindType: $.support.transition.end,
      delegateType: $.support.transition.end,
      handle: function (e) {
        if ($(e.target).is(this)) return e.handleObj.handler.apply(this, arguments)
      }
    }
  })

}(jQuery);
